#include "knoerre.h"

static jmp_buf knoerre_procstate;

/**
 * $Id: common_timeout.c 691 2017-06-04 23:12:04Z fwbergm $
 */

/**
 * Jump back on SIGALRM
 *
 * This alarm handler is used for the knoerre common timeout.
 *
 * @param num The signal caught.
 */
void common_timeout_alarm_sighandler(int num) {
  longjmp(knoerre_procstate,1);
}

/**
 * This function manages the common timeout.
 *
 * This function sets up an alarm sighandler, starts
 * the alarm timer and callssetjmp() to set a "return"
 * point which will be jumped to by the alarm sighandler.
 */
void common_timeout(unsigned int ctimeout, int print_http_header) {
  register char *p;
  char buf[BUF_SIZE+1];

  signal(SIGALRM,common_timeout_alarm_sighandler);
  alarm(ctimeout);
  if (!setjmp(knoerre_procstate)) return;

  /* on timeout we just send an error message and exit */
  p = buf;
  if (print_http_header) p = stpcpy(p, str200);
  p = stpcpy(p, ERROR_COMMON_TIMEOUT_STRING);
  write1(buf, p-buf);
  _exit(0);
}
