#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_RSBACKUP

/*
config KNOERRE_CHECK_RSBACKUP
    bool "check_rsbackup"
    default n
    help
      This knoerre module may be used to check for last backups in a specific way.
      More information offers man-page knoerre(1). If unsure say "N".

config KNOERRE_CHECK_RSBACKUP_KEY
    string "key string for check_rsbackup"
    default "rsbackup "
    depends on KNOERRE_CHECK_RSBACKUP
    help
      The key string for check_rsbackup as described in man-page knoerre(1).

config KNOERRE_CHECK_RSBACKUP_TS_FILE
    string "file with timestamp for check_rsbackup"
    default "/var/log/backup.timestamp"
    depends on KNOERRE_CHECK_RSBACKUP
    help
      The timestamp file for check_rsbackup as described in man-page knoerre(1).

config KNOERRE_CHECK_RSBACKUP_START_HOUR
    int "backup start hour for check_rsbackup"
    default 22
    depends on KNOERRE_CHECK_RSBACKUP
    help
      Backup start hour for check_rsbackup as described in man-page knoerre(1).

config KNOERRE_CHECK_RSBACKUP_START_MINUTE
    int "backup start minute for check_rsbackup"
    default 10
    depends on KNOERRE_CHECK_RSBACKUP
    help
      Backup start minute for check_rsbackup as described in man-page knoerre(1).
*/

/**
 * Return minutes since last backup.
 *
 * This function is called by knoerre main().
 * The timestamp of the last backup in format YYYYMMDD is
 * taken from /var/log/backup.timestamp and the difference in minutes
 * to current time is returned.
 * $Id: do_check_backup.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 */
void do_check_backup(register char *buf, const int buflen, char *bufp) {
  register char *p;
  register long tmp;
  int bak_year,bak_mon,bak_mday;
  char readbuf[STRING_SIZE+1];
  time_t tp,tnow;
  struct tm t;
  register int fd;

  p=readbuf;
  fd=open(CONFIG_KNOERRE_CHECK_RSBACKUP_TS_FILE, O_RDONLY);
  if (fd == -1) {
    p = stpcpy(p, "can't open ");
    p = stpcpy(p, CONFIG_KNOERRE_CHECK_RSBACKUP_TS_FILE);
    p = stpcpy(p, "\n"ERROR_COMMON_VALUE_STRING);
    tmp = -1;
    goto exit;
  }
  tmp=read(fd, p, 32); close(fd);
  if (tmp<=0) {
    p = stpcpy(p, "can't read ");
    p = stpcpy(p, CONFIG_KNOERRE_CHECK_RSBACKUP_TS_FILE);
    p = stpcpy(p, "\n"ERROR_COMMON_VALUE_STRING);
    goto exit;
  }
  p[tmp]='\0';
  bak_year=(*p++-'0')*1000; bak_year+=(*p++-'0')*100;
  bak_year+=(*p++-'0')*10; bak_year+=*p++-'0';
  bak_mon=(*p++-'0')*10; bak_mon+=*p++-'0';
  bak_mday=(*p++-'0')*10; bak_mday+=*p-'0';
  if((bak_year<2007)||(bak_year>2050)) goto error;
  if((bak_mon<0)||(bak_mon>12)) goto error;
  if((bak_mday<0)||(bak_mday>31)) goto error;

  tp = time(NULL);
  if (tp<10) goto error;
  tnow=tp;
  if(localtime_r(&tp,&t)==NULL) goto error;
  t.tm_year=bak_year-1900;
  t.tm_mon=bak_mon-1; t.tm_mday=bak_mday;
  t.tm_hour=CONFIG_KNOERRE_CHECK_RSBACKUP_START_HOUR;
  t.tm_min=CONFIG_KNOERRE_CHECK_RSBACKUP_START_MINUTE;
  tp=mktime(&t);
  if(tp<0) goto error;
  tmp=(long)((tnow-tp)/3600);
  if(tmp<0) tmp+=23;
  if(tmp<-24) goto error;
  if(tmp<0) tmp=0;

  exit:
  p = stpcpy(bufp, readbuf);
  if (tmp>=0) {
    p = sputlongs(p,tmp);
  }
  write1(buf, p-buf);
  _exit(0);

  error:
  p = readbuf+strlen(readbuf);
  p = stpcpy(p, "can't parse ");
  p = stpcpy(p, CONFIG_KNOERRE_CHECK_RSBACKUP_TS_FILE);
  p = stpcpy(p, "\n"ERROR_COMMON_VALUE_STRING);
  tmp = -1;
  goto exit;
}

#endif
