#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_CACHEDVALUE

/*
config KNOERRE_CHECK_CACHEDVALUE
    bool "check_cachedvalue"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module returns a check-result which was written to disk.
      If you want i.e. measure CPU user% you can call
      'echo OK nologin >result;test -x /etc/nologin && echo CRITICAL nologin >result'
      and get it later with 'knoerre cachedvalue/10/absolute/path/to/result;echo $?'.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_CACHEDVALUE_KEY
    string "key string for check_cachedvalue"
    default "cachedvalue/"
    depends on KNOERRE_CHECK_CACHEDVALUE
    help
      The key string for check_cachedvalue as described in man-page knoerre(1).
*/

/**
 * Check a cached value in a file and return result.
 *
 * This function is called by knoerre main().
 * The first arg is the file's max age in minutes. If the file is too old
 * then CRITICAL will be returned.
 * Check the last line of the file: If it does not start with "OK" or
 * "WARNING" then return CRITICAL.
 * If the start of the line is "valid" then return its value.
 * $Id: do_check_cachedvalue.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * void do_check_cachedvalue(register char *buf, const int buflen, char *bufp, char *arg1) {
 *
 * @return size of data to be sent (knoerred)
 */
int do_check_cachedvalue() {
  register char *p;
  int rf;
  uint64_t maxage;
  int fd, i;
  time_t tp;
  struct stat statbuf;

  p = strchr(knoerre_global_arg1, ' ');
  if (!p) p = strchr(knoerre_global_arg1, '\n');
  *p = 0;
  knoerre_global_buf_cursor = stpcpy(knoerre_global_buf_cursor, "CACHED ");
  p = str2uint64(knoerre_global_arg1, &maxage);
  if (!p) {
    knoerre_global_buf_cursor = stpcpy(knoerre_global_buf_cursor, "CRITICAL - argument error\n");
    goto error;
  }
  
  rf = lstat(p, &statbuf);
  if (rf) {
    knoerre_global_buf_cursor = stpcpy(knoerre_global_buf_cursor, "CRITICAL - lstat() error\n");
    goto error;
  }
  
  tp = time(NULL) - statbuf.st_mtime;
  if(tp < 0) tp = 0;
  tp = tp / 60;
  if (tp > maxage) {
    p = stpcpy(knoerre_global_buf_cursor, "CRITICAL - file age ");
    p = sputlongs(p, tp);
    knoerre_global_buf_cursor = stpcpy(p-1, " minutes\n");
    goto error;
  }

  if (!S_ISREG(statbuf.st_mode)) {
    knoerre_global_buf_cursor = stpcpy(knoerre_global_buf_cursor, "CRITICAL - not a regular file\n");
    goto error;
  }
  if (statbuf.st_size == 0) {
    knoerre_global_buf_cursor = stpcpy(knoerre_global_buf_cursor, "CRITICAL - empty file\n");
    goto error;
  }
  if (statbuf.st_size > CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS) {
    knoerre_global_buf_cursor = stpcpy(knoerre_global_buf_cursor, "CRITICAL - file too large\n");
    goto error;
  }

  fd = open(p, O_RDONLY);
  if (fd == -1) {
    knoerre_global_buf_cursor = stpcpy(knoerre_global_buf_cursor, "CRITICAL - open() error\n");
    goto error;
  }
  p = knoerre_global_buf_cursor;
  i = read(fd, p, CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS);
  close(fd);
  if (i < 1) {
    knoerre_global_buf_cursor = stpcpy(p, "CRITICAL - read() error\n");
    goto error;
  }
  p += i; --p;
  if (*p != '\n') *++p = '\n';
  *++p = 0;

#ifdef KNOERRED
  return p - knoerre_global_buf;
#endif
  i = 2;
  if (!strncmp(knoerre_global_buf_cursor, "OK ", 3)) i = 0;
  if (!strncmp(knoerre_global_buf_cursor, "WARNING ", 8)) i = 1;
  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(i);

  error:
#ifndef KNOERRED
  if (knoerre_global_buf_cursor != knoerre_global_buf)
    write1(knoerre_global_buf, knoerre_global_buf_cursor - knoerre_global_buf);
  check_error_exit(2);
#endif
  return knoerre_global_buf_cursor - knoerre_global_buf;

}

#endif
