#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_CAT

/*
config KNOERRE_CHECK_CAT
    bool "check_cat"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module shows the content of a file like "cat" does.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_CAT_KEY
    string "key string for check_cat"
    default "cat/"
    depends on KNOERRE_CHECK_CAT
    help
      The key string for check_cat as described in man-page knoerre(1).
*/

/**
 * This function reads the content of a file and writes it as http data out.
 *
 * This function is called by knoerre main().
 * It reads the content of a file and writes it to stdout.
 * To avoid blocking we use knoerre common timeout.
 * In the last before line the name and a timestamp will be printed.
 * Example:
 *   
 *   # knoerre cat/proc/sys/net/ipv4/netfilter/ip_conntrack_count
 *   /proc/sys/net/ipv4/netfilter/ip_conntrack_count 20110320
 *   250
 *   # 
 *
 * $Id: do_check_cat.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @return size of data to be sent (knoerred)
 */
int do_check_cat() {
  register char *p;
  register ssize_t bytes;
  register int i;
  char fbuf[2*PATHNAME_SIZE];
  char check_cat_filename[PATHNAME_SIZE+1];
  struct stat statbuf;
  time_t tp;
  struct tm t;

  p = strchr(knoerre_global_arg1, ' ');
  if (!p) p = strchr(knoerre_global_arg1, '\n');
  *p = 0;
  strcpy(check_cat_filename, knoerre_global_arg1);

  /* copy filename in output buffer */
  p = stpcpy(fbuf, knoerre_global_arg1);

  i = lstat(check_cat_filename, &statbuf);
  if (!i) {
    tp = statbuf.st_mtime;
    localtime_r(&tp, &t);
    i = (t.tm_year + 1900) * 10000;
    i += ((t.tm_mon + 1) * 100);
    i += t.tm_mday;
    /* copy SPACE and timestamp in output buffer */
    *p = ' '; ++p;
    p = sputlongs(p, i);
  } else {
    *p = '\n'; ++p;
  }

  /* as a default write the error string in output buffer */
  strcpy(p, ERROR_COMMON_VALUE_STRING);

  i = open(check_cat_filename, O_RDONLY);
  if (i == -1) goto exit;
  bytes = read(i, p, PATHNAME_SIZE);
  close(i);
  if (bytes >= 0) p += bytes;
  *p = '\0';

  exit:
  p = stpcpy(knoerre_global_buf_cursor, fbuf);
#ifdef KNOERRED
  return p - knoerre_global_buf;
#endif
  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(0);
}

#endif
