#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_CMDLINE
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_CMDLINE
    bool "check_cmdline"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module counts processes by cmdline match.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_CMDLINE_KEY
    string "key string for check_cmdline"
    default "cmdline/"
    depends on KNOERRE_CHECK_CMDLINE
    help
      The key string for check_cmdline as described in man-page knoerre(1).
*/

/**
 * Like do_check_process() but for /proc/.../cmdline
 *
 * This function is called by knoerre main().
 * It steps through "/proc/" and counts the number of running
 * instances of a process by searching /proc/.../cmdline and
 * write it as http data out and exists.
 * If no instance is running just return "9999" by calling check_error_exit().
 * $Id: do_check_cmdline.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 * @param arg1 The name of the process to be counted.
 */
int do_check_cmdline(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  register char *r;
  register ssize_t cl;
  register uint32_t i;
  register DIR *dptr;
  register struct dirent *eptr;
  register int pstat;
  char pathname[2*PATHNAME_SIZE];
  char statbuf[PATHNAME_SIZE+1];
  char key[PATHNAME_SIZE+1];

  p = strchr(arg1, ' ');
  if (!p) p = strchr(arg1, '\n');
  *p = 0;
  p = stpcpy(key, arg1);
  *p = '\n'; ++p;
  *p = 0;
  
  /**
   * step through /proc/ files
   */
  dptr = opendir("/proc");
  if (!dptr)
#ifdef KNOERRED
    return 0;
#else
    _exit(1);
#endif
  for (i = 0;(eptr = readdir(dptr));) {
    if ((*(eptr->d_name) < '0') || (*(eptr->d_name) > '9')) continue;
    p = pathname;
    p = stpcpy(p, "/proc/");
    p = stpcpy(p, eptr->d_name);
    p = stpcpy(p, "/cmdline");
    p = pathname;
    pstat = open(p, O_RDONLY);
    if (pstat >= 0) {
      p = statbuf;
      cl = read(pstat,p,PATHNAME_SIZE);
      if (cl > 2) {
        r = p + cl;
        for (; p < r;++p) if (!*p) *p = ' ';
        *r = 0;
        if (strstr(statbuf, arg1)) ++i;
      }
      close(pstat);
    }
  };
  closedir(dptr);
#ifndef KNOERRED
  if (bufp == buf) {
    /* cmdline mode */
    if (i > 0) --i;
  }
#endif

  /**
   * write the data into buffer and print it out
   */
  p = stpcpy(bufp, key);
  if (i < 1) {
    p = stpcpy(p, ERROR_COMMON_VALUE_STRING);
  } else {
    p = sputlongs(p,i);
  }
#ifdef KNOERRED
  return p - buf;
#endif
  write1(buf, p-buf);
  _exit(0);

}
#endif
