#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_CMP

/*
config KNOERRE_CHECK_CMP
    bool "check_cmp"
    default y
    help
      This knoerre module compares a string to the content of a file.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_CMP_KEY
    string "key string for check_cmp"
    default "cmp/"
    depends on KNOERRE_CHECK_CMP
    help
      The key string for check_cmp as described in man-page knoerre(1).
*/

/**
 * This function compares a string to the content of a file.
 *
 * This function is called by knoerre main().
 * It compares a string (chars up to first '/' in arg1) to
 * the contents of a file (name from first '/' in arg1 upwards).
 * If the strings doesn't match the content ('\n' are ignored)
 * then '1' and otherwise '0' is printed on last line.
 * To avoid blocking we use knoerre common timeout.
 * $Id: do_check_cmp.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param arg1 Search string and filename.
 */
int do_check_cmp(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  register ssize_t bytes;
  register int i;
  char fbuf[2*PATHNAME_SIZE];
  char check_cmp_filename[PATHNAME_SIZE+1];
  int ret = 1;

  p = strchr(arg1, ' ');
  if (!p) p = strchr(arg1, '\n');
  *p = 0;
  p = strchr(arg1, '/');
  if (!p) goto err_exit;
  
  strcpy(check_cmp_filename, p);
  *p = 0;

  p = stpcpy(fbuf, arg1);
  *p++ = '\n';

  strcpy(p, ERROR_COMMON_VALUE_STRING);

  i = open(check_cmp_filename, O_RDONLY);
  if (i == -1) goto exit;
  *p = 0;
  bytes = read(i, p, PATHNAME_SIZE);
  close(i);
  if (bytes >= 0) p[bytes] = 0;
  for (i = 0;;) {
    if (!p[i]) break;
    if (p[i] == '\n') {
      p[i] = 0;
      break;
    }
    ++i;
  }
  if (!strcmp(arg1, p)) ret = 0;

  exit:
  p = stpcpy(bufp, fbuf);
  *p++ = '\n';
  p = sputlongs(p, ret);
#ifdef KNOERRED
  return p - buf;
#endif
  write1(buf, p-buf);
  _exit(0);

  err_exit:
  p = stpcpy(bufp, ERROR_COMMON_VALUE_STRING);
#ifdef KNOERRED
  return p - buf;
#endif
  write1(buf, p-buf);
  _exit(0);
}

#endif

