#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_SOCKETS
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_SOCKETS
    bool "check_sockets"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module counts tcp/udp sockets per port.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).
      If unsure say "Y".

config KNOERRE_CHECK_SOCKETS_KEY
    string "key string for check_sockets"
    default "sockets/"
    depends on KNOERRE_CHECK_SOCKETS
    help
      The key string for check_sockets as described in man-page knoerre(1).

config KNOERRE_CHECK_SOCKETS_MAX_ADDRESSES
    int "max count of IP addresses recognized"
    default 500
    depends on KNOERRE_CHECK_SOCKETS
    help
      This is the maximum of IP addresses to be expected. Default is 500.
*/

static struct countsockets_data {
  char procfilename[15];
  int remote, local;
  char *exclude_addr;
  char *port;
  int port_all;
  int count_timewait;
  char a_addresses[CONFIG_KNOERRE_CHECK_SOCKETS_MAX_ADDRESSES+1][67];
  uint16_t a_addresscount[CONFIG_KNOERRE_CHECK_SOCKETS_MAX_ADDRESSES+1];
  uint32_t sc;
} cs_data;

static void read_stats_from_procfs() {
  register char *p;
  char *remote_addr;
  char *remote_port;
  char *local_port;
  ssize_t br;
  int c;
  char filebuf[2*BUF_SIZE+1];
  int fd;

  fd = open(cs_data.procfilename, O_RDONLY);
  if (fd < 0) return;

  read_again:
  p = filebuf;
  br = read(fd, p, 2*BUF_SIZE);
  if (br < 1) goto leave_stats;
  p[br] = 0;
  p = strchr(p, '\n'); if(!p) goto leave_stats;

  next_port:
  p = strchr(p, ':'); if(!p) goto read_again;
  p += 2; /* local addr */
  if (cs_data.exclude_addr && !cs_data.remote) {
    if (!strncmp(p, cs_data.exclude_addr, 8)) goto end_of_line;
  }
  p = strchr(p, ':'); if(!p) goto read_again;
  remote_addr = remote_port = 0;

  /* local port */
  ++p;
  local_port = p;
  if (cs_data.remote) {
    p += 5; /* remote addr */
    remote_addr = p;
    if (cs_data.exclude_addr) {
      if (!strncmp(p, cs_data.exclude_addr, 8)) goto end_of_line;
    }
    p = strchr(p, ':'); if(!p) goto read_again;
    ++p; /* remote port */
    remote_port = p;
  }

  if(cs_data.port_all || !strncmp(p, cs_data.port, 4) || (cs_data.local && !strncmp(local_port, cs_data.port, 4))) {

    if (!cs_data.remote) {
      p += 5; /* remote addr */
      remote_addr = p;
    }
    p = strchr(p, ' '); if(!p) goto read_again;

    /* TCP_TIME_WAIT won't be counted if count_timewait is not set */
    if (cs_data.count_timewait || (strncmp(p, " 06 ", 4))) {

      ++cs_data.sc;

      p = remote_addr;
      p = strchr(p, ':'); if(!p) goto read_again;
      *p = 0; ++p;

      /* search for address in our list and increment counter for this address */
      c = 0;
      for(;;) {
        if (c >= CONFIG_KNOERRE_CHECK_SOCKETS_MAX_ADDRESSES) break;
        if (cs_data.a_addresses[c][0] == 0) {
          strncpy(cs_data.a_addresses[c], remote_addr, 67);
          cs_data.a_addresscount[c] = 1;
          cs_data.a_addresses[++c][0] = 0; break;
        }
        if (!strncmp(cs_data.a_addresses[c], remote_addr, 67)) {
          cs_data.a_addresscount[c] += 1;
          break;
        }
        c++;
      }
    }
  }

  end_of_line:
  p = strchr(p, '\n'); if(!p) goto read_again; /* maybe one matched line will be missed */
  goto next_port;

  leave_stats:
  close(fd);
  return;
}

/**
 * This function counts the number of sockets open on a port.
 *
 * This function is called by knoerre main().
 * Count matching sockets in /proc/net/...
 * Parse arg1 for tcp/udp/... and local/remote and port number in hex.
 * For instance a arg1 "tcp/local/0050" will count all apache sockets.
 * $Id: do_check_countsockets.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 * @param arg1 The file/directory which lines to count
 */
void do_check_countsockets(char *buf, const int buflen, char *bufp, char *arg1) {

  register char *p;
  char *proto_set;
  char *netfile;
  char *max1_address;
  uint16_t max1_count;
  char *max2_address;
  uint16_t max2_count;
  char *max3_address;
  uint16_t max3_count;
  int c;

/*
 * example line from /proc/net/tcp:
 *    9: 3200101C:56B8 3C82141C:0019 06 00000000:00000000 03:0000155C 00000000     0        0 0 2 d6904880
 */
  cs_data.sc = cs_data.remote = cs_data.local = 0;
  cs_data.exclude_addr = 0;
  cs_data.count_timewait = 0;
  cs_data.port_all = 0;
  cs_data.a_addresses[0][0] = 0;
  proto_set = 0;
  p = arg1;
  c = strlen(p) - 1;
  if (p[c] == ' ') p[c] = 0;

  strcpy(cs_data.procfilename, "/proc/net/");
  netfile = cs_data.procfilename + 10;
  if (!strncmp(p, "tcp", 3)) {
    p += 3;
    strcpy(netfile, "tcp");
    goto tcpudp;
  }
  if (!strncmp(p, "udp", 3)) {
    p += 3;
    strcpy(netfile, "udp");
    goto tcpudp;
  }
  if (strncmp(p, "set-", 4)) {
    goto err_exit;
  } else {
    p += 4;
    proto_set = p;
    c = 0;
    for (;;) {
      ++p;
      if (!*p) break;
      if (*p == '/') { ++p; break; }
      ++c;
      if (c == 4) break;
    }
    goto read_addrtype;
  }
  tcpudp:
  if (*p == '6') {
    ++p; if (*p != '/') goto err_exit; ++p;
    netfile[3] = '6'; netfile[4] = 0;
  } else {
    if (*p != '/') goto err_exit; ++p;
  }

  read_addrtype:
  if (*p == 'w') {
    cs_data.count_timewait = 1;
    ++p;
  }

  if(!strncmp(p, "remote", 6)) {
    cs_data.remote = 1;
    p += 7;
  } else {
    if(!strncmp(p, "local", 5)) {
      cs_data.local = 1;
      p += 6;
    } else {
      cs_data.remote = cs_data.local = 1;
      if (*p) {
        if(strncmp(p, "all", 3)) goto err_exit;
        p += 3;
      }
    }
  }
  if (*p == '/') ++p;
  if (*p) {
    cs_data.port = p;
    p += 4;
    if (*p == '/') cs_data.exclude_addr = ++p;
  } else {
    cs_data.port_all = 1;
  }


  if (proto_set) {
    c = 0;
    for (;;) {

      *netfile = 0;
      switch(*proto_set) {
        case 't': strcpy(netfile, "tcp"); break;
        case 'T': strcpy(netfile, "tcp6"); break;
        case 'u': strcpy(netfile, "udp"); break;
        case 'U': strcpy(netfile, "udp6"); break;
        case '/': goto got_all_stats; break;
        case 0: goto got_all_stats; break;
        default: goto err_exit; break;
      }

      read_stats_from_procfs();

      ++proto_set;
      ++c;
      if (c == 4) break;
    }

  } else {

      read_stats_from_procfs();

  }


  got_all_stats:
  /* walk through our list and get the 3 top addresses with most sockets running */
  max1_address = max2_address = max3_address = 0;
  max1_count = max2_count = max3_count = 0;
  c = 0;
  for(;;) {
    if (c >= CONFIG_KNOERRE_CHECK_SOCKETS_MAX_ADDRESSES) break;
    if (cs_data.a_addresses[c][0] == 0) { break; }
    if (cs_data.a_addresscount[c] > max1_count) {
      max3_address = max2_address; max3_count = max2_count;
      max2_address = max1_address; max2_count = max1_count;
      max1_address = cs_data.a_addresses[c]; max1_count = cs_data.a_addresscount[c];
    } else {
      if (cs_data.a_addresscount[c] > max2_count) {
        max3_address = max2_address; max3_count = max2_count;
        max2_address = cs_data.a_addresses[c]; max2_count = cs_data.a_addresscount[c];
      } else {
        if (cs_data.a_addresscount[c] > max3_count) {
          max3_address = cs_data.a_addresses[c]; max3_count = cs_data.a_addresscount[c];
        }
      }
    }
    c++;
  }

  /* top user(s) */
  p = bufp;
  if (max1_count > 0) {
    p = procnetipaddr_to_humanreadable((const char *) max1_address, p);
    *p = '='; ++p; p = sputlongs(p, max1_count);
    if (max2_count > 0) {
      --p; *p = ','; ++p;
      p = procnetipaddr_to_humanreadable((const char *) max2_address, p);
      *p = '='; ++p; p = sputlongs(p, max2_count);
      if (max3_count > 0) {
        --p; *p = ','; ++p;
        p = procnetipaddr_to_humanreadable((const char *) max3_address, p);
        *p = '='; ++p; p = sputlongs(p, max3_count);
      }
    }
  }

  /* total count */
  p = sputlongs(p, cs_data.sc);
  write1(buf, p - buf);
  _exit(0);

  err_exit:
  check_error_exit(0);
}

#endif
