#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_CPU
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_CPU
    bool "check_cpu"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module measures cpu times user%, system% and idle%.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).
      If unsure say "Y".
*/

/**
 * This function gets one specific CPU percentage value and writes it as http data out.
 *
 * This function is called by knoerre main().
 * It reads cpu values from /proc/stat,
 * calculates the requested type in percentage and
 * writes it as http-data out and exists.
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 * @param reqtype The requested cpu stat type like i.e. user, nice.
 * @return size of data to be sent, 0 on error
 */
int do_check_cpu(register char *buf, const int buflen, char *bufp, int reqtype, char *arg1) {
  register char *p;
#ifndef KNOERRED
  /* knoerre stuff: */

  register uint64_t total1,total2;
  register int i;
  register ssize_t s;
  uint64_t colval;
  uint64_t user, nice, system, idle, wait, irq, softirq;
  char cpustat[BUF_SIZE+1];
  uint64_t cpucurrsleepinterval;

  p = arg1;

  if (*++p == '/') {
    p = str2uint64(++p, &cpucurrsleepinterval);
    if (!p) cpucurrsleepinterval = CURRSLEEPINTERVAL;
  } else {
    cpucurrsleepinterval = CURRSLEEPINTERVAL;
  }
  i = open("/proc/stat", O_RDONLY);
  if (i < 0) main_error_redirect_exit(5);
  p = cpustat;
  s = read(i, p, BUF_SIZE);
  close(i);
  if (s < 2) main_error_redirect_exit(6);
  p[s] = '\0';

  if ((*p++ != 'c') || (*p++ != 'p') || (*p++ != 'u')) main_error_redirect_exit(7);
  total1 = total2 = user = nice = system = idle = wait = irq = softirq = 0;
  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) main_error_redirect_exit(8);
  p = str2uint64(p, &colval); if (!p) main_error_redirect_exit(8);
  user = colval; total1 += colval;
  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) main_error_redirect_exit(8);
  p = str2uint64(p, &colval); if (!p) main_error_redirect_exit(8);
  nice = colval; total1 += colval;
  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) main_error_redirect_exit(8);
  p = str2uint64(p, &colval); if (!p) main_error_redirect_exit(8);
  system = colval; total1 += colval;
  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) main_error_redirect_exit(8);
  p = str2uint64(p, &colval); if (!p) main_error_redirect_exit(8);
  idle = colval; total1 += colval;
  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) main_error_redirect_exit(8);

  if (*p == '\n') goto read1done; /* kernel 2.4 */

  p = str2uint64(p, &colval); if (!p) main_error_redirect_exit(8);
  wait = colval; total1 += colval;
/*
  irq and softirq (and also steal and guest) ignored
*/

  read1done:

  if (reqtype >= CPUUSERCURR) {

    reqtype -= CPUUSERCURR;

    sleep(cpucurrsleepinterval);

    i = open("/proc/stat", O_RDONLY);
    if (i < 0) main_error_redirect_exit(5);
    p = cpustat;
    s = read(i, p, BUF_SIZE);
    close(i);
    if (s < 2) main_error_redirect_exit(6);
    p[s] = '\0';
  
    p += 4;
    for(;;) { if (*p != ' ') break; p++; }
    p = str2uint64(p, &colval);
    user = colval - user; total2 += colval;
    for(;;) { if (*p != ' ') break; p++; }
    p = str2uint64(p, &colval);
    nice = colval - nice; total2 += colval;
    for(;;) { if (*p != ' ') break; p++; }
    p = str2uint64(p, &colval);
    system = colval - system; total2 += colval;
    for(;;) { if (*p != ' ') break; p++; }
    p = str2uint64(p, &colval);
    idle = colval - idle; total2 += colval;
    for(;;) { if (*p != ' ') break; p++; }
  
    if (*p == '\n') goto read2done; /* kernel 2.4 */
  
    p = str2uint64(p, &colval);
    wait = colval - wait; total2 += colval;
  /*
    for(;;) { if (*p != ' ') break; p++; }
    if (!*p) main_error_redirect_exit(8);
    p = str2uint64(p, &colval); if (!p) main_error_redirect_exit(8);
    total2 += colval;
    for(;;) { if (*p != ' ') break; p++; }
    if (!*p) main_error_redirect_exit(8);
    p = str2uint64(p, &colval); if (!p) main_error_redirect_exit(8);
    total2 += colval;
  irq and softirq (and also steal and guest) ignored
  */
  
    read2done:

    total1 = total2 - total1;

  }
  
  p = stpcpy(bufp, "total ticks: ");
  p = sputlongs(p, total1);

  if (total1 == 0) total1 = 1;
  user = 100 * user / total1;
  nice = 100 * nice / total1;
  system = 100 * system / total1;
  idle = 100 * idle / total1;
  wait = 100 * wait / total1;

  p = stpcpy(p, "user%: "); p = sputlongs(p, user);
  p = stpcpy(p, "nice%: "); p = sputlongs(p, nice);
  p = stpcpy(p, "system%: "); p = sputlongs(p, system);
  p = stpcpy(p, "idle%: "); p = sputlongs(p, idle);
  p = stpcpy(p, "user%: "); p = sputlongs(p, user); p--;
  p = stpcpy(p, " wait%: "); p = sputlongs(p, wait);

  switch(reqtype) {
    case CPUNICETOTAL: p = sputlongs(p, nice); break;
    case CPUSYSTEMTOTAL: p = sputlongs(p, system); break;
    case CPUIDLETOTAL: p = sputlongs(p, idle); break;
    case CPUWAITTOTAL: p = sputlongs(p, wait); break;
    case CPUIDLEINVERSETOTAL: if (idle > 100) idle = 100; p = sputlongs(p, 100-idle); break;
    default: p = sputlongs(p, user); break;
  }

  write1(buf, p-buf);
  _exit(0);

#else
  /* knoerred stuff: */

  {
    int tmpcursor = current_values_cursor;
    p = bufp;
    switch(reqtype) {
      case CPUUSERCURR: p = sputlongs(p, cpu_user_percent[tmpcursor]); break;
      case CPUSYSTEMCURR: p = sputlongs(p, cpu_system_percent[tmpcursor]); break;
      case CPUIDLECURR: p = sputlongs(p, cpu_idle_percent[tmpcursor]); break;
      case CPUWAITCURR: p = sputlongs(p, cpu_wait_percent[tmpcursor]); break;
      case CPUIDLEINVERSECURR: p = sputlongs(p, 100-cpu_idle_percent[tmpcursor]); break;
      default: break;
    }
  }
  return p - buf;

#endif

}
#endif
