#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_CTXTSWITCH
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_CTXTSWITCH
    bool "check_ctxtswitch"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module counts context switches.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_CTXTSWITCH_KEY
    string "key string for check_ctxtswitch"
    default "ctxtswitch"
    depends on KNOERRE_CHECK_CTXTSWITCH
    help
      The key string for check_ctxtswitch as described in man-page knoerre(1).
*/

/**
 * This function counts context switches per second.
 *
 * This function is called by knoerre main().
 * It reads ctxt values from /proc/stat,
 * waits some time, reads it again
 * and writes the average value
 * as http-data out and exists.
 * $Id: do_check_ctxtswitch.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 * @param arg1 The file/directory to analyze.
 */
void do_check_ctxtswitch(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  register uint64_t count;
  register int i;
  register ssize_t s;
  uint64_t colval;
  char ctxtstat[BUF_SIZE+1];
  uint64_t ctxtcurrsleepinterval;

  p = arg1;
  if (*p == '/') {
    p = str2uint64(++p, &ctxtcurrsleepinterval);
    if (!p) ctxtcurrsleepinterval = CURRSLEEPINTERVAL;
  } else {
    ctxtcurrsleepinterval = CURRSLEEPINTERVAL;
  }

  i = open("/proc/stat", O_RDONLY);
  if (i < 0) main_error_redirect_exit(5);
  p = ctxtstat;
  s = read(i, p, BUF_SIZE);
  close(i);
  if (s < 2) main_error_redirect_exit(6);
  p[s] = '\0';

  if ((*p++ != 'c') || (*p++ != 'p') || (*p++ != 'u')) main_error_redirect_exit(7);
  count = 0;
  i = str_index2(p, "\nctxt ");
  if (i == -1) main_error_redirect_exit(8);
  p += i;
  p = str2uint64(p, &colval); if (!p) main_error_redirect_exit(9);
  count = colval;

  sleep(ctxtcurrsleepinterval);

  i = open("/proc/stat", O_RDONLY);
  if (i < 0) main_error_redirect_exit(5);
  p = ctxtstat;
  s = read(i, p, BUF_SIZE);
  close(i);
  if (s < 2) main_error_redirect_exit(6);
  p[s] = '\0';

  i = str_index2(p, "\nctxt ");
  if (i == -1) main_error_redirect_exit(8);
  p += i;
  p = str2uint64(p, &colval); if (!p) main_error_redirect_exit(9);

  if (colval >= count) {
    count = colval - count;
  } else {
    /* overflow; is it 32 Bit? we can't be sure: set to 0 */
    count = 0;
  }

  count /= ctxtcurrsleepinterval;

  p = stpcpy(bufp, "seconds: ");
  p = sputlongs(p, ctxtcurrsleepinterval);

  p = sputlongs(p, count);

  write1(buf, p-buf);
  _exit(0);
}
#endif
