#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_DIRLEVELS

/*
config KNOERRE_CHECK_DIRLEVELS
    bool "check_dirlevels"
    default y
    help
      This knoerre module counts the recursive depth of a directory and its subdirectories.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_DIRLEVELS_KEY
    string "key string for check_dirlevels"
    default "dirlevels/"
    depends on KNOERRE_CHECK_DIRLEVELS
    help
      The key string for check_dirlevels as described in man-page knoerre(1).
*/

/*
 * do_check_dirlevels.c
 * $Id: do_check_dirlevels.c 691 2017-06-04 23:12:04Z fwbergm $
 */

/**
 * Step recursively into a directory and count recursion-level.
 *
 * This is a helper function of do_check_dirlevels.
 */
static int walkdirlevels(
    const char *name,
    uint32_t *levels_maxdir,
    char *pathname_maxdir,
    uint16_t pathname_size,
    int32_t curdepth,
    const char *subdir,
    char *errfile,
    dev_t *topmostdev) {

  register DIR *dptr;
  register struct dirent *eptr;
  struct stat statbuf;
  register char *nptr;
  register char *p;
  register int dotdirs;
  register int32_t hlinks;
  register int ret;
  int r2;
  char pathname[PATHNAME_SIZE*2];

  dptr = opendir(name);
  if (dptr == NULL) return -1;
  ret = 0;
  dotdirs = 0;
  hlinks = 2;
  if (lstat(name, &statbuf) == 0) {
    /* simple cross-fs check */
    if (!*topmostdev) {
      *topmostdev = statbuf.st_dev;
    } else {
      if (*topmostdev != statbuf.st_dev) return 0;
    }
    hlinks = statbuf.st_nlink;
    if (hlinks >= 2) hlinks -= 2; /* leaf optimization */
  }
  for(;;) {
    eptr = readdir(dptr); if (eptr == NULL) break;
    nptr = eptr->d_name;
    if ((dotdirs<2)&&(*nptr == '.')) {
      if (nptr[1]=='\0') { dotdirs+=1;continue; }
      if (nptr[1]=='.') {
        if (nptr[2]=='\0') { dotdirs+=1;continue; }
      }
    }
    if (hlinks>0) { /* leaf optimization */
      p = stpcpy(pathname, name);
      *p++ = '/';
      p = stpcpy(p, nptr);
      if (subdir!=NULL) {
         if(*subdir!=0) {
           *p++ = '/';
           p = stpcpy(p, subdir);
         }
      }
      if (lstat(pathname,&statbuf)==0){
        if ((statbuf.st_mode&S_IFMT)==S_IFDIR) {
          hlinks--;
          r2 = walkdirlevels(pathname,levels_maxdir,pathname_maxdir,pathname_size,curdepth+1,NULL,errfile,topmostdev);
          if (r2 != -1) ret = r2;
        } /* statbuf */
      } else {
        ret = errno;
        strncpy(errfile, pathname, PATHNAME_SIZE);
      } /* lstat */
    } /* hlinks */
  }
  closedir(dptr);
  if (curdepth>*levels_maxdir) {
    *levels_maxdir = curdepth;
    strncpy(pathname_maxdir,name,PATHNAME_SIZE);
  }
  return ret;
}


/**
 * This function counts recursion-levels in a dir and prints the max level.
 *
 * This function is called by knoerre main().
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param arg1 The directory to analyze.
 */
void do_check_dirlevels(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  register char *subdir;
  register int ret;
  register char *basedir;
  uint32_t entries_maxdir;
  char pathname_maxdir[PATHNAME_SIZE+1];
  char errfile[(2*PATHNAME_SIZE)+1];
  dev_t topmostdev;

  basedir = arg1;
  p = strchr(basedir, ' ');
  *p = 0;
  entries_maxdir=0;
  *errfile = 0;
  *pathname_maxdir = 0;
  topmostdev = 0;

  raise_stacklimit();

  /* wildcards? */
  subdir = strstr(basedir,"/./");
  if (!subdir) subdir = strstr(basedir,"/@/");
  /* strip trailing slash */
  ret = strlen(basedir);
  if(ret > 1) {
    p = basedir + ret - 1;
    if(*p == '/') *p = 0;
  }
  if (subdir) {
    *subdir = 0;
    subdir += 3;
    ret = walkdirlevels(basedir,&entries_maxdir,pathname_maxdir,PATHNAME_SIZE,0,subdir,errfile,&topmostdev);
  } else {
    ret = walkdirlevels(basedir,&entries_maxdir,pathname_maxdir,PATHNAME_SIZE,0,NULL,errfile,&topmostdev);
    if (ret!=0) goto error;
  }
  p = stpcpy(bufp, pathname_maxdir);
  *p++ = '\n';
  p = sputlongs(p, entries_maxdir);
  write1(buf, p-buf);
  _exit(ret);

  error:
  check_error_exit(ret);
#ifdef NULLDEF
  check_error_exit_ns(buf, buflen, 1, (uint32_t)ret, strerror(ret), errfile);
#endif
}

#endif
