#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_DISKINODES

/*
config KNOERRE_CHECK_DISKINODES
    bool "check_diskinodes"
    default y
    help
      This knoerre module shows inode usage in % like "df -i" does.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_DISKINODES_KEY
    string "key string for check_diskinodes"
    default "diskinodes/"
    depends on KNOERRE_CHECK_DISKINODES
    help
      The key string for check_diskinodes as described in man-page knoerre(1).
*/

/**
 * This function gets disk usage stats and writes it as http data out.
 *
 * This function is called by knoerre main().
 * It gets disk usage stats for a given mount point or file,
 * writes it as http data out and exists.
 * $Id: do_check_diskinodes.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 */
int do_check_diskinodes() {
  register char *p;
  register uint64_t a;
  register uint64_t s;
  struct statfs stat_buf;

  p = strchr(knoerre_global_arg1, ' ');
  if (!p) p = strchr(knoerre_global_arg1, '\n');
  *p = 0;

  if (statfs(knoerre_global_arg1, &stat_buf) != 0) goto error;

  p = stpcpy(knoerre_global_buf_cursor, knoerre_global_arg1);
  *p++ = ' ';
  a = stat_buf.f_ffree;
  p = sputlongs(p, a);
  p = stpcpy(p-1, " inodes free");
  *p++ = '\n';

  /* percentage */
  s = stat_buf.f_files - a; /* used inodes */
  a = stat_buf.f_files;
  if (a == 0) {
    /* reiserfs, btrfs, fat */
    s = 0;
  } else {
    if ((a + 50) < a) {
      /* extremely rare case... need cpu's carry bit ;-) */
      a /= 100;
    } else {
      a = (a + 50) / 100;
    }
    if (a != 0) s /= a;
  }

  p = sputlongs(p, s);
#ifdef KNOERRED
  return p - knoerre_global_buf;
#endif
  write1(knoerre_global_buf, p-knoerre_global_buf);
  _exit(0);

  error:
#ifdef KNOERRED
  return 0;
#endif
  check_error_exit(0);
  return 0;

}
#endif
