#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_DISKUSAGE

/*
config KNOERRE_CHECK_DISKUSAGE
    bool "check_diskusage"
    default y
    help
      This knoerre module shows diskusage in % as df does.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_DISKUSAGE_KEY
    string "key string for check_diskusage"
    default "diskusage/"
    depends on KNOERRE_CHECK_DISKUSAGE
    help
      The key string for check_diskusage as described in man-page knoerre(1).
*/

/**
 * This function gets disk usage stats and writes it as http data out.
 *
 * This function is called by knoerre/knoerred main().
 * It gets disk usage stats for a given mount point or file,
 * writes it as http data out and exists.
 * If we are called by knoerred then output is limited to two lines.
 * $Id: do_check_diskusage.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @return size of data to be sent, 0 on error
 */
int do_check_diskusage() {
  register char *p;
  FS_STAT fsstat;

  p = strchr(knoerre_global_arg1, ' ');
  if (!p) p = strchr(knoerre_global_arg1, '\n');
  *p = 0;
  /* our output (est. max. 120) plus key must fit in buffer */
  if ((p - knoerre_global_arg1 + 120) > knoerre_global_bufsize) goto error;

  fsstat.path = knoerre_global_arg1;
  if (get_fs_stats(&fsstat) == -1) {
    p = stpcpy(knoerre_global_buf_cursor, ERROR_OPEN_STRING);
    goto exit_func;
  }

  /* last before line */
  p = stpcpy(knoerre_global_buf_cursor, fsstat.path);
  *p = ' '; ++p;
  p = sputuint32(p, fsstat.percent_used);
  *p = '%'; ++p;
  *p = ','; ++p;
  *p = ' '; ++p;
  p = sputuint32(p, fsstat.all);
  *p = '/'; ++p;
  p = sputuint32(p, fsstat.used);
  *p = '/'; ++p;
  p = sputuint32(p, fsstat.free);
  p = stpcpy(p, " GB all/used/free");
  *p = '\n'; ++p;

  /* last line */
  p = sputuint32(p, fsstat.percent_used);
  *p = '\n'; ++p;

  exit_func:
#ifdef KNOERRED
  return p - knoerre_global_buf;
#else
  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(0);
#endif

  error:
#ifndef KNOERRED
  check_error_exit(0);
#endif
  return 0;
}
#endif
