#ifdef PLUGIN
#ifndef KNOERRED
#define KNOERRED
#endif
#endif
#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_DISKUSAGELOCAL
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_DISKUSAGELOCAL
    bool "check_diskusagelocal"
    default y
    help
      This knoerre module shows diskusage in % as df does.
      It recognizes local mounts and returns the most filled fs.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_DISKUSAGELOCAL_KEY
    string "key string for check_diskusagelocal"
    default "diskusagelocal"
    depends on KNOERRE_CHECK_DISKUSAGELOCAL
    help
      The key string for check_diskusage as described in man-page knoerre(1).
*/

/**
 * This function gets disk usage stats and writes it as http data out.
 *
 * This function is called by knoerre/knoerred main().
 * It gets disk usage stats for local mounts,
 * writes it as http data out and exists.
 * If we are called by knoerred then output is limited to two lines.
 * $Id: do_check_diskusagelocal.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @return size of data to be sent, 0 on error
 */
int do_check_diskusagelocal() {
  register char *p;
  FS_STAT top1stat, top2stat;

  (void) get_all_local_fs_stats(&top1stat, &top2stat);

  p = stpcpy(knoerre_global_buf_cursor, top1stat.path);
  *p = ' '; ++p;
  p = sputuint32(p, top1stat.percent_used);
  *p = '%'; ++p;
  *p = ','; ++p;
  *p = ' '; ++p;
  p = sputuint32(p, top1stat.all);
  *p = '/'; ++p;
  p = sputuint32(p, top1stat.used);
  *p = '/'; ++p;
  p = sputuint32(p, top1stat.free);
  p = stpcpy(p, " GB");
#ifndef KNOERRED
  p = stpcpy(p, " all/used/free");
  if (!top2stat.path) goto lastline;
  *p = ';'; ++p;
  *p = ' '; ++p;
  p = stpcpy(p, top2stat.path);
  *p = ' '; ++p;
  p = sputuint32(p, top2stat.percent_used);
  *p = '%'; ++p;
  *p = ','; ++p;
  *p = ' '; ++p;
  p = sputuint32(p, top2stat.all);
  *p = '/'; ++p;
  p = sputuint32(p, top2stat.used);
  *p = '/'; ++p;
  p = sputuint32(p, top2stat.free);
  p = stpcpy(p, " GB all/used/free");
  lastline:
#endif
  *p = '\n'; ++p;
  p = sputuint32(p, top1stat.percent_used);
  *p = '\n'; ++p;

  /* last line */
  /* p = sputlongs(p, percent); */
#ifdef KNOERRED
#ifdef PLUGIN
  ++p;
  *((uint32_t *)p) = (uint32_t)(top1stat.percent_used);
  --p;
#endif
  return p - knoerre_global_buf;
#else
  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(0);
#endif

}
#endif
