#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_DMESG
#include <sys/klog.h>

/*
config KNOERRE_CHECK_DMESG
    bool "check_dmesg"
    default y
    help
      This knoerre module searches the kernel message ring buffer for "bad" entries.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_DMESG_KEY
    string "key string for check_dmesg"
    default "dmesg "
    depends on KNOERRE_CHECK_DMESG
    help
      The key string for check_dmesg as described in man-page knoerre(1).
*/

extern const KLE kernel_bad_entries[];
#define HIGH_WEIGHT_THRESHOLD 9

/**
 * This function gets the "bad" entries from kernel message ring buffer and prints them.
 *
 * $Id: do_check_dmesg.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 */
int do_check_dmesg() {
  register char *p;
  register uint32_t matches,points;
  register int c;
  char badlines[LOGCHECK_MATCH_LINES_CACHE][CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS+1];
  char *rbuf;
  int ret;
  int klogsize;
  uint16_t high_weight_found;

  ret = 127;
  for(c=0;c<LOGCHECK_MATCH_LINES_CACHE;++c) *badlines[c] = '\0';
  matches = points = 0;

  klogsize = klogctl(10, 0, 0);
  ++klogsize;
  p = rbuf = malloc(klogsize);
  if (p == (void *) -1) goto error;
  klogsize = klogctl(3, p, klogsize);

  p = rbuf + klogsize;
  *p = '\0';

  /* get last line */
  for(;;) { p--; if (p<rbuf) break; if (*p>=32) break; *p = '\0'; }
  for(;;) { p--; if (p<rbuf) break; if (*p<32) break; }
  *p = 0;

  high_weight_found = 0;
  for(;;) { p--; if (p<rbuf) break; if (*p>=32) break; *p = '\0'; }
  c = 1;
  /* scan all lines in rbuf */
  for(;;) {
    --p;
    if (*p < 32) {

      if (c) {
        for(c=0; kernel_bad_entries[c].string; c++) {
          if(strstr(p+1, kernel_bad_entries[c].string)) {
            ++matches;
            points += kernel_bad_entries[c].weight;
            if (matches < LOGCHECK_MATCH_LINES_CACHE) {
              if ((matches < 3) || (kernel_bad_entries[c].weight > HIGH_WEIGHT_THRESHOLD)) {
                strncpy(badlines[matches], p+1, CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS);
              }
              if (!high_weight_found && (kernel_bad_entries[c].weight > HIGH_WEIGHT_THRESHOLD)) {
                /* no "light-weight" badline should be last line if high-weight lines are found */
                for(;high_weight_found<matches;high_weight_found++)
                  badlines[high_weight_found][0] = 0;
              }
            }
          }
        }
      }
      c = 0;
      *p = '\0';

    } else {
      c = 1;
    }
    if (p < rbuf) break;
  }

#ifdef KNOERRED
  free(rbuf);
#endif

  p = stpcpy(knoerre_global_buf_cursor, "matchlines: ");
  p = sputlongs(p, matches);
  for(c=LOGCHECK_MATCH_LINES_CACHE;c>0;) {
    c--;
    if(*badlines[c]) {
      p = stpcpy(p, badlines[c]);
      *p++ = '\n';
    }
  }
  p = sputlongs(p, points);
#ifdef KNOERRED
  return p - knoerre_global_buf;
#endif
  write1(knoerre_global_buf, p-knoerre_global_buf);
  _exit(0);

  error:
  p = stpcpy(knoerre_global_buf_cursor, ERROR_COMMON_VALUE_STRING);
#ifdef KNOERRED
  return p - knoerre_global_buf;
#endif
  write1(knoerre_global_buf, p-knoerre_global_buf);
  _exit(ret);
}
#endif
