#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_FILEEXISTS

/*
config KNOERRE_CHECK_FILEEXISTS
    bool "check_fileexists"
    default y
    help
      This knoerre module checks if a file exists.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_FILEEXISTS_KEY
    string "key string for check_fileexists"
    default "fileexists/"
    depends on KNOERRE_CHECK_FILEEXISTS
    help
      The key string for check_fileexists as described in man-page knoerre(1).
*/

/**
 * This function checks if a file exists.
 *
 * This function is called by knoerre main().
 * It checks if a file exists. Without "options"
 * it returns 0 if a file of any type (i.e. dir, link)
 * exists. Logical inversion is also possible:
 * fileexists/F/var/run/bot.pid
 * returns 0 if a *file* /var/run/bot.pid does NOT exist
 * ( "F" = not "f").
 * Possible types:
 * f(ile), d(ir), c(har dev), b(lock dev), p(ipe), l(ink), s(ocket), a(ny type)
 * 
 * $Id: do_check_fileexists.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 * @param arg1 The file/directory to analyze.
 */
int do_check_fileexists(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  int rf;
  char chk_filetype, act_filetype;
  mode_t fmode;
  int logical_inverse = 0;
  int ret = 1;
  struct stat statbuf;

  p = arg1;
  ++p;
  p[strlen(p)-1] = '\0';

  chk_filetype = *p;
  if (chk_filetype < 'a') {
    chk_filetype += 32;
    logical_inverse = 1;
  }
  if ((chk_filetype!='f') &&
      (chk_filetype!='d') &&
      (chk_filetype!='c') &&
      (chk_filetype!='b') &&
      (chk_filetype!='p') &&
      (chk_filetype!='l') &&
      (chk_filetype!='s') &&
      (chk_filetype!='a')) goto error;

  ++p;
  rf = lstat(p, &statbuf);
  if (rf) { act_filetype = 'A'; goto got_type; }
  fmode = statbuf.st_mode;

  if (S_ISREG(fmode)) {
    act_filetype = 'f';
    goto got_type;
  }
  if (S_ISDIR(fmode)) {
    act_filetype = 'd';
    goto got_type;
  }
  if (S_ISCHR(fmode)) {
    act_filetype = 'c';
    goto got_type;
  }
  if (S_ISBLK(fmode)) {
    act_filetype = 'b';
    goto got_type;
  }
  if (S_ISFIFO(fmode)) {
    act_filetype = 'p';
    goto got_type;
  }
  if (S_ISLNK(fmode)) {
    act_filetype = 'l';
    goto got_type;
  }
  if (S_ISSOCK(fmode)) {
    act_filetype = 's';
    goto got_type;
  }
  act_filetype = 'a';

  got_type:

  ret = 1;
  if ((act_filetype == chk_filetype) || ((chk_filetype == 'a') && (act_filetype != 'A'))) ret = 0;
  if (logical_inverse) ret = (ret + 1) & 0x1;
    

  p = bufp;

  /* if it's a small regular file then print also its content */
  if (act_filetype == 'f') {
    if ((statbuf.st_size > 0) && (statbuf.st_size <= CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS)) {
      int fd, tmp;
      fd = open(arg1+2, O_RDONLY);
      if (fd == -1) goto out;
      tmp = read(fd, p, CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS); close(fd);
      if (tmp <= 0) goto out;
      p += tmp;
      if (*--p != '\n') *++p = '\n';
      ++p;
    }
  }

  out:
  p = sputlongs(p, ret);
#ifdef KNOERRED
  return p - buf;
#endif
  write1(buf, p-buf);
  _exit(0);

  error:
  p = stpcpy(bufp, "\n"ERROR_COMMON_VALUE_STRING);
#ifdef KNOERRED
  return p - buf;
#endif
  write1(buf, p-buf);
  _exit(0);
}

#endif
