#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_FILESIZES

/*
config KNOERRE_CHECK_FILESIZES
    bool "check_filesizes"
    default y
    help
      This knoerre module shows the (max) filesize(s) of (a) file(s).
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_FILESIZES_KEY
    string "key string for check_filesizes"
    default "filesizes/"
    depends on KNOERRE_CHECK_FILESIZES
    help
      The key string for check_filesizes as described in man-page knoerre(1).
*/

/**
 * This function gets the (maximum) file size of one or more files and writes it as http data out.
 *
 * This function is called by knoerre main().
 * It gets the file size of a single file or
 * the maximum filesize of a group of files with
 * the same basedir and subdirpaths.
 * Then it writes the result as http data out and exists.
 * $Id: do_check_filesizes.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param arg1 The file/directory to analyze.
 */
void do_check_filesizes(register char *buf, const int buflen, char *bufp, char *arg1, int mode) {
  register char *p;
  register char *nptr;
  off_t ss;
  char *bd;
  char *sd;
  uint32_t dotdirs;
  char *pattern;
  int16_t depth;
  int16_t maxage;
  struct stat stat_buf;
  /* int i; */
  char pathname[2*PATHNAME_SIZE];
  char pathname_max[2*PATHNAME_SIZE];
  char pathname_max2[2*PATHNAME_SIZE];
  struct dirent *eptr;
  DIR *dptr;
  off_t maxsize;
  off_t maxsize2;
  time_t ts;

  ss = maxsize = maxsize2 = depth = maxage = 0;
  sd = pattern = NULL;
  bd = arg1;
  bd[strlen(bd)-1]='\0';
  *pathname_max = '\0';
  ts = time(0);

  if (mode > 0) {
    pattern = bd+1;
    bd = strchr(pattern, '/');
    *bd = 0;
  }
  if (mode >= 0) {
    bd++;
    depth = *bd - '0';
    bd++;
  }
  if (mode == 4) {
    bd++;
    maxage = *bd - '0';
    bd++;
    if ((*bd >= '0') && (*bd <= '9')) {
      maxage = (maxage * 10) + *bd - '0';
      bd++;
    }
    ts -= maxage * 86400; /* days */
  }

  /* i = str_index(bd,"/./"); */
  p = strstr(bd,"/./"); /* "wildcard" like "*" in shell */
  if (!p) p = strstr(bd,"/@/"); /* "wildcard" like "*" in shell */
  if (p) {
    *p = 0;
    sd = p + 3;
    if (!*sd) { --sd; sd = 0; }
  }

  if (mode >= 0) {
    /* filesizesrecursive, filesizesbysuffix and filesizesbyprefix */
    walkdir_filesizes(bd,&maxsize,pathname_max,PATHNAME_SIZE,depth,0,sd,pattern,mode,ts);
    bd = pathname_max;
    goto out;
  }

  /* mode == -1, /filesizes/... for a single or a bunch of i.e. access_log file(s) */
  /* if (i > 0) { */
  if (p) {
    /*
     * step through all entries in bd
     * and search for matching files in subdir,
     * we don't need leaf optimization here,
     * just go through the whole bd
     */
    dptr = opendir(bd);
    if(dptr==NULL) goto error;
    dotdirs = 0;
    for(;;) {
      eptr = readdir(dptr); if(eptr==NULL) break;
      nptr = eptr->d_name;
      if((dotdirs<2)&&(*nptr=='.')) {
        if(nptr[1]=='\0') { dotdirs += 1;continue; }
        if(nptr[1]=='.') {
          if(nptr[2]=='\0') { dotdirs += 1;continue; }
        }
      }
      p = stpcpy(pathname, bd);
      *p++ = '/';
      p = stpcpy(p, nptr);
      if (sd) {
        *p++ = '/';
        p = stpcpy(p, sd);
      }
      p = pathname;
      if(lstat(p,&stat_buf)==0){
        ss = stat_buf.st_size/1024;
        if (ss > maxsize) {
          maxsize2 = maxsize;
          stpcpy(pathname_max2, pathname_max);
          maxsize = ss;
          stpcpy(pathname_max, nptr);
        } else {
          if (ss > maxsize2) {
            maxsize2 = ss;
            stpcpy(pathname_max2, nptr);
          }
        }
      }
    }
    closedir(dptr);
    bd = pathname_max;

  } else {

    /*
     * just a single file to check, no wildcard '.' given
     */
    *pathname_max = '\0';
    if(lstat(bd,&stat_buf)!=0) goto error;
    maxsize = stat_buf.st_size/1024; /* calculate KB */

  }

  out:
  p = bufp;
  if (maxsize > 0) {
    p = stpcpy(p, bd);
    *p++ = '=';
    p = sputlongs(p, maxsize);
    if (maxsize2 > 0) {
      --p;
      *p++ = ' ';
      p = stpcpy(p, pathname_max2);
      *p++ = '=';
      p = sputlongs(p, maxsize2);
    }
    --p;
    *p++ = '\n';
  }
  p = sputlongs(p, maxsize);
  write1(buf, p-buf);
  _exit(0);

  error:
  p = stpcpy(bufp, ERROR_COMMON_VALUE_STRING);
  write1(buf, p-buf);
  _exit(1);
}

#endif
