#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_FILETIMESTAMP

/*
config KNOERRE_CHECK_FILETIMESTAMP
    bool "check_filetimestamp"
    default y
    help
      This knoerre module returns the age of a file in minutes.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_FILETIMESTAMP_KEY
    string "key string for check_filetimestamp"
    default "filetimestamp/"
    depends on KNOERRE_CHECK_FILETIMESTAMP
    help
      The key string for check_filetimestamp as described in man-page knoerre(1).
*/

/**
 * Return minutes since timestamp of a file.
 *
 * This function is called by knoerre main().
 * It prints the number of minutes since the
 * last change, access, mod or the oldest
 * of these timestamps of a given file.
 * If the file is a small regular file print also its content.
 * $Id: do_check_filetimestamp.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 * @param arg1 The file/directory to analyze.
 */
int do_check_filetimestamp(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  char mode;
  int rf;
  time_t tp,tn;
  struct stat statbuf;

  p = arg1 + 1;
  mode = *p; ++p;
  p[strlen(p)-1] = '\0';
  
  rf = lstat(p, &statbuf);
  if ((rf) && (mode >= 'a')) goto error;
  if (mode <= 'Z') mode += 32;
  
  if (rf) {
    tp = 0;
  } else {
    switch (mode) {
      case 'm': tp = statbuf.st_mtime; break;
      case 'c': tp = statbuf.st_ctime; break;
      case 'a': tp = statbuf.st_atime; break;
      case 'o': tp = statbuf.st_atime;
                tn = statbuf.st_mtime;
                if (tn < tp) tp = tn;
                tn = statbuf.st_ctime;
                if (tn < tp) tp = tn;
                break;
      default: tp = statbuf.st_mtime; break;
    }
    tp = time(NULL) - tp;
  }

  if(tp < 0) tp = 0;
  tp = tp / 60;

  p = bufp;
  /* if it's a small regular file then print also its content */
  if (S_ISREG(statbuf.st_mode)) {
    if ((statbuf.st_size > 0) && (statbuf.st_size <= CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS)) {
      int fd, tmp;
      fd = open(arg1+2, O_RDONLY);
      if (fd == -1) goto out;
      tmp = read(fd, p, CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS); close(fd);
      if (tmp <= 0) goto out;
      p += tmp;
      if (*--p != '\n') *++p = '\n';
      ++p;
    }
  }

  out:
  p = sputlongs(p, tp);
#ifdef KNOERRED
  return p - buf;
#endif
  write1(buf, p-buf);
  _exit(0);

  error:
  p = stpcpy(bufp, "can't lstat()");
  p = stpcpy(p, "\n"ERROR_COMMON_VALUE_STRING);
#ifdef KNOERRED
  return p - buf;
#endif
  write1(buf, p-buf);
  _exit(0);
}
#endif
