#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_LOAD1
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_LOAD1
    bool "check_load1"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module returns the load per one minute average.
      It needs PROC_ACCESS. If unsure say "Y" because it is a basic parameter.

config KNOERRE_CHECK_LOAD1_KEY
    string "key string for check_load1"
    default "load1 "
    depends on KNOERRE_CHECK_LOAD1
    help
      The key string for check_load1.
*/


/**
 * This function gets load1 value and writes it as http data out.
 *
 * This function is called by knoerre main().
 * It gets the load1 value from /proc/loadavg,
 * writes it as http-data out and exists.
 *
 * @param ovz If true then divide load by cpu-cores (for OpenVZ).
 */
int do_check_load1(int ovz) {
  register char *p;
  register char *l;
  register int i;
  register ssize_t s;
  char loadavg[STRING_SIZE+1];

  i = open("/proc/loadavg", O_RDONLY);
  if (__unlikely(i < 0))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(5);
#endif
  p = loadavg;
  s = read(i, p, STRING_SIZE);
#ifdef KNOERRED
  close(i);
#endif
  if (__unlikely(s < 2))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(6);
#endif
  p[s] = 0;
  /* copy key for multi requests */
  l = stpcpy(knoerre_global_buf_cursor, "key load1\n");
  /* copy loadavg */
  l = stpcpy(l, p);

#ifndef OPENVZDEFAULT

  for (;;) {
    if (*p == ' ') break;
    *l = *p;
    ++l; ++p;
  }
  *l = '\n'; ++l;

  if (!ovz) {
#ifdef KNOERRED
    return l - knoerre_global_buf;
#endif
    write1(knoerre_global_buf, l-knoerre_global_buf);
    _exit(0);
  }
#endif

  {
    char rs[BUF_SIZE+1];
    int fd;
    uint64_t iload;
    char *lastline;

    fd = open("/proc/stat", O_RDONLY);
    if (fd < 0) goto out;
    i = read(fd, rs, BUF_SIZE);
#ifdef KNOERRED
    close(fd);
#endif
    if(i <= 1) goto out;
    rs[i] = 0;
    p = strchr(rs, '\n');
    lastline = p;
    while(!strncmp(p, "\ncpu", 4)) {
      ++p;
      lastline = p;
      p = strchr(p, '\n');
    }
    p = lastline + 3;
    i = *p - '0';
    ++p;
    if(*p != ' ') {
      i *= 10;
      i += (*p - '0');
    }
    ++i; /* cpu count starts at 0 */
    if(!str2uint64(loadavg, &iload)) goto out;
    iload /= i;
    --l; *l = ' '; ++l;
    l = sputlongs(l, i);
    l = sputlongs(l, iload);
  }

  out:
#ifdef KNOERRED
  return l - knoerre_global_buf;
#endif
  write1(knoerre_global_buf, l-knoerre_global_buf);
  _exit(0);
}
#endif
