#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_LOAD5
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_LOAD5
    bool "check_load5"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module returns the load per five minutes average.
      It needs PROC_ACCESS. If unsure say "Y".

config KNOERRE_CHECK_LOAD5_KEY
    string "key string for check_load5"
    default "load5 "
    depends on KNOERRE_CHECK_LOAD5
    help
      The key string for check_load5.
*/

/**
 * This function gets load5 value and writes it as http data out.
 *
 * This function is called by knoerre main().
 * It gets the load5 value from /proc/loadavg,
 * writes it as http-data out and exists.
 * $Id: do_check_load5.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 */
int do_check_load5(register char *buf, const int buflen, char *bufp) {
  register char *p;
  char *l;
  int i;
  ssize_t s;
  char loadavg[STRING_SIZE+1];

  i = open("/proc/loadavg", O_RDONLY);
  if (__unlikely(i < 0))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(5);
#endif
  p = loadavg;
  s = read(i, p, STRING_SIZE);
#ifdef KNOERRED
  close(i);
#endif
  if (__unlikely(s < 2))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(6);
#endif
  p[s] = '\0';
  /* copy key for multi requests */
  l = stpcpy(bufp, "key load5\n");
  /* copy loadavg */
  l = stpcpy(l, p);
  /* search for 2nd field */
  for(;;) { if(__unlikely(!*p)) break; if(*p==' ') break; ++p; }
  ++p;

#ifndef OPENVZDEFAULT

  for (;;) {
    if (*p == ' ') break;
    *l = *p;
    ++l; ++p;
  }
  *l = '\n'; ++l;

#else

  {
    char rs[BUF_SIZE+1];
    int fd;
    uint64_t iload;
    char *lastline;
    char *r;

    /* load5 */
    r = p;

    fd = open("/proc/stat", O_RDONLY);
    if (fd < 0) goto out;
    i = read(fd, rs, BUF_SIZE);
#ifdef KNOERRED
    close(fd);
#endif
    if(i <= 1) goto out;
    rs[i] = 0;
    p = strchr(rs, '\n');
    lastline = p;
    while(!strncmp(p, "\ncpu", 4)) {
      ++p;
      lastline = p;
      p = strchr(p, '\n');
    }
    p = lastline + 3;
    i = *p - '0';
    ++p;
    if(*p != ' ') {
      i *= 10;
      i += (*p - '0');
    }
    ++i; /* cpu count starts at 0 */
    if(!str2uint64(r, &iload)) goto out;
    iload /= i;
    --l; *l = ' '; ++l;
    l = sputlongs(l, i);
    l = sputlongs(l, iload);
  }

  out:
#endif

#ifdef KNOERRED
  return l - buf;
#endif
  write1(buf, l-buf);
  _exit(0);
  return 0;
}
#endif
