#ifdef KNOERREHD
#include "knoerrehd.h"
#else
#ifdef KNOERRED
#include "knoerred.h"
#else
#error loadmulti check is only available in knoerred
#endif
#endif
#ifdef CONFIG_KNOERRE_CHECK_LOADMULTI
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_LOADMULTI
    bool "check_loadmulti"
    default y
    depends on KNOERRED
    help
      This knoerre module is a multicheck module. It delivers
      load, hostname, time, cpu%, nettraf, swaps, diskusage, netlinksdown, proccount
      and even more with a single request. This check is only available in knoerred.
      It needs PROC_ACCESS. More information offers man-page knoerred(1).

config KNOERRE_CHECK_LOADMULTI_KEY
    string "key string for check_loadmulti"
    default "loadmulti/"
    depends on KNOERRE_CHECK_LOADMULTI
    help
      The key string for check_loadmulti as described in man-page knoerred(1).

config KNOERRED_MULTIRESULTS
    string "file to be included on loadmulti check"
    default "/nagios-cache/multiresults"
    depends on KNOERRE_CHECK_LOADMULTI
    help
      The file to be included on loadmulti check as described in man-page knoerred(1).
*/

/**
 * This function gets load values and more and write it as http data out.
 *
 * This function is called by knoerre main().
 * It gets the load values from /proc/loadavg,
 * write it as http-data out and exists.
 * $Id: do_check_loadmulti.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 */
int do_check_loadmulti(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  char *l;
  int i;
  ssize_t s;
  uint32_t mb;
  int64_t v;
  uint64_t a;
  uint64_t t;
  int swaps_count;
  char readbuf[STRING_SIZE+1];
  struct statfs *statp;
  time_t tp;
  char *hname_given;
#ifdef KNOERRED
  int tmpcursor;
#else
  struct statfs stat_buf;
  DIR *dptr;
  struct dirent *eptr;
  char devcarrier[STRING_SIZE+28];
  char *devptr;
  int fd_carrier;
  char carrierbuf[2];
#endif

  /* given time */
  p = str2uint64(arg1 + 1, (uint64_t *) &v);
  if (p) {
    tp = (time_t) v;
    tp = time(0) - tp;
    if (tp < 0) tp = 0 - tp;
  } else {
    tp = (time_t) - 1;
    p = arg1 + 1;
  }
  /* given hostname */
  if (*p == '/') {
    ++p;
    hname_given = p;
    for (;;) {
      if (*p == ' ') *p = 0;
      if (!*p) break;
      ++p;
    }
  } else {
    hname_given = 0;
  }

  l = bufp;

  /* copy key */
  l = stpcpy(l, " hostname\n");
  p = readbuf;
#ifdef KNOERRED
  tmpcursor = current_values_cursor;
  if (strcmp(hname_given, hostname[tmpcursor])) {
#else
  i = open("/proc/sys/kernel/hostname", O_RDONLY);
  if (__unlikely(i < 0))
    main_error_redirect_exit(5);
  s = read(i, p, STRING_SIZE);
  close(i);
  if (__unlikely(s < 2))
    main_error_redirect_exit(6);
  --s;
  p[s] = 0;

  if (strcmp(hname_given, p)) {
#endif
    *l = '1';
  } else {
    *l = '0';
  }
  ++l; *l = '\n'; ++l;

#ifdef KNOERRED

  /* include multiresults */
  p = multiresults[tmpcursor];
  if (*p == 0) goto multiresultsdone;
  l = stpcpy(l, p);

/*
#else
  p = l;
  i = open(CONFIG_KNOERRED_MULTIRESULTS, O_RDONLY);
  if (i < 0) goto multiresultsdone;
  s = read(i, p, UDP_BUFSIZE / 2);
  close(i);
  if (s < 2) goto multiresultsdone;
  p[s] = 0;
  l += s;
*/

  multiresultsdone:
#endif

#ifdef KNOERRED

  /* some values are collected by something like a "thread" */
  /* the cloned function is interval_measure() */

  /* copy key */
  l = stpcpy(l, " timediff\n");
  l = sputlongs(l, tp);

  /* create generic cpu string */
  p = readbuf;
  *p = 'u'; ++p;
  p = sputlongs(p, cpu_user_percent[tmpcursor]);
  --p;
  *p = 's'; ++p;
  p = sputlongs(p, cpu_system_percent[tmpcursor]);
  --p;
  *p = 'w'; ++p;
  p = sputlongs(p, cpu_wait_percent[tmpcursor]);

  /* copy key */
  l = stpcpy(l, " cpuu ");
  l = stpcpy(l, readbuf);
  l = sputlongs(l, cpu_user_percent[tmpcursor]);

  /* copy key */
  l = stpcpy(l, " cpus ");
  l = stpcpy(l, readbuf);
  l = sputlongs(l, cpu_system_percent[tmpcursor]);

  /* copy key */
  l = stpcpy(l, " cpuw ");
  l = stpcpy(l, readbuf);
  l = sputlongs(l, cpu_wait_percent[tmpcursor]);

  /* copy key */
  l = stpcpy(l, " cpuI ");
  l = stpcpy(l, readbuf);
  l = sputlongs(l, 100 - cpu_idle_percent[tmpcursor]);

  /* copy key */
  l = stpcpy(l, " ctxt\n");
  l = sputlongs(l, ctxt_switches_persecond[tmpcursor]);

  /* copy key */
  l = stpcpy(l, " nettraf0\n");
  l = sputlongs(l, nettraf0_kb_persecond[tmpcursor]);

  /* copy key */
  l = stpcpy(l, " nettraf1\n");
  l = sputlongs(l, nettraf1_kb_persecond[tmpcursor]);

#endif


  /* we read some values from /proc/... */


#ifdef KNOERRED
  p = swaps[tmpcursor];
  if (__unlikely(*p == 0))
    return 0;
  s = 1;
#else
  i = open("/proc/swaps", O_RDONLY);
  if (__unlikely(i < 0))
    main_error_redirect_exit(5);
  p = readbuf;
  s = read(i, p, STRING_SIZE);
  close(i);
  if (__unlikely(s < 2))
    main_error_redirect_exit(6);
  p[s] = 0;
#endif

  /* copy key */
  l = stpcpy(l, " swaps");
  if (s == 0) {
    no_swaps:
    l = stpcpy(l, " no swaps\n0\n");
    goto swaps_done;
  }
  p = strchr(p, '\n');
  if (__unlikely(!p))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(6);
#endif
  ++p;
  if (!*p) goto no_swaps;

  mb = swaps_count = 0;
  for(;;) {
    p = str_field(p, 2, '\t', 1);
    if (__unlikely(!p))
#ifdef KNOERRED
      return 0;
#else
      main_error_redirect_exit(9);
#endif
    if (__unlikely(!str2uint64(p,(uint64_t *) &v)))
#ifdef KNOERRED
      return 0;
#else
      main_error_redirect_exit(10);
#endif
    mb = mb + ((uint32_t) (v/1024)); ++swaps_count;
    p = strchr(p, '\n');
    if (__unlikely(!p))
#ifdef KNOERRED
      return 0;
#else
      main_error_redirect_exit(11);
#endif
    ++p;
    if (!*p) break;
  }
/*
  l = stpcpy(l, "swaps:");
  l = sputlongs(l, swaps_count);
*/
  *l = '\n'; ++l;
  l = sputlongs(l, mb);


  swaps_done:

#ifdef CONFIG_KNOERRE_CHECK_DISKUSAGELOCAL
  if (fsstat_top[tmpcursor].all != 0) {
    l = stpcpy(l, " diskusagelocal ");
    l = stpcpy(l, fsstat_top[tmpcursor].path);
    *l = '\n'; ++l;
    l = sputuint32(l, fsstat_top[tmpcursor].percent_used);
    *l = '\n'; ++l;
  }
#endif


#ifdef KNOERRED
  statp = &statfs_root[tmpcursor];
  if (statfs_root_err[tmpcursor] != 0) goto disk_stuff_done;
#else
  statp = &stat_buf;
  if (statfs("/", statp) != 0) { l = stpcpy(l, "9999\n"); goto out; }
#endif

#ifdef WANT_OLD_DISKUSAGE__NULLDEF
  /* copy key */
  l = stpcpy(l, " diskusage/ ");
  a = statp->f_bavail; /* avail for non-root */
  b = statp->f_bfree - a; /* free for root - avail = reserved */
  b = statp->f_blocks - b; /* all for non-root */
  /* calc GB free */
  t = a / 262144; /* avail in 4 mega blocks */
  t *= statp->f_bsize;
  t /= 4096; /* avail in GB */
  l = sputlongs(l, t);
  l = stpcpy(l-1, "GB free");
  *l++ = '\n';

  /* percentage */
  t = b - a; /* used */
  if (t < 0) {
    t = 0;
  } else {
    if ((b + 50) < b) {
      /* extremely rare case... need cpu's carry bit ;-) */
      b /= 100;
    } else {
      b = (b + 50) / 100;
    }
    if (b != 0) t /= b; else t = 100;
  }
  l = sputlongs(l, t);
#endif


  /* copy key */
  l = stpcpy(l, " diskinodes/ ");
  a = statp->f_ffree;
  l = sputlongs(l, a);
  l = stpcpy(l-1, " free");
  *l = '\n'; ++l;

  /* percentage */
  t = statp->f_files - a; /* used inodes */
  a = statp->f_files;
  if (a == 0) {
    /* reiserfs, btrfs, fat */
    t = 0;
  } else {
    if ((a + 50) < a) {
      /* extremely rare case... need cpu's carry bit ;-) */
      a /= 100;
    } else {
      a = (a + 50) / 100;
    }
    if (a != 0) t /= a;
  }
  l = sputlongs(l, t);


  disk_stuff_done:
  /* copy key */
  l = stpcpy(l, " netlinksdown ");

#ifdef KNOERRED
  i = netlinksdown[tmpcursor];
#else
  devptr = stpcpy(devcarrier, "/sys/class/net/");
  dptr = opendir(devcarrier);
  if (!dptr)
    _exit(1);
  i = 0;
  for (; (eptr=readdir(dptr));) {
    if (*(eptr->d_name) == '.') continue;
    l = stpcpy(l, eptr->d_name);
    *devptr = 0;
    strncat(devptr, eptr->d_name, STRING_SIZE);
    strncat(devptr, "/carrier", STRING_SIZE);
    fd_carrier = open(devcarrier, O_RDONLY);
    if (fd_carrier >= 0) {
      if (read(fd_carrier, carrierbuf, 1) > 0) {
        if (*carrierbuf == '0') {
          *l = '*'; ++l;
          ++i;
        }
      }
      close(fd_carrier);
    }
    *l = ','; ++l;
  }
  closedir(dptr);
#endif
  --l; *l = '\n'; ++l;
  l = sputlongs(l, i);


  /* we read 4 values from /proc/loadavg */
#ifdef KNOERRED
  p = loadavg[tmpcursor];
  if (__unlikely(*p == 0))
    return 0;
#else
  i = open("/proc/loadavg", O_RDONLY);
  if (__unlikely(i < 0))
    main_error_redirect_exit(5);
  p = readbuf;
  s = read(i, p, STRING_SIZE);
  close(i);
  if (__unlikely(s < 2))
    main_error_redirect_exit(6);
  p[s] = 0;
#endif

  /* step forward to load5 */
  for (;;++p) if (*p == ' ') break;

  /* copy key */
  l = stpcpy(l, " load5\n");

  ++p;
  for (;;) {
    if (*p == ' ') break;
    *l = *p;
    ++l; ++p;
  }
  *l = '\n'; ++l;

  /* copy key */
  l = stpcpy(l, " load15\n");

  ++p;
  for (;;) {
    if (*p == ' ') break;
    *l = *p;
    ++l; ++p;
  }
  *l = '\n'; ++l;

  /* copy key */
  l = stpcpy(l, " proccount\n");

  for (;;) { if (__unlikely(!*p)) break; if (*p == '/') break; ++p; } ++p;
  for (;;) {
    if (*p == ' ') break;
    *l = *p;
    ++l; ++p;
  }
  *l = '\n'; ++l;

  /* copy key */
  l = stpcpy(l, " load1\n");
#ifdef KNOERRED
  p = loadavg[tmpcursor];
#else
  p = readbuf;
#endif
  l = stpcpy(l, p);
  for (;;) {
    if (*p == ' ') break;
    *l = *p;
    ++l; ++p;
  }
  *l = '\n'; ++l;


#ifdef KNOERRED
  return l - buf;
#else
  out:
#endif
  write1(buf, l-buf);
  _exit(0);
}
#endif
