#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_LOGCHECKERR

/*
config KNOERRE_CHECK_LOGCHECKERR
    bool "check_logcheckerr"
    default y
    help
      This knoerre module scans a kernel logfile for "bad" lines.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_LOGCHECKERR_KEY
    string "key string for check_logcheckerr"
    default "logcheckerr/"
    depends on KNOERRE_CHECK_LOGCHECKERR
    help
      The key string for check_logcheckerr as described in man-page knoerre(1).
*/

/*
 * $Id: do_check_logcheckerr.c 691 2017-06-04 23:12:04Z fwbergm $
 */

/**
 * "Bad" strings and their weight in logcheck.
 *
 * These strings are some "bad" strings that nobody wants
 * to have in logfiles.
 * The more bad the log-entry is the higher is its weight.
 * These strings are actually strings and no regex's.
 * This list has to be extended.
 */
static const KLE l_bad_entries[] = {
  { "ERROR", 100 },
  { "Error", 100 },
  { "error", 100 },
  { "FAIL", 100 },
  { "Fail", 100 },
  { "fail", 100 },
  { "WARNING", 10 },
  { "Warning", 10 },
  { "warning", 10 },
  { 0, 0 }
};

#define HIGH_WEIGHT_THRESHOLD 19


/**
 * This function gets the last "bad" entry from logfile and writes it as http data out.
 *
 * $Id: do_check_logcheckerr.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 * @param arg1 The file/directory to analyze.
 */

/*
config KNOERRE_CHECK_LOGCHECKERR
    bool "check_logcheckerr"
    default y
    help
      This knoerre module searches a logfile for "error" or "fail" entries.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_LOGCHECKERR_KEY
    string "key string for check_logcheckerr"
    default "logcheckerr/"
    depends on KNOERRE_CHECK_LOGCHECKERR
    help
      The key string for check_logcheckerr as described in man-page knoerre(1).
*/

void do_check_logcheckerr(char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  register off_t ib;
  register uint32_t matches,points;
  register int c;
  register ssize_t rb;
  char pathname[PATHNAME_SIZE+1];
  char badlines[LOGCHECK_MATCH_LINES_CACHE][CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS+1];
  char rbuf[LARGEBUF_SIZE+1];
  int fd;
  int ret;

  uint16_t high_weight_found;
  uint16_t read_count;

  ret = 127;
  p = arg1;
  p[strlen(p)-1] = '\0';
  stpcpy(pathname, p);
  p = pathname;
  for(c=0;c<LOGCHECK_MATCH_LINES_CACHE;c++) *badlines[c] = 0;

  matches = points = 0;

  fd = open(pathname, O_RDONLY);
  if (fd < 0) { ret=1; goto error; }
  ib = lseek(fd, 0, SEEK_END);
  if (ib < 0) { ret=2; goto error; }
  if (ib <= LARGEBUF_SIZE) {
    ib = lseek(fd, 0, SEEK_SET);
  } else {
    ib -= LARGEBUF_SIZE;
    ib = lseek(fd, ib, SEEK_SET);
  }
  if (ib < 0) { ret=3; goto error; }
  p = rbuf;
  rb = read(fd, p, LARGEBUF_SIZE);
  if (rb < 0) { ret=4; goto error; }
  p[rb] = 0;
/*
  if (rb < CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS) goto out;
*/
  p += rb;
  read_count = 1;

  high_weight_found = 0;
  for(;;) {

    for(;;) { p--; if (p<rbuf) break; if (*p>=32) break; *p = '\0'; }
    c = 1;
    /* scan all lines in rbuf */
    for(;;) {
      --p;
      if (*p < 32) {

        if (c) {
          /* for(c=0; kernel_bad[c]; c++) { */
          for(c=0; l_bad_entries[c].string; c++) {
            if(strstr(p+1, l_bad_entries[c].string)) {
              /* ++matches; */
              matches++;
              points += l_bad_entries[c].weight;
              if (matches < LOGCHECK_MATCH_LINES_CACHE) {
                if ((matches < 3) || (l_bad_entries[c].weight > HIGH_WEIGHT_THRESHOLD)) {
                  strncpy(badlines[matches-1], p+1, CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS);
                }
                if (!high_weight_found && (l_bad_entries[c].weight > HIGH_WEIGHT_THRESHOLD)) {
                  /* no "light-weight" badline should be last line if high-weight lines are found */
                  for(;high_weight_found<matches;high_weight_found++)
                    badlines[high_weight_found][0] = 0;
                }
              }
            }
          }
        }
        c = 0;
        *p = '\0';

      } else {
        c = 1;
      }
      if (p < rbuf) break;
    }

    /* is there more data to read from file? */
    if (ib <= 0) goto out;
    /* maybe logfile is too big, limit read()-calls */
    ++read_count;
    if (read_count > 50) goto out;

    /* fill buffer again with previous data from file */
    p = rbuf;
    if (ib < (LARGEBUF_SIZE - STRING_SIZE)) {
      if (lseek(fd, 0, SEEK_SET) != 0) { ret=5; goto error; }
      rb = read(fd, p, ib+STRING_SIZE);
      ib = 0;
    } else {
      ib = ib - LARGEBUF_SIZE + STRING_SIZE;
      if ((ib=lseek(fd, ib, SEEK_SET)) < 0) { ret=6; goto error; }
      rb = read(fd, p, LARGEBUF_SIZE);
    }
    if (rb < 0) { ret=7; goto error; }
    p = p + rb;
    *p = '\0'; /* be just a little bit paranoid */
  }

  out:
  close(fd);

  p = stpcpy(bufp, "matchlines: ");
  p = sputlongs(p, matches);
  for(c=LOGCHECK_MATCH_LINES_CACHE;c>0;) {
    c--;
    if(*badlines[c]) {
      p = stpcpy(p, badlines[c]);
      *p++ = '\n';
    }
  }
  p = sputlongs(p, points);
  write1(buf, p-buf);
  _exit(0);

  error:
  if (fd >= 0) close(fd);
  p = stpcpy(bufp, ERROR_COMMON_VALUE_STRING);
  write1(buf, p-buf);
  _exit(ret);
}
#endif
