#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_LOGENTRIES

/*
config KNOERRE_CHECK_LOGENTRIES
    bool "check_tslogentries"
    default y
    help
      This knoerre module counts last lines in a logfile with the same beginning of line.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_LOGENTRIES_KEY
    string "key string for check_tslogentries"
    default "tslogentries/"
    depends on KNOERRE_CHECK_LOGENTRIES
    help
      The key string for check_tslogentries as described in man-page knoerre(1).
*/

#define MAX_NOMATCH_LINES 4

/*
 * $Id: do_check_logentriesdaily.c 691 2017-06-04 23:12:04Z fwbergm $
 */

uint32_t lines_matchingindex(register char *s, register char *t) {
  register char *u;

  u = s;
  for(;;) {
    if (*s != *t) break; if (!*s) break; if (!*t) break; ++s; ++t;
    if (*s != *t) break; if (!*s) break; if (!*t) break; ++s; ++t;
    if (*s != *t) break; if (!*s) break; if (!*t) break; ++s; ++t;
    if (*s != *t) break; if (!*s) break; if (!*t) break; ++s; ++t;
  }
  return s-u;
}


/**
 * getlastdailyentries() is a helper function of do_check_logentriesdaily.
 *
 * ...
 */
int64_t getlastdailyentries(const char *pathname, off_t fsize, int field, char sep) {
  register char *p;
  register off_t ib;
  register ssize_t rb;
  register uint32_t matches,nomatches;
  register int c, len;
  char *line;
  int fd;
  char buf[LARGEBUF_SIZE+1];
  char pattern[STRING_SIZE+1];
  int ret;

  matches = 1;
  nomatches = 0;
  fd = open(pathname, O_RDONLY);
  if (fd < 0) { ret=-1; goto error_out; }
  ib = fsize;
  if (ib <= LARGEBUF_SIZE) {
    ib = lseek(fd, 0, SEEK_SET);
  } else {
    ib -= LARGEBUF_SIZE;
    ib = lseek(fd, ib, SEEK_SET);
  }
  if (ib < 0) { ret=-3; goto error_out; }
  p = buf;
  rb = read(fd, p, LARGEBUF_SIZE);
  if (rb < 0) { ret=-4; goto error_out; }
  if (rb < 2) goto out;
  p += rb;
  *p = '\0';

  /* get last line */
  for(;;) { p--; if (p<buf) break; if (*p>=32) break; *p = '\0'; }
  for(;;) { p--; if (p<buf) break; if (*p<32) break; }
  p++;
  line = p;

  /* get pattern from beginning */
  for(;;) {
    if (*p == sep) field--; if (!field) break; ++p;
  }
  *p = '\0';
  stpcpy(pattern, line);
  p = line - 1;

  len = strlen(pattern);

  for(;;) {

    for(;;) { p--; if (p<buf) break; if (*p>=32) break; *p = '\0'; }
    c = 1;
    /* count all matching lines in buf */
    for(;;) {
      --p;
      if (p < buf) break;
      if (*p < 32) {
        if (c) {
          if (!strncmp(p+1, pattern, len)) {
            ++matches;
            nomatches = 0;
          } else {
            ++nomatches;
            if (nomatches > MAX_NOMATCH_LINES) break;
          }
        }
        c = 0;
        *p = '\0';
      } else {
        c = 1;
      }
    }
    if (nomatches > MAX_NOMATCH_LINES) goto out;
  
    if (ib <= 0) goto out;

    /* fill buffer again with previous data from file */
    p = buf;
    if (ib < (LARGEBUF_SIZE-len)) {
      if (lseek(fd, 0, SEEK_SET) < 0) { ret=-5; goto error_out; }
      rb = read(fd, p, ib);
      ib = 0;
    } else {
      ib = ib - LARGEBUF_SIZE + len;
      if ((ib=lseek(fd, ib, SEEK_SET)) < 0) { ret=-6; goto error_out; }
      rb = read(fd, p, LARGEBUF_SIZE);
    }
    if (rb < 0) { ret=-7; goto error_out; }
    p = p + rb;
    *p = '\0'; /* be just a little bit paranoid */
  }

  out:
  close(fd);
  return matches;
  
  error_out:
  if (fd < 0) close(fd);
  return ret;
}


/**
 * This function counts the last (current day?) lines of a logfile.
 *
 * This function is called by knoerre main().
 * On logfiles with a timestamp at the beginning of a line this check can
 * be used to count the number of lines of the current day.
 * The variable 'field' will hold the column up to that a timestamp is
 * unique for a day. Take the timestamp up to 'field' from the last
 * logline and count all lines beginning with this timestamp from the last
 * line upwards until no match is found.
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 * @param arg1 The file/directory to analyze.
 */
void do_check_logentriesdaily(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  register char *nptr;
  register char *bd;
  register char *sd;
  register uint32_t dotdirs;
  register int64_t s;
  register char *pattern;
  int16_t field;
  char sep;
  struct stat stat_buf;
  /* int i; */
  char pathname[2*PATHNAME_SIZE];
  char pathname_max[2*PATHNAME_SIZE];
  char pathname_max2[2*PATHNAME_SIZE];
  struct dirent *eptr;
  DIR *dptr;
  uint32_t maxentries;
  uint32_t maxentries2;

  s = maxentries = maxentries2 = 0;
  sep = ' ';
  sd = pattern = NULL;
  bd = arg1;
  bd[strlen(bd)-1]='\0';
  *pathname_max = *pathname_max2 = '\0';

  bd++;
  field = *bd - '0';
  bd++;
  if (*bd != '/') {
    sep = *bd;
    if (sep == '_') sep = ' ';
    bd++;
  }

  /* i = str_index(bd,"/./"); */
  p = strstr(bd,"/./"); /* "wildcard" like "*" in shell */
  if (!p) p = strstr(bd,"/@/"); /* "wildcard" like "*" in shell */
  if (p) {
    *p = 0;
    sd = p + 3;
  }

  /* if (i > 0) { */
  if (p) {

    /*
     * step through all entries in bd
     * and search for matching files in subdir,
     * we don't need leaf optimization here,
     * just go through the whole bd
     */
    dptr = opendir(bd);
    if(dptr==NULL) goto error;
    dotdirs = 0;
    for(;;) {
      eptr = readdir(dptr); if(eptr==NULL) break;
      nptr = eptr->d_name;
      if((dotdirs<2)&&(*nptr=='.')) {
        if(nptr[1]=='\0') { dotdirs += 1;continue; }
        if(nptr[1]=='.') {
          if(nptr[2]=='\0') { dotdirs += 1;continue; }
        }
      }
      p = stpcpy(pathname, bd);
      *p++ = '/';
      p = stpcpy(p, nptr);
      *p++ = '/';
      p = stpcpy(p, sd);
      p = pathname;
      if(lstat(p,&stat_buf)==0){
        s = getlastdailyentries(p, stat_buf.st_size, field, sep);
        if (s > maxentries) {
          maxentries2 = maxentries;
          stpcpy(pathname_max2, pathname_max);
          maxentries = s;
          stpcpy(pathname_max, nptr);
        } else {
          if (s > maxentries2) {
            maxentries2 = s;
            stpcpy(pathname_max2, nptr);
          }
        }
      }
    }
    closedir(dptr);

  } else {

    /*
     * just a single file to check, no wildcard '.' given
     */
    *pathname_max = '\0';
    if(lstat(bd,&stat_buf)!=0) goto error;
    maxentries = getlastdailyentries(pathname, stat_buf.st_size, field, sep);

  }

  p = bufp;
  if (maxentries > 0) {
    p = stpcpy(p, pathname_max);
    *p++ = '=';
    p = sputlongs(p, maxentries);
    if (maxentries2 > 0) {
      --p;
      *p++ = ' ';
      p = stpcpy(p, pathname_max2);
      *p++ = '=';
      p = sputlongs(p, maxentries2);
    }
    --p;
    *p++ = '\n';
  }
  p = sputlongs(p, maxentries);
  write1(buf, p-buf);
  _exit(0);

  error:
  p = stpcpy(bufp, ERROR_COMMON_VALUE_STRING);
  write1(buf, p-buf);
  _exit(1);
}
#endif
