#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_LONGPROCP
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_LONGPROCP
    bool "check_longprocp"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module delivers the process age in minutes of the longest running user process.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_LONGPROCP_KEY
    string "key string for check_longprocp"
    default "longprocp/"
    depends on KNOERRE_CHECK_LONGPROCP
    help
      The key string for check_longprocp as described in man-page knoerre(1).
*/

/**
 * This function gets the time in minutes of the longest running user process and writes it as http data out.
 *
 * This function is called by knoerre main().
 * $Id: do_check_longprocp.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 * @param arg1 The file/directory which lines to count
 */
void do_check_longprocp(char *buf, const int buflen, char *bufp, char *arg1) {

  register char *p;
  int16_t c;
  int rf;
  uint64_t mins;
  uint64_t pid;
  char maxtime_proc[STRING_SIZE+1];
  uint64_t maxtime_mins;
  uint64_t maxtime_pid;
  char maxtime_proc2[STRING_SIZE+1];
  uint64_t maxtime_mins2;
  uint64_t maxtime_pid2;
  char maxtime_proc3[STRING_SIZE+1];
  uint64_t maxtime_mins3;
  uint64_t maxtime_pid3;
  int64_t lval;
  int64_t uptime;
  uint64_t uid_min;
  uint64_t uid_max;
  char name[2*PATHNAME_SIZE];
  char filebuf[BUF_SIZE+1];
  char *excludes[17];
  DIR *dptr;
  struct dirent *eptr;

  maxtime_mins3=maxtime_pid3=maxtime_mins2=maxtime_pid2=maxtime_mins=maxtime_pid=0;


  p = arg1;
  p[strlen(p)-1]='\0';

  if (!str2uint64(p, &uid_min)) goto err_exit;
  while((*p >= '0') && (*p <= '9')) p++;
  if (*p != '/') goto err_exit;

  p++;
  if (!str2uint64(p, &uid_max)) goto err_exit;
  while((*p >= '0') && (*p <= '9')) p++;

  if (uid_max < uid_min) goto err_exit;

  for(c=0;c<17;c++) excludes[c]=0;
  if (*p == 0) goto end_excludes;
  if (*p != '/') goto err_exit;

  for(c=0;;c++) {
    ++p;
    if (*p == '/') goto err_exit;
    if (*p == 0) goto err_exit;
    if (c > 15) goto err_exit;
    excludes[c] = p;
    while((*p != 0) && (*p != '/')) p++;
    if (*p == 0) break;
    *p = 0;
  }

  end_excludes:
  /* get uptime */
  p = name;
  stpcpy(p,"/proc/uptime");
  rf=open(p, O_RDONLY);
  if(rf<0) goto err_exit;
  p = filebuf;
  if(read(rf,p,BUF_SIZE)<2) { close(rf); goto err_exit; }
  p[BUF_SIZE]='\0';
  str2uint64(p, (uint64_t *) &uptime);
  uptime*=100;
  close(rf);

  /* scan all /proc/NUMBER/stat */
  dptr=opendir("/proc");
  if(dptr==NULL) goto err_exit;

  for(;(eptr=readdir(dptr))!=NULL;) {

    if ((*(eptr->d_name) < '0') || (*(eptr->d_name) > '9')) continue;

    /* don't use stat/lstat(), read .../status, works on 32- and 64-Bit os */
    p = stpcpy(name, "/proc/");
    p = stpcpy(p, eptr->d_name);
    p = stpcpy(p, "/status");
    p = name;

    rf = open(p, O_RDONLY);
    if (rf < 0) continue;
    p = filebuf;
    if (read(rf, p, BUF_SIZE) <2 ) goto next_file;
    p[BUF_SIZE] = '\0';
    close(rf);
    if ((c = str_index2(p, proc_status_uid_string)) < 0) continue;
    p += c; /* points to real uid */
    /* c = field_index(p, 1); */
    /* c = str_field(p, 1, '\t', 0); */
    p = str_field(p, 1, '\t', 0);
    if (!str2uint64(p, (uint64_t *) &lval)) /* effective user id */
      lval = uid_max - 1; /* on error: check this proc, too */
    if ((lval < uid_min) || (lval > uid_max)) continue;

    /* read in a single stat file */
    p = stpcpy(name, "/proc/");
    p = stpcpy(p, eptr->d_name);
    p = stpcpy(p, "/stat");
    p = name;

    /* read process stat */
    rf=open(name, O_RDONLY);
    if(rf>=0) {
      p = filebuf;
      if(read(rf, p, BUF_SIZE)<2) goto next_file;
      p[BUF_SIZE]='\0';

      /* go through the proc exclude list */
      for(c=0;excludes[c];c++) {
        p = stpcpy(name, " (");
        p = stpcpy(p, excludes[c]);
        p = stpcpy(p, ") ");
        if(str_index2(filebuf,name)>=0) goto next_file;
      }
      p = filebuf;
      str2uint64(p, &pid); /* no error check */

      /* get jiffies and compare */
      /* c=field_index(p,21); */
      p = str_field(p, 21, ' ', 0);
      str2uint64(p, (uint64_t *) &lval);
      if (lval<uptime) {
        /* not just started */
        mins=(uptime-lval)/6000;

        /* longest running proc? */
        if (mins > maxtime_mins) {
          maxtime_mins3 = maxtime_mins2;
          maxtime_pid3 = maxtime_pid2;
          stpcpy(maxtime_proc3, maxtime_proc2);
          maxtime_mins2 = maxtime_mins;
          maxtime_pid2 = maxtime_pid;
          stpcpy(maxtime_proc2, maxtime_proc);
          maxtime_mins = mins;
          maxtime_pid = pid;
          p = filebuf;
          p = strchr(p, '('); p++;
/*
          p[str_chr(p, ')')] = '\0';
*/
          *strchr(p, ')') = 0;
          strcpy(maxtime_proc, p);
        } else {
          if (mins > maxtime_mins2) {
            maxtime_mins3 = maxtime_mins2;
            maxtime_pid3 = maxtime_pid2;
            stpcpy(maxtime_proc3, maxtime_proc2);
            maxtime_mins2 = mins;
            maxtime_pid2 = pid;
            p = filebuf;
            p = strchr(p, '('); p++;
            *strchr(p, ')') = '\0';
            strcpy(maxtime_proc2, p);
          } else {
            if (mins > maxtime_mins3) {
              maxtime_mins3 = mins;
              maxtime_pid3 = pid;
              p = filebuf;
              p = strchr(p, '('); p++;
              *strchr(p, ')') = '\0';
              strcpy(maxtime_proc3, p);
            }
          }
        }
      }

      next_file:
      close(rf);
    }

  }
  closedir(dptr);

  p = stpcpy(bufp, "uid_min=");
  p = sputlongs(p, uid_min);
  p = stpcpy(p, "uid_max=");
  p = sputlongs(p, uid_max);
  p = stpcpy(p, maxtime_proc);
  *p++ = '/';
  p = sputlongs(p, maxtime_pid);
  --p;
  *p++ = '=';
  p = sputlongs(p, maxtime_mins);
  if (maxtime_mins2 > 0) {
    --p;
    *p++ = ' ';
    p = stpcpy(p, maxtime_proc2);
    *p++ = '/';
    p = sputlongs(p, maxtime_pid2);
    --p;
    *p++ = '=';
    p = sputlongs(p, maxtime_mins2);
    if (maxtime_mins3 > 0) {
      --p;
      *p++ = ' ';
      p = stpcpy(p, maxtime_proc3);
      *p++ = '/';
      p = sputlongs(p, maxtime_pid3);
      --p;
      *p++ = '=';
      p = sputlongs(p, maxtime_mins3);
    }
  }

  p = sputlongs(p, maxtime_mins);
  write1(buf, p - buf);
  _exit(0);

  err_exit:
  check_error_exit(0);
}

#endif
