#ifdef CONFIG_KNOERRE_CHECK_LONGPROCS
#error DEPRECATED, LONGPROCS is removed from source
#include "knoerre.h"
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/**
 * This function gets the time in minutes of the longest running user process and writes it as http data out.
 *
 * This function is called by knoerre main().
 * $Id: do_check_longuserprocesses.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param uid_min The minimum uid the process must match.
 * @param uid_max The maximum uid the process must match.
 * @param excludes A list of process names to exclude.
 */
void do_check_longuserprocesses(
    register char *buf, const int buflen, char *bufp, uint64_t uid_min, uint64_t uid_max, char *excludes[]) {

  register char *p;
  register int64_t lval;
  register int16_t c;
  register int rf;
  register int64_t uptime;
  register uint64_t mins;
  register uint64_t pid;
  char maxtime_proc[STRING_SIZE+1];
  register uint64_t maxtime_mins;
  register uint64_t maxtime_pid;
  char maxtime_proc2[STRING_SIZE+1];
  register uint64_t maxtime_mins2;
  register uint64_t maxtime_pid2;
  char maxtime_proc3[STRING_SIZE+1];
  register uint64_t maxtime_mins3;
  register uint64_t maxtime_pid3;
  char name[2*PATHNAME_SIZE];
  char filebuf[BUF_SIZE+1];
  DIR *dptr;
  struct dirent *eptr;
  char *excludes_default[]={"vsftpd", "bash", "sftp-server", 0};

  if(!excludes) excludes=excludes_default;
  maxtime_mins3=maxtime_pid3=maxtime_mins2=maxtime_pid2=maxtime_mins=maxtime_pid=0;

  /* get uptime */
  p = name;
  stpcpy(p,"/proc/uptime");
  rf = open(p, O_RDONLY);
  if (rf < 0) goto err_exit;
  p = filebuf;
  if(read(rf,p,BUF_SIZE)<2) { close(rf); goto err_exit; }
  p[BUF_SIZE]='\0';
  uptime=str2uint63(p)*100;
  close(rf);

  /* scan all /proc/NUMBER/stat */
  dptr=opendir("/proc");
  if(dptr==NULL) goto err_exit;

  for(;(eptr=readdir(dptr))!=NULL;) {

    if ((*(eptr->d_name) < '0') || (*(eptr->d_name) > '9')) continue;

    /* don't use stat/lstat(), read .../status, works on 32- and 64-Bit os */
    p = stpcpy(name, "/proc/");
    p = stpcpy(p, eptr->d_name);
    p = stpcpy(p, "/status");
    p = name;

    rf = open(p, O_RDONLY);
    if (rf < 0) continue;
    p = filebuf;
    if (read(rf, p, BUF_SIZE) <2 ) goto next_file;
    p[BUF_SIZE] = '\0';
    if ((c = str_index2(p, proc_status_uid_string)) < 0) goto next_file;
    p += c; /* points to real uid */
    c = field_index(p, 1);
    lval = str2uint63(p+c); /* effective user id */
    if ((lval < uid_min) || (lval > uid_max)) goto next_file;
    close(rf);

    /* read in a single stat file */
    p = stpcpy(name, "/proc/");
    p = stpcpy(p, eptr->d_name);
    p = stpcpy(p, "/stat");
    p = name;

    /* read process stat */
    rf = open(name, O_RDONLY);
    if (rf >= 0) {
      p = filebuf;
      if(read(rf, p, BUF_SIZE)<2) goto next_file;
      p[BUF_SIZE]='\0';

      /* go through the proc exclude list */
      for(c=0;excludes[c];c++) {
        p = stpcpy(name, " (");
        p = stpcpy(p, excludes[c]);
        p = stpcpy(p, ") ");
        if(str_index2(filebuf,name)>=0) goto next_file;
      }
      p = filebuf;
      pid=str2uint63(p); /* no error check */

      /* get jiffies and compare */
      c=field_index(p,21);
      lval=str2uint63(p+c);
      if (lval<uptime) {
        /* not just started */
        mins=(uptime-lval)/6000;

        /* longest running proc? */
        if (mins > maxtime_mins) {
          maxtime_mins3 = maxtime_mins2;
          maxtime_pid3 = maxtime_pid2;
          stpcpy(maxtime_proc3, maxtime_proc2);
          maxtime_mins2 = maxtime_mins;
          maxtime_pid2 = maxtime_pid;
          stpcpy(maxtime_proc2, maxtime_proc);
          maxtime_mins = mins;
          maxtime_pid = pid;
          p = filebuf;
          p = strchr(p, '('); p++;
          *strchr(p, ')') = '\0';
          strcpy(maxtime_proc, p);
        } else {
          if (mins > maxtime_mins2) {
            maxtime_mins3 = maxtime_mins2;
            maxtime_pid3 = maxtime_pid2;
            stpcpy(maxtime_proc3, maxtime_proc2);
            maxtime_mins2 = mins;
            maxtime_pid2 = pid;
            p = filebuf;
            p = strchr(p, '('); p++;
            *strchr(p, ')') = '\0';
            strcpy(maxtime_proc2, p);
          } else {
            if (mins > maxtime_mins3) {
              maxtime_mins3 = mins;
              maxtime_pid3 = pid;
              p = filebuf;
              p = strchr(p, '('); p++;
              *strchr(p, ')') = '\0';
              strcpy(maxtime_proc3, p);
            }
          }
        }
      }

      next_file:
      close(rf);
    }

  }
  closedir(dptr);

  p = stpcpy(bufp, "uid_min=");
  p = sputlongs(p, uid_min);
  p = stpcpy(p, "uid_max=");
  p = sputlongs(p, uid_max);
  p = stpcpy(p, maxtime_proc);
  *p++ = '/';
  p = sputlongs(p, maxtime_pid);
  --p;
  *p++ = '=';
  p = sputlongs(p, maxtime_mins);
  if (maxtime_mins2 > 0) {
    --p;
    *p++ = ' ';
    p = stpcpy(p, maxtime_proc2);
    *p++ = '/';
    p = sputlongs(p, maxtime_pid2);
    --p;
    *p++ = '=';
    p = sputlongs(p, maxtime_mins2);
    if (maxtime_mins3 > 0) {
      --p;
      *p++ = ' ';
      p = stpcpy(p, maxtime_proc3);
      *p++ = '/';
      p = sputlongs(p, maxtime_pid3);
      --p;
      *p++ = '=';
      p = sputlongs(p, maxtime_mins3);
    }
  }

  p = sputlongs(p, maxtime_mins);
  write(1, buf, p - buf);
  _exit(0);

  err_exit:
  check_error_exit(buf, buflen, 0);
}
#endif
