#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_MAILBACKUP

/*
config KNOERRE_CHECK_MAILBACKUP
    bool "check_mailbackup"
    default n
    help
      This knoerre module returns the hours since last backup.
      If unsure say "N".

config KNOERRE_CHECK_MAILBACKUP_KEY
    string "key string for check_mailbackup"
    default "mailbackup/"
    depends on KNOERRE_CHECK_MAILBACKUP
    help
      The key string for check_mailbackup.
*/

/**
  * Return hours since last backup.
  *
  * This function is called by knoerre main().
  * The timestamp of the last backup in format YYYYMMDD is
  * taken from file /arg1/ and the difference in hours
  * to current time is returned.
  * $Id: do_check_mailbackup.c 691 2017-06-04 23:12:04Z fwbergm $
  *
  * @param buf The http data buf for returning data.
  * @param buflen The size of the http data buf.
  * @param bufp The pointer to char after http-header in buf.
  * @param arg1 The file containing the timestamp ( absolute path).
  */
void do_check_mailbackup(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  register long tmp;
  int bak_year,bak_mon,bak_mday;
  char readbuf[STRING_SIZE+1];
  time_t tp,tnow;
  struct tm t;
  register int fd;
  char pathname[PATHNAME_SIZE+1];

  p = arg1;
  p[strlen(p)-1] = '\0';
  stpcpy(pathname, p);


  p=readbuf;
  fd=open(pathname, O_RDONLY);
  if (fd == -1) {
    p = stpcpy(p, "can't open ");
    p = stpcpy(p, pathname);
    p = stpcpy(p, "\n"ERROR_COMMON_VALUE_STRING);
    tmp = -1;
    goto exit;
  }
  tmp=read(fd, p, 32); close(fd);
  if (tmp<=0) {
    p = stpcpy(p, "can't read ");
    p = stpcpy(p, pathname);
    p = stpcpy(p, "\n"ERROR_COMMON_VALUE_STRING);
    goto exit;
  }
  p[tmp]='\0';
  bak_year=(*p++-'0')*1000; bak_year+=(*p++-'0')*100;
  bak_year+=(*p++-'0')*10; bak_year+=*p++-'0';
  bak_mon=(*p++-'0')*10; bak_mon+=*p++-'0';
  bak_mday=(*p++-'0')*10; bak_mday+=*p-'0';
  if((bak_year<2007)||(bak_year>2050)) goto error;
  if((bak_mon<1)||(bak_mon>12)) goto error;
  if((bak_mday<1)||(bak_mday>31)) goto error;

  tp = time(NULL);
  if (tp<10) goto error;
  tnow=tp;
  if(localtime_r(&tp,&t)==NULL) goto error;
  t.tm_year=bak_year-1900;
  t.tm_mon=bak_mon-1; t.tm_mday=bak_mday;
  t.tm_hour=0; t.tm_min=0;
  tp=mktime(&t);
  if(tp<0) goto error;
  tmp=(long)((tnow-tp)/3600);
  if(tmp<0)  goto error;


  exit:
  p = stpcpy(bufp, readbuf);
  if (tmp>=0) {
    p = sputlongs(p,tmp);
  }
  write1(buf, p-buf);
  _exit(0);

  error:
  p = readbuf+strlen(readbuf);
  p = stpcpy(p, "can't parse ");
  p = stpcpy(p, pathname);
  p = stpcpy(p, "\n"ERROR_COMMON_VALUE_STRING);
  tmp = -1;
  goto exit;
}

#endif
