#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_MAXFILESIZES_KEY

#define MAXCOUNT_MAXFILESIZES 20

/*
config KNOERRE_CHECK_MAXFILESIZES
    bool "check_maxfilesizes"
    default y
    help
      This knoerre module returns the biggest file size recursively.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_MAXFILESIZES_KEY
    string "key string for check_maxfilesizes"
    default "maxfilesizes/"
    depends on KNOERRE_CHECK_MAXFILESIZES
    help
      The key string for check_maxfilesizes as described in man-page knoerre(1).

config KNOERRE_CHECK_MAXFILESIZESSUM_KEY
    string "key string for check_maxfilesizessum"
    default "maxfilesizessum/"
    depends on KNOERRE_CHECK_MAXFILESIZES
    help
      The key string for check_maxfilesizessum as described in man-page knoerre(1).
*/

typedef struct struct_filesizes {
  char *paths[MAXCOUNT_MAXFILESIZES];
  uint32_t mbsizes[MAXCOUNT_MAXFILESIZES];
} MFS;


/*
 * do_check_maxsizes.c
 * $Id: do_check_maxsizes.c 691 2017-06-04 23:12:04Z fwbergm $
 */

/**
 * Search recursively for biggest files.
 *
 * This is a helper function of do_check_maxsizes.
 */
int walkdirsize(name, pathname_size, depth, curdepth, subdir, errfile, maxfilesizes, sumsize)
const char *name;
uint16_t pathname_size;
int32_t depth;
int32_t curdepth;
const char *subdir;
char *errfile;
MFS *maxfilesizes;
uint64_t *sumsize;
{

  register DIR *dptr;
  register struct dirent *eptr;
  struct stat statbuf;
  register char *nptr;
  register char *p;
  register uint32_t stsizemb;
  register int dotdirs;
  register int ret;
  int r2;
  char pathname[PATHNAME_SIZE*2];

  dptr = opendir(name);
  if (dptr == NULL) return -1;
  ret = 0;
  dotdirs = 0;
  for(;;) {
    eptr = readdir(dptr); if (eptr == NULL) break;
    nptr = eptr->d_name;
    if ((dotdirs<2)&&(*nptr == '.')) {
      if (nptr[1]=='\0') { dotdirs+=1;continue; }
      if (nptr[1]=='.') {
        if (nptr[2]=='\0') { dotdirs+=1;continue; }
      }
    }
    p = stpcpy(pathname, name);
    *p++ = '/';
    p = stpcpy(p, nptr);
    if (subdir!=NULL) {
      if(*subdir!=0) {
        *p++ = '/';
        p = stpcpy(p, subdir);
      }
    }

    if (lstat(pathname,&statbuf)==0){
      if ((statbuf.st_mode&S_IFMT)==S_IFDIR) {
        if (depth>0) {
          if (curdepth<depth) {
            r2 = walkdirsize(pathname,pathname_size,depth,curdepth+1,NULL,errfile, maxfilesizes, sumsize);
            if (r2 != -1) ret = r2;
          }
        } else {
          r2 = walkdirsize(pathname,pathname_size,depth,0,NULL,errfile, maxfilesizes, sumsize);
          if (r2 != -1) ret = r2;
        } /* depth */
      } else {
        stsizemb = statbuf.st_size / 1048576;
        *sumsize = *sumsize + stsizemb;
        r2 = MAXCOUNT_MAXFILESIZES - 1;
        if (stsizemb > maxfilesizes->mbsizes[r2]) {
          maxfilesizes->mbsizes[r2] = stsizemb;
          strncpy(maxfilesizes->paths[r2], pathname, PATHNAME_SIZE);
          for(; r2 > 0; ) {
            --r2;
            if (stsizemb <= maxfilesizes->mbsizes[r2]) break;
            maxfilesizes->mbsizes[r2+1] = maxfilesizes->mbsizes[r2];
            strncpy(maxfilesizes->paths[r2+1], maxfilesizes->paths[r2], PATHNAME_SIZE);
            maxfilesizes->mbsizes[r2] = stsizemb;
            strncpy(maxfilesizes->paths[r2], pathname, PATHNAME_SIZE);
          }
        }
      } /* statbuf */
    } else { /* lstat */
      ret = errno;
      strncpy(errfile, pathname, PATHNAME_SIZE);
    } /* lstat */

  }
  closedir(dptr);
  return ret;
}


/**
 * This function search the biggest files recursively in a dir.
 *
 * This function is called by knoerre main().
 * Return biggest file size recursively.
 * Format: maxfilesizes/X/absolute/path/to/dir
 *         maxfilesizessum/X/absolute/path/to/dir
 * where cipher X is the recursive search depth.
 * Find the biggest files and print paths and sizes in MB.
 * The return value is the size of the biggest file in MB or the
 * sum of sizes of all scanned files (maxfilesizessum, like "du -s").
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param arg1 The directory to analyze.
 * @param depth The recursion level for stepping into the directory.
 */
void do_check_maxsizes(register char *buf, const int buflen, char *bufp, char *arg1, uint16_t depth, int sum_mode) {
  register char *p;
  register char *subdir;
  register int r;
  register char *basedir;
  register uint32_t maxsize;
  uint64_t sumsize;
  char errfile[(2*PATHNAME_SIZE)+1];
  char msfpaths[MAXCOUNT_MAXFILESIZES][PATHNAME_SIZE+1];
  MFS maxfilesizes;

  sumsize = 0;
  for(r=0; r < MAXCOUNT_MAXFILESIZES; r++) {
    msfpaths[r][0] = 0;
    maxfilesizes.paths[r] = msfpaths[r];
    maxfilesizes.mbsizes[r] = 0;
  }
  maxsize = 0;
  basedir = arg1;
  p = strchr(basedir, ' ');
  *p = 0;
  *errfile = 0;

  /* wildcards? */
  subdir = strstr(basedir,"/./");
  if (!subdir) subdir = strstr(basedir,"/@/");
  /* strip trailing slash */
  r = strlen(basedir);
  if(r > 1) {
    p = basedir + r - 1;
    if(*p == '/') *p = 0;
  }
  if (subdir) {
    *subdir = 0;
    subdir += 3;
    r = walkdirsize(basedir,PATHNAME_SIZE,depth,0,subdir,errfile, &maxfilesizes, &sumsize);
  } else {
    r = walkdirsize(basedir,PATHNAME_SIZE,depth,0,NULL,errfile, &maxfilesizes, &sumsize);
    if (r!=0) goto error;
  }

  p = bufp;
  for(r = MAXCOUNT_MAXFILESIZES; r > 0;) {
    --r;
    if (maxfilesizes.mbsizes[r] > 0) {
      p = stpcpy(p, maxfilesizes.paths[r]);
      *p++ = '=';
      p = sputlongs(p, maxfilesizes.mbsizes[r]);
      if (maxfilesizes.mbsizes[r] > maxsize) maxsize = maxfilesizes.mbsizes[r];
    }
  }
  p = stpcpy(p, "sum=");
  p = sputlongs(p, sumsize);
  if (sum_mode) {
    p = sputlongs(p, sumsize);
  } else {
    p = sputlongs(p, maxsize);
  }
  write1(buf, p-buf);
  _exit(r);

  error:
  check_error_exit(r);
#ifdef NULLDEF
  check_error_exit_ns(buf, buflen, 1, (uint32_t)r, strerror(r), errfile);
#endif
}

#endif
