#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_MOUNTOPTS
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_MOUNTOPTS
    bool "check_mountopts"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module checks the mount options used.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_MOUNTOPTS_KEY
    string "key string for check_mountopts"
    default "mountopts/"
    depends on KNOERRE_CHECK_MOUNTOPTS
    help
      The key string for check_mountopts as described in man-page knoerre(1).
*/

/**
 * This function checks mountopts of a mountpoint.
 *
 * This function is called by knoerre main().
 * The first arg is the default mount option string and the second arg is
 * the path to the mounted fs.
 * The first line of /proc/mounts with a matching mount point is checked
 * for the same *beginning* of the given default mount options.
 *
 * Example:
 * # cat /proc/mounts
 * rootfs / rootfs rw 0 0
 * tmpfs /lib/init/rw tmpfs rw,nosuid 0 0
 * proc /proc proc rw,nosuid,nodev,noexec 0 0
 * sysfs /sys sysfs rw,nosuid,nodev,noexec 0 0
 * # knoerre/knoerre mountopts/rw,nosuid/home
 * /home==rw,nosuid?
 * /dev/sda7 /home ext3 rw,nosuid,nodev,data=ordered 0 0
 * 0
 *
 * $Id: do_check_mountopts.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param arg1 Search string and filename.
 */
void do_check_mountopts(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  register ssize_t bytes;
  register int i;
  register char *r;
  register char *line;
  char device[PATHNAME_SIZE+1];
  char chk_mountopts[PATHNAME_SIZE+1];
  char fbuf[BUF_SIZE+1];

  p = strchr(arg1, ' ');
  if (!p) goto err_exit;
  *p = 0;
  p = strchr(arg1, '/');
  if (!p) goto err_exit;
  
  strcpy(device, p);
  *p = 0;
  strcpy(chk_mountopts, arg1);

  p = stpcpy(bufp, device);
  *p++ = ' ';
  r = p;
  *r = 0;

  i=open("/proc/mounts", O_RDONLY);
  if (i == -1) goto err_exit;
  p = fbuf;
  line = p;
  *p = 0;
  bytes=read(i, p, BUF_SIZE);
  close(i);
  if (bytes<=0) goto err_exit;
  p[bytes] = 0;
  i = r - bufp;
  r = bufp;
  for(;;) {
    p = strchr(p, ' ');
    if (!p) break;
    ++p;
    if (!strncmp(p, r, i)) goto found;
    p = strchr(p+1, '\n');
    if (!p) break;
    ++p; line = p;
  }

  err_exit:
  p = stpcpy(bufp, ERROR_COMMON_VALUE_STRING);
  write1(buf, p-buf);
  _exit(0);

  found:
  r = stpcpy(bufp, device);
  *r++ = '='; *r++ = '=';
  r = stpcpy(r, chk_mountopts);
  *r++ = '?'; *r++ = '\n';
  strcpy(r, line);
  r = strchr(r, '\n');
  if (!r) goto err_exit;
  ++r;

  p = strchr(p, ' ');
  if (!p) goto err_exit;
  p = strchr(p+1, ' ');
  if (!p) goto err_exit;
  ++p;
  if (!strncmp(p, chk_mountopts, strlen(chk_mountopts))) {
    *r++ = '0';
  } else {
    *r++ = '1';
  }
  *r++ = '\n';

  write1(buf, r-buf);
  _exit(0);
}

#endif

