#include "knoerre.h"
#ifdef CONFIG_KNOERRE_CHECK_NETLINKSDOWN
#ifndef CONFIG_KNOERRE_SYS_ACCESS
#error SYS_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_NETLINKSDOWN
    bool "check_netlinksdown"
    default y
    depends on KNOERRE_SYS_ACCESS
    help
      This knoerre module checks all configured eth interfaces for missing links
      (no "RUNNING" flag shown on ifconfig).
      This module needs SYS_ACCESS. More information offers man-page knoerre(1).
      If unsure say "Y".

config KNOERRE_CHECK_NETLINKSDOWN_KEY
    string "key string for check_netlinksdown"
    default "netlinksdown"
    depends on KNOERRE_CHECK_NETLINKSDOWN
    help
      The key string for check_netlinksdown as described in man-page knoerre(1).
*/

/**
 * Count net interfaces with link down
 *
 * This function is called by knoerre main().
 * It steps through /sys/class/net* and counts carrier with "0".
 * $Id: do_check_netlinksdown.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @return size of data to be sent, 0 on error
 */
int do_check_netlinksdown() {
  register char *p;
  register uint32_t i;
  DIR *dptr;
  struct dirent *eptr;
  char devcarrier[STRING_SIZE+28];
  char *devptr;
  int fd_carrier;
  char carrierbuf[2];

  /**
   * step through /sys/class/net/ files
   */
  devptr = stpcpy(devcarrier, "/sys/class/net/");
  dptr = opendir(devcarrier);
  if (!dptr)
#ifdef KNOERRED
    return 0;
#else
    _exit(1);
#endif
  p = knoerre_global_buf_cursor;
  i = 0;
  for (; (eptr=readdir(dptr));) {
    if (*(eptr->d_name) == '.') continue;
    p = stpcpy(p, eptr->d_name);
    *devptr = 0;
    strncat(devptr, eptr->d_name, STRING_SIZE);
    strncat(devptr, "/carrier", STRING_SIZE);
    fd_carrier = open(devcarrier, O_RDONLY);
    if (fd_carrier >= 0) {
      if (read(fd_carrier, carrierbuf, 1) > 0) {
        if (*carrierbuf == '0') {
          *p = '*'; ++p;
          ++i;
        }
      }
      close(fd_carrier);
    }
    *p = ','; ++p;
  };
  closedir(dptr);
  --p; *p = '\n'; ++p;

  /**
   * write the data into buffer and print it out
   */
  p = sputlongs(p, i);
#ifdef KNOERRED
  return p - knoerre_global_buf;
#endif
  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(0);

}
#endif
