#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_NETTRAF
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_NETTRAF
    bool "check_nettraf"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module counts network traffic.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_NETTRAF_KEY
    string "key string for check_nettraf"
    default "nettraf/"
    depends on KNOERRE_CHECK_NETTRAF
    help
      The key string for check_nettraf as described in man-page knoerre(1).
*/


/**
 * This function calculates network traffic and writes it as http data out.
 *
 * This function is called by knoerre main().
 * It reads network traffic values from /proc/net/dev,
 * calculates the traffic in KiB/s and
 * writes it as http-data out and exists.
 * $Id: do_check_nettraf.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @return size of data to be sent, 0 on error
 */
int do_check_nettraf() {
  register char *p;
#ifndef KNOERRED
  uint64_t lval;
  uint64_t total1receive, total1transmit, total2receive, total2transmit, totaldiff;
  unsigned int sleepinterval;
  int rf;
  char netdevstat[BUF_SIZE+1];
  ssize_t s;
  uint16_t devicenamelen;
#endif
  const char *devicename;

  p = knoerre_global_arg1;
#ifndef KNOERRED
  if (!*p) main_error_redirect_exit(5);
#else
  if (!*p) return 0;
#endif
  devicename = p;
  for(;;) {
    ++p; if (!*p) break; if (*p == '/') break;
  }
#ifndef KNOERRED
  devicenamelen = p - devicename;
  if (*p == '/') {
    *p = 0;
    p = str2uint64(++p, &lval);
    if (!p) main_error_redirect_exit(6);
    sleepinterval = (unsigned int) lval;
  } else {
    sleepinterval = 10;
  }


  rf = open("/proc/net/dev", O_RDONLY);
  if (rf < 0) goto err_exit;
  p = netdevstat;
  s = read(rf, p, BUF_SIZE);
  close(rf);
  if (s < 2) main_error_redirect_exit(6);
  p[s] = 0;
  /*
    lo:118983402 1307291    0    0    0     0          0         0 118983402 1307291    0    0    0     0       0          0
  eth0:2104787106 223573269    2    0    0     1          0      4314 1922834560 217901029    0    0    0     0       0          0
  eth1:215615193 3016942    0    0    0     0          0      1938 84721398 1062218    0    0    0     0       0          0
  */
  p = strstr(p, devicename);
  if (!p) main_error_redirect_exit(6);
  p += devicenamelen;
  if (*p != ':') main_error_redirect_exit(6);
  ++p;
  for(;;++p) if (*p != ' ') break;
  /* p now points to receive bytes */
  p = str2uint64(p, &lval);
  if (!p) main_error_redirect_exit(7);
  total1receive = lval;
  p = str_field(p, 8, ' ', 1);
  /* p now points to transmit bytes */
  p = str2uint64(p, &lval);
  if (!p) main_error_redirect_exit(7);
  total1transmit = lval;

  sleep(sleepinterval);

  rf = open("/proc/net/dev", O_RDONLY);
  if (rf < 0) goto err_exit;
  p = netdevstat;
  s = read(rf, p, BUF_SIZE);
  close(rf);
  if (s < 2) main_error_redirect_exit(6);
  p[s] = 0;
  p = strstr(p, devicename);
  if (!p) main_error_redirect_exit(6);
  p += devicenamelen;
  if (*p != ':') main_error_redirect_exit(6);
  ++p;
  for(;;++p) if (*p != ' ') break;
  /* p now points to receive bytes */
  p = str2uint64(p, &lval);
  if (!p) main_error_redirect_exit(7);
  total2receive = lval;
  p = str_field(p, 8, ' ', 1);
  /* p now points to transmit bytes */
  p = str2uint64(p, &lval);
  if (!p) main_error_redirect_exit(7);
  total2transmit = lval;

  /* try to detect a carry over on 32 Bit and correct it */
  if (total2receive < total1receive) {
    total2receive += (1ULL<<32);
    if (total2receive < total1receive) {
      /* carry unknown: reset to 0 */
      total2receive = total1receive = 0;
    }
  }
  if (total2transmit < total1transmit) {
    total2transmit += (1ULL<<32);
    if (total2transmit < total1transmit) {
      /* carry unknown: reset to 0 */
      total2transmit = total1transmit = 0;
    }
  }

  totaldiff = (total2receive + total2transmit - total1receive - total1transmit) / 1024;

  p = stpcpy(knoerre_global_buf_cursor, "total: ");
  p = sputlongs(p, totaldiff); --p;
  p = stpcpy(p, " KiB, interval: ");
  p = sputlongs(p, sleepinterval); --p;
  p = stpcpy(p, " s\n");
  totaldiff /= sleepinterval;
  p = sputlongs(p, totaldiff);

  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(0);

  err_exit:
  check_error_exit(0);

  return 0;

#else

  {
    int tmpcursor;
    p = (char *) devicename;
    /**
     * support two formats:
     * - nettraf/eth0
     * - nettraf0
     */
    if (*p != 'e') goto eth_number; ++p;
    if (*p != 't') return 0; ++p;
    if (*p != 'h') return 0; ++p;
    eth_number:
    tmpcursor = current_values_cursor;
    if (*p == '0') {
      p = sputlongs(knoerre_global_buf_cursor, nettraf0_kb_persecond[tmpcursor]);
    } else {
      if (*p == '1') {
        p = sputlongs(knoerre_global_buf_cursor, nettraf1_kb_persecond[tmpcursor]);
      } else {
        return 0;
      }
    }
  }
/*
  return p - knoerre_global_buf;
*/
  return p - knoerre_global_buf;

#endif

}
#endif
