#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_PROCCOUNT
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_PROCCOUNT
    bool "check_proccount"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module shows the count of all processes as shown by /proc/loadavg (including "threads").
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_PROCCOUNT_KEY
    string "key string for check_proccount"
    default "proccount "
    depends on KNOERRE_CHECK_PROCCOUNT
    help
      The key string for check_proccount as described in man-page knoerre(1).
*/


/**
 * Count all processes
 *
 * This function is called by knoerre/knoerred main().
 * It reads the number of all procs from /proc/loadavg.
 * This includes all "threads" aka processes with shared resources.
 * To count "real" processes aka processes with pid==tgid you must
 * use proccounttg which counts the pid-dirs in /proc.
 * $Id: do_check_proccount.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @return size of data to be sent, 0 on error
 */
int do_check_proccount() {
  register char *p;
  char *l;
  int i;
  ssize_t s;
  char loadavg[STRING_SIZE+1];

  i = open("/proc/loadavg", O_RDONLY);
  if (__unlikely(i < 0))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(5);
#endif
  p = loadavg;
  s = read(i, p, STRING_SIZE);
  close(i);
  if (__unlikely(s < 2))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(6);
#endif
  p[s] = '\0';
  /* copy loadavg */
  l = stpcpy(knoerre_global_buf_cursor, p);
  /* search for '/' */
  for (;;) { if (__unlikely(!*p)) break; if (*p == '/') break; ++p; } ++p;

  for (;;) {
    if (*p == ' ') break;
    *l = *p;
    ++l; ++p;
  }
  *l = '\n'; ++l;

#ifdef KNOERRED
  return l - knoerre_global_buf;
#endif
  write1(knoerre_global_buf, l - knoerre_global_buf);
  _exit(0);
  return 0;
}
#endif
