#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_PROCCOUNTOVZ
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_PROCCOUNTOVZ
    bool "check_proccountovz"
    default n
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module counts processes by stepping through /proc and count every PID-dir
      (no "threads", just processes with pid==tgid).
      In addition to check_proccount it shows the three "top" instances of OpenVZ in the line before last line.
      If unsure say "N". It will be enabled by make target knoerreovz.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_PROCCOUNTOVZ_KEY
    string "key string for check_proccountovz"
    default "proccountovz "
    help
      The key string for check_proccountovz as described in man-page knoerre(1).
*/

/**
 * Count all processes
 *
 * This function is called by knoerre main().
 * It steps through "/proc/" and counts the number of
 * processes. It differs from do_check_proccount() by reading
 * also the VPID from /proc/PID/stat and showing the "top"
 * OpenVZ instance.
 * $Id: do_check_proccountovz.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 */
void do_check_proccountovz() {
  register char *p;
  uint32_t c;
  ssize_t rb;
  DIR *dptr;
  struct dirent *eptr;
  char pname[STRING_SIZE+1];
  uint32_t ovz[MAX_OVZ_INSTANCES];
  uint32_t pcount[MAX_OVZ_INSTANCES];
  int i;
  char rbuf[BUF_SIZE+1];
  uint64_t ovzid;
  uint32_t max_cnt1, max_id1, max_cnt2, max_id2, max_cnt3, max_id3;


  /**
   * step through /proc/ files
   */
  dptr = opendir("/proc");
  if (dptr == NULL) _exit(1);
  ovz[0] = 0;
  c = 0;
  p = knoerre_global_buf_cursor;
  for(;(eptr=readdir(dptr))!=NULL;) {

    /* pid dir? */
    if (*(eptr->d_name) < '0') continue;
    if (*(eptr->d_name) > '9') continue;

    c++; /* general proc count */
    p = stpcpy(pname, "/proc/");
    p = stpcpy(p, eptr->d_name);
    stpcpy(p, "/stat");

    i = open(pname, O_RDONLY);
    if (i < 0) continue;

    p = rbuf;
    rb = read(i, p, BUF_SIZE);
    close(i);
    if (rb < 50) continue;

    p[rb] = 0;
    p = str_field(p, 50, ' ', 0);
    if (!p) continue;
    if (!str2uint64(p, &ovzid)) continue;

    /* search for ovzid in our list and increment process counter for this id */
    i = 0;
    for(;;) {
      if (i >= MAX_OVZ_INSTANCES) break;
      if (ovz[i] == 0) {
        ovz[i] = ovzid; pcount[i] = 1;
        ovz[++i] = 0; break;
      }
      if (ovz[i] == ovzid) {
        pcount[i] += 1; break;
      }
      ++i;
    }

  }
  closedir(dptr);

  /**
   * write the data into buffer and print it out
   */
  if (!p) {
    p = stpcpy(knoerre_global_buf_cursor, "no openvz kernel\n");
  } else {
    p = knoerre_global_buf_cursor;
  }
  i = 0;
  max_cnt1 = max_id1 = max_cnt2 = max_id2 = max_cnt3 = max_id3 = 0;
  for(;;) {
    if (ovz[i] == 0) break;
    if (pcount[i] > max_cnt1) {
      max_id3 = max_id2; max_cnt3 = max_cnt2;
      max_id2 = max_id1; max_cnt2 = max_cnt1;
      max_id1 = ovz[i]; max_cnt1 = pcount[i];
    } else if (pcount[i] > max_cnt2) {
      max_id3 = max_id2; max_cnt3 = max_cnt2;
      max_id2 = ovz[i]; max_cnt2 = pcount[i];
    } else if (pcount[i] > max_cnt3) {
      max_id3 = ovz[i]; max_cnt3 = pcount[i];
    }
/*
    p = sputlongs(p, ovz[i]); --p;
    *p++ = ' ';
    p = sputlongs(p, pcount[i]);
*/
    ++i;
  }
  if (max_cnt1 > 0) {
    p = sputlongs(p, max_id1);
    --p; *p++ = '=';
    p = sputlongs(p, max_cnt1);
    if (max_cnt2 > 0) {
      --p; *p++ = ' ';
      p = sputlongs(p, max_id2);
      --p; *p++ = '=';
      p = sputlongs(p, max_cnt2);
      if (max_cnt3 > 0) {
        --p; *p++ = ' ';
        p = sputlongs(p, max_id3);
        --p; *p++ = '=';
        p = sputlongs(p, max_cnt3);
      }
    }
  }
  if (c < 1) {
    p = stpcpy(p, ERROR_COMMON_VALUE_STRING);
  } else {
    p = sputlongs(p,c);
  }
  write(1, knoerre_global_buf, p - knoerre_global_buf);
  _exit(0);

}
#endif
