#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_PROCCOUNTTG
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_PROCCOUNTTG
    bool "check_proccounttg"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module counts processes by stepping through /proc and count every PID-dir (no "threads", just processes with pid==tgid).
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_PROCCOUNTTG_KEY
    string "key string for check_proccounttg"
    default "proccounttg "
    depends on KNOERRE_CHECK_PROCCOUNTTG
    help
      The key string for check_proccounttg as described in man-page knoerre(1).
*/

/**
 * Count all processes which are "thread group leaders"
 *
 * This function is called by knoerre/knoerred main().
 * It steps through "/proc/" and counts the number of
 * processes with pid == tgid.
 * $Id: do_check_proccounttg.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @return size of data to be sent, 0 on error
 */
int do_check_proccounttg() {
  register char *p;
  uint32_t c;
  DIR *dptr;
  struct dirent *eptr;
  int i;
  ssize_t s;

  /**
   * step through /proc/ files
   */
  dptr = opendir("/proc");
  if (__unlikely(dptr == 0))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(5);
#endif
  c = 0;
  for(;(eptr=readdir(dptr))!=NULL;) {
    /* pid dir? */
    if (*(eptr->d_name) > '9') continue;
    if (*(eptr->d_name) < '0') continue;
    ++c;
  }
  closedir(dptr);

  p = knoerre_global_buf_cursor;

  /* read loadavg for last before line*/
  i = open("/proc/loadavg", O_RDONLY);
  if (__unlikely(i < 0))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(6);
#endif
  s = read(i, p, knoerre_global_buf + knoerre_global_bufsize - p);
  close(i);
  if (__unlikely(s < 2))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(7);
#endif
  p += s;

  if (c < 1) {
    p = stpcpy(p, ERROR_COMMON_VALUE_STRING);
  } else {
    p = sputlongs(p, c);
  }

#ifdef KNOERRED
  return p - knoerre_global_buf;
#else
  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(0);
#endif

  return 0;
}
#endif
