#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_PROCESS
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_PROCESS
    bool "check_process"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module counts instances of a process.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_PROCESS_KEY
    string "key string for check_process"
    default "process/"
    depends on KNOERRE_CHECK_PROCESS
    help
      The key string for check_process as described in man-page knoerre(1).
      Its default is "process/".
      This key string is also the base for "process0/" and "processd/".
*/

/**
 * Get the number of running instances of a process and write it as http data out.
 *
 * This function is called by knoerre main().
 * It steps through "/proc/" and counts the number of running
 * instances of a process and
 * write it as http data out and exists.
 * If the key is not "process" but "processd" then only _real_ daemons
 * with init as father will be counted.
 * If no instance is running just return "9999" by calling check_error_exit().
 * $Id: do_check_process.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 */
int do_check_process() {
  register char *p;
  uint32_t i;
  int pstat;
  int plen;
  char *vz;
  char *ctid;
  int vzlen;
  int daemon = 0;
  int nullasnull = 0;
  DIR *dptr;
  struct dirent *eptr;
  char pathname[2*PATHNAME_SIZE];
  char statbuf[PATHNAME_SIZE+1];
  char procname[PATHNAME_SIZE+6];
  char key[PATHNAME_SIZE+1];
  char *r;

  p = strrchr(knoerre_global_arg1, ' ');
  if (!p) p = strchr(knoerre_global_arg1, '\n');
  *p = 0;
  p = knoerre_global_arg1;
  if (*p == 'd') { daemon = 1; ++p; }
  if (*p == '0') { nullasnull = 1; ++p; }
  vzlen = 0;
  ++p;
  if (!strncmp(p, "OpenVZ-CTID_", 12)) {
    /* until now no daemon detection in VPS is implemented => error */
    if (daemon)
#ifdef KNOERRED
      return 0;
#else
      main_error_redirect_exit(111);
#endif
    /* dirty, don't make "OpenVZ-CTID_" too short */
    vz = p;
    p = strchr(p, '/');
    if (!p)
#ifdef KNOERRED
      return 0;
#else
      _exit(2);
#endif
    /* see warning above */
    memcpy(vz, vz+12, p-vz-11);
    vz = strchr(vz, '/');
    *vz++ = ' ';
    *vz = 0;
    vzlen = vz - knoerre_global_arg1 - 2;
    vz = knoerre_global_arg1 + 2;
    ++p;
  } else {
    vz = 0;
  }
  knoerre_global_arg1 = p;

  p = stpcpy(key, p);
  *p++ = '\n'; *p = 0;
  
  /*
   * the procname in stat-files
   * is actually " (process) "
   */
  p = stpcpy(procname, " (");
  p = stpcpy(p, knoerre_global_arg1);
  plen = stpcpy(p, ") ") - procname;

  /**
   * step through /proc/ files
   */
  dptr = opendir("/proc");
  if (!dptr)
#ifdef KNOERRED
    return 0;
#else
    _exit(1);
#endif
  for (i=0; (eptr=readdir(dptr)); ) {
    if ((*(eptr->d_name) < '0') || (*(eptr->d_name) > '9')) continue;
    p = stpcpy(pathname, "/proc/");
    p = stpcpy(p, eptr->d_name);
    strcpy(p, "/stat");
    pstat = open(pathname, O_RDONLY);
    if (pstat >= 0) {
      if (read(pstat, statbuf, PATHNAME_SIZE) > 2) {
        statbuf[PATHNAME_SIZE] = '\0';
        p = strchr(statbuf, ' ');
        if (!strncmp(p, procname, strlen(procname))) { 
          if (vz) {
            ctid = str_field(p, 50, ' ', 0);
            if ((!ctid) || (strncmp(vz, ctid, vzlen))) goto next_file;
          }
          if (daemon) {
            *p = 0;
            p += plen + 2;
            // ppid == 1 ?
            if ((*p++ == '1') && (*p++ == ' ')) {
              p = strchr(p, ' ') + 1;
              r = strchr(p, ' ');
              *r = 0;
              /* leader of process group? */
              if (!strcmp(statbuf, p)) {
                /* be careful: some programs like dcron don't run as real
                 * daemon with process and session leadership and no
                 * controlling terminal */
                 i++;
                 knoerre_global_buf_cursor = stpcpy(knoerre_global_buf_cursor, p);
                 *knoerre_global_buf_cursor = ' '; ++knoerre_global_buf_cursor;
              }
            }
          } else {
            i++;
          }
        }
      }
      next_file:
      close(pstat);
    }
  };
  closedir(dptr);

  /**
   * write the data into buffer and print it out
   */
  p = stpcpy(knoerre_global_buf_cursor, key);
  if (i || nullasnull) {
    p = sputlongs(p,i);
  } else {
    p = stpcpy(p, ERROR_COMMON_VALUE_STRING);
  }
#ifdef KNOERRED
  return p - knoerre_global_buf;
#endif
  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(0);

}
#endif
