#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_SWAP
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_SWAP
    bool "check_swap"
    default n
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module shows used swap space in MB.
      If unsure say "N" and use KNOERRE_CHECK_SWAPS instead.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_SWAP_KEY
    string "key string for check_swap"
    default "swap "
    depends on KNOERRE_CHECK_SWAP
    help
      The key string for check_swap as described in man-page knoerre(1).
*/

/**
 * This function get used swap space in MB and writes it as http data out.
 *
 * This function is called by knoerre main().
 * It gets the amount of used swap space by reading /proc/meminfo and
 * writes it as http-data out and exists.
 *
 * The format of /proc/meminfo must be something like that:
 * MemTotal:      1033364 kB
 * [...]
 * SwapTotal:     1953784 kB
 * SwapFree:      1915996 kB
 *
 * $Id: do_check_swap.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 */
void do_check_swap(register char *buf, const int buflen, char *bufp) {
  register char *p;
  register int32_t i;
  register uint32_t mb;
  uint64_t kb;
  uint32_t memtotal;
  uint32_t swaptotal;
  char procmeminfo[BUF_SIZE+1];
  ssize_t s;

  i = (int32_t) open("/proc/meminfo", O_RDONLY);
  if (i < 0) main_error_redirect_exit(5);
  p = procmeminfo;
  s = read(i, p, BUF_SIZE);
  close(i);
  if (s == 0) {
    p = stpcpy(bufp, "/proc/meminfo empty\n");
    mb = 0;
    goto out;
  }
  if (s < 0) main_error_redirect_exit(106);
  p[s] = '\0';

  /* MemTotal */
  p = strchr(p, ' ');
  if (p == NULL) main_error_redirect_exit(107);
  for(;*p == ' ';++p);
  if (*p == 0) main_error_redirect_exit(108);
  if (!str2uint64(p, &kb)) main_error_redirect_exit(109);
  memtotal = kb / 1024;

  /* SwapTotal */
  p = strstr(p, "SwapTotal: ");
  if (p == NULL) main_error_redirect_exit(110);
  p += 11;
  for(;*p == ' ';++p);
  if (*p == 0) main_error_redirect_exit(111);
  if (!str2uint64(p, &kb)) main_error_redirect_exit(112);
  swaptotal = kb / 1024;

  /* SwapFree */
  p = strchr(p, ':');
  if (p == NULL) main_error_redirect_exit(113);
  ++p;
  for(;*p == ' ';++p);
  if (*p == 0) main_error_redirect_exit(114);
  if (!str2uint64(p, &kb)) main_error_redirect_exit(115);

  /* swap used in mb */
  mb = swaptotal - (kb / 1024);

  p = stpcpy(bufp, "memtotal:");
  p = sputlongs(p, memtotal);
  --p; *p++ = ' ';
  p = stpcpy(p, "swaptotal:");
  p = sputlongs(p, swaptotal);

  out:
  p = sputlongs(p, mb);
  write(1, buf, p-buf);
  _exit(0);
}
#endif
