#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_SWAPS
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed for this check
#endif

/*
config KNOERRE_CHECK_SWAPS
    bool "check_swaps"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module shows used swap(s) space in MB.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).
      If unsure say "Y".

config KNOERRE_CHECK_SWAPS_KEY
    string "key string for check_swaps"
    default "swaps "
    depends on KNOERRE_CHECK_SWAPS
    help
      The key string for check_swaps as described in man-page knoerre(1).
*/

/**
 * This function get used swap(s) space in MB and writes it as http data out.
 *
 * This function is called by knoerre main().
 * It gets the amount of used swap(s) space by reading /proc/swaps and
 * writes it as http-data out and exists.
 * $Id: do_check_swaps.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 */
int do_check_swaps(char *buf, const int buflen, char *bufp) {
  register char *p;
  uint32_t mb;
  int64_t i;
  char *l;
  ssize_t s;
  char procswaps[BUF_SIZE+1];
  int swaps;

  i = (int64_t) open("/proc/swaps", O_RDONLY);
  if (__unlikely(i < 0))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(5);
#endif
  p = procswaps;
  s = read(i, p, BUF_SIZE);
  close(i);
  if (s == 0) {
    l = stpcpy(bufp, "/proc/swaps empty - no swaps present\n");
    mb = 0;
    goto out;
  }
  if (__unlikely(s < 2))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(6);
#endif
  p[s] = '\0';
  p = strchr(p, '\n');
  if (__unlikely(!p))
#ifdef KNOERRED
    return 0;
#else
    main_error_redirect_exit(6);
#endif
  ++p;
  if (!*p) {
    l = stpcpy(bufp, "no entry in /proc/swaps - no swaps present\n");
    mb = 0;
    goto out;
  }
  l = stpcpy(bufp, p);
  mb = swaps = 0;
  for(;;) {
    p = str_field(p, 2, '\t', 1);
    if (__unlikely(!p))
#ifdef KNOERRED
      return 0;
#else
      main_error_redirect_exit(9);
#endif
    if (__unlikely(!str2uint64(p,(uint64_t *) &i)))
#ifdef KNOERRED
      return 0;
#else
      main_error_redirect_exit(10);
#endif
    mb = mb + ((uint32_t) (i/1024)); ++swaps;
    p = strchr(p, '\n');
    if (__unlikely(!p))
#ifdef KNOERRED
      return 0;
#else
      main_error_redirect_exit(11);
#endif
    ++p;
    if (!*p) break;
  }
  p = stpcpy(l, "swaps:");
  l = sputlongs(p, swaps);

  out:
  p = sputlongs(l, mb);
#ifdef KNOERRED
  return p - buf;
#endif
  write1(buf, p-buf);
  _exit(0);
  return 0;
}
#endif
