#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_TCP
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/*
config KNOERRE_CHECK_TCP
    bool "check_tcp"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module checks an open tcp port.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_TCP_KEY
    string "key string for check_tcp"
    default "tcp/"
    depends on KNOERRE_CHECK_TCP
    help
      The key string for check_tcp as described in man-page knoerre(1).
*/

/**
 * This function does a tcp connect to a given address/port.
 *
 * This function is called by knoerre/knoerred main().
 * It does a connect() to a given address and port. On connect-error
 * a common error value is returned. If the connect succeeds then
 * the time to connect is returned in milliseconds.
 * If we are called by knoerred then output is limited to two lines.
 * The key format is "tcp/ADDRESS/PORT".
 * $Id: do_check_tcp.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @return size of data to be sent (knoerred)
 */
int do_check_tcp() {
  register char *p;
  char *ip_address;
  int port;
  uint64_t val64;
  struct sockaddr_in server_sockaddr;
  int sockfd;
  int64_t runtime;
  struct timespec rt_tspec;
  int connected;

  /* our output (est. max. 20) plus key must fit in buffer */
  if ((20 + sizeof CONFIG_KNOERRE_CHECK_TCP_KEY) > knoerre_global_bufsize) goto error;

  /* copy key (argv[1]) to output buffer */
  p = stpcpy(knoerre_global_buf_cursor, knoerre_global_arg1);
  if (*(p-1) != ' ') {
    *p = ' ';
    ++p;
  }
  knoerre_global_buf_cursor = p;

  /* parse key */
  p = knoerre_global_arg1 - 1 + sizeof CONFIG_KNOERRE_CHECK_TCP_KEY;
  ip_address = p;
  for(;;) {
    ++p; if (!*p) goto error; if (*p == '/') break;
  }
  *p = 0;
  ++p;
  p = str2uint64(p, &val64);
  if (!p) goto error;
  port = (int) val64;

  /* set up connection */
  if (inet_aton(ip_address, &server_sockaddr.sin_addr) == 0) goto error;
  server_sockaddr.sin_family = AF_INET;
  server_sockaddr.sin_port = htons(port);
  rt_tspec.tv_sec = 0;
  rt_tspec.tv_nsec = 0;
  connected = 0;
  if (!clock_gettime(CLOCK_REALTIME, &rt_tspec))
    runtime = (1000000 * rt_tspec.tv_sec) + (rt_tspec.tv_nsec / 1000);
  else runtime = 0;
  if ((sockfd=socket(PF_INET,SOCK_STREAM,0)) == -1) goto error;
  if (connect(sockfd,(struct sockaddr *)&server_sockaddr,sizeof(server_sockaddr)) != -1)
    connected = 1;
  close(sockfd);
  if (!clock_gettime(CLOCK_REALTIME, &rt_tspec))
    runtime = ((1000000 * rt_tspec.tv_sec) + (rt_tspec.tv_nsec / 1000)) - runtime;

  /* connect-error? */
  if (!connected) {
    knoerre_global_buf_cursor = stpcpy(knoerre_global_buf_cursor, ERROR_CONNECT_STRING);
#ifndef KNOERRED
    check_error_exit(0);
#endif
    return knoerre_global_buf_cursor - knoerre_global_buf;
  }

  /* "normal" output */
  p = knoerre_global_buf_cursor - 1;
  *p = '\n'; ++p;
  p = sputlongs(p, runtime);
#ifdef KNOERRED
  return p - knoerre_global_buf;
#else
  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(0);
#endif

  error:
  knoerre_global_buf_cursor = stpcpy(knoerre_global_buf_cursor - 1, "\n"ERROR_COMMON_VALUE_STRING);
#ifndef KNOERRED
  check_error_exit(0);
#endif
  return knoerre_global_buf_cursor - knoerre_global_buf;
}
#endif
