#ifdef PLUGIN
#ifndef KNOERRED
#define KNOERRED
#endif
#endif
#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_TIME

/*
config KNOERRE_CHECK_TIME
    bool "check_time"
    default y
    help
      This knoerre module measures the execution time of the given command.
      For security reasons the command must be placed in /usr/bin.
      More information offers man-page knoerre(1).

config KNOERRE_CHECK_TIME_KEY
    string "key string for check_time"
    default "time/"
    depends on KNOERRE_CHECK_TIME
    help
      The key string for check_time as described in man-page knoerre(1).
*/


static pid_t do_check_time_pid;
int64_t do_check_time_runtime;
struct timespec do_check_time_rt_tspec;

/**
 * This signal handler will reset checktime if child terminated successfully.
 *
 * This is a helper function of do_check_proctime().
 * It will be executed if our forked child finished its job
 * and terminated itself well.
 *
 * @param num The signal caught.
 */
static void check_time_sig_handler(int num) {
  register char *p;
  int child_status;
  int signaled_pid;

  /* ignore all signals, check_time_sig_handler should only be called once */
  signal(SIGCHLD, SIG_IGN);

  /* match do_check_time_pid? */
  signaled_pid = waitpid(do_check_time_pid, &child_status, 0);
  if ((signaled_pid == -1) && (errno == ECHILD)) goto child_finished_normally;
  if (do_check_time_pid && (signaled_pid != do_check_time_pid)) goto leave_handler;
  p = knoerre_global_buf_cursor;
  /* child killed? */
  if (WIFSIGNALED(child_status)) goto errexit;
  /* child exited normally? */
  if (!WIFEXITED(child_status)) goto leave_handler;

  child_finished_normally:

  do_check_time_pid = 0;

  /* calc execution time */
  if (!clock_gettime(CLOCK_REALTIME, &do_check_time_rt_tspec))
    do_check_time_runtime = ((1000000 * do_check_time_rt_tspec.tv_sec) + (do_check_time_rt_tspec.tv_nsec / 1000)) - do_check_time_runtime;
  p = sputlongs(knoerre_global_buf_cursor, do_check_time_runtime);

  /* did our child the execve()? */
  if (WEXITSTATUS(child_status) == 127) {
    p = stpcpy(p, "return value (execution failed?): ");
    p = sputlongs(p, WEXITSTATUS(child_status));
    p = stpcpy(p, ERROR_OPEN_STRING);
    goto errexit2;
  }

  /* child finished with error? */
  if (WEXITSTATUS(child_status) != 0) {
    p = stpcpy(p, "return value: ");
    p = sputlongs(p, WEXITSTATUS(child_status));
    p = stpcpy(p, ERROR_COMMON_VALUE_STRING);
    goto errexit2;
  }

#ifdef KNOERRED
  knoerre_global_buf_cursor = p;
  /* return p - knoerre_global_buf; */
  return;
#else
  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(0);
#endif

  leave_handler:
  return;

  errexit:
  p = sputlongs(knoerre_global_buf_cursor, do_check_time_runtime);
  p = stpcpy(p, ERROR_COMMON_TIMEOUT_STRING);

  errexit2:
#ifdef KNOERRED
  knoerre_global_buf_cursor = p;
  return p - knoerre_global_buf;
#else
  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(2);
#endif
}

/**
 * This function measures the execution time of a given command.
 *
 * This function is called by knoerre/knoerred main().
 * If we are called by knoerred then output is limited to two lines.
 * $Id: do_check_time.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @return size of data to be sent, 0 on error
 */
int do_check_time(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  char arg0[BUF_SIZE+10];

  /* catch a SIGCHILD of a well teminated child */
  if (signal(SIGCHLD, check_time_sig_handler) == SIG_ERR) goto error;

  do_check_time_rt_tspec.tv_sec = 0;
  do_check_time_rt_tspec.tv_nsec = 0;
  if (!clock_gettime(CLOCK_REALTIME, &do_check_time_rt_tspec))
    do_check_time_runtime = (1000000 * do_check_time_rt_tspec.tv_sec) + (do_check_time_rt_tspec.tv_nsec / 1000);
  else do_check_time_runtime = 0;

  if ((do_check_time_pid = fork()) < 0) {

    goto error;

  } else if (do_check_time_pid == 0) {

    /* child: execute given command */

    (void) signal(SIGCHLD, SIG_IGN);
    close(0); close(1); close(2);
    p = stpcpy(arg0, "/usr/bin/");
    p = stpcpy(p, arg1);
    --p; *p = 0;
    const char *argv[] = {arg0, 0};
    execve(arg0, (char*const*)argv, 0);
    _exit(127); /* child */
  }

  /* parent: sleep a short time while child maybe does his job */
  /* it will be interrupted by the signal */
  sleep(KNOERRE_COMMON_TIMEOUT - 2);
  /* this kill will not be executed if sighandler did reset pid on success */
  if (kill(do_check_time_pid, 9) == 0) sleep(1);
  /*
   * if kill was successful (child did still exist and did not exit) then 
   * sleep a short time while sighandler maybe does his job
   */

  error:
#ifndef KNOERRED
  check_error_exit(0);
#endif
  return 0;
}
#endif
