#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_UPTIME

/**
 * This function gets disk usage stats and writes it as http data out.
 *
 * This function is called by knoerre/knoerred main().
 * It returns an "inverted" uptime in seconds. The value returned is
 * INVERSIONLEVEL - uptime or 0 if the value would be negative. The
 * inversionlevel may be specified by the key string, i.e. uptime/3600.
 * If no inversionlevel was specified then a default of 86400 will be
 * used.
 * If we are called by knoerred then output is limited to two lines.
 * $Id: do_check_uptime.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @return size of data to be sent, 0 on error
 */

/*
config KNOERRE_CHECK_UPTIME
    bool "check_uptime"
    default y
    depends on KNOERRE_PROC_ACCESS
    help
      This knoerre module offers checking of uptime for low or high values.
      It needs PROC_ACCESS. More information offers man-page knoerre(1).

config KNOERRE_CHECK_UPTIME_KEY
    string "key string for check_uptime"
    default "uptime"
    depends on KNOERRE_CHECK_UPTIME
    help
      The key string for check_uptime as described in man-page knoerre(1).
*/

int do_check_uptime() {
  register char *p;
  int i;
  ssize_t s;
  uint64_t inversionlevel;
  uint64_t val64;

  p = knoerre_global_arg1 + sizeof CONFIG_KNOERRE_CHECK_UPTIME_KEY - 1;
  /**
   * was uptime inversed requested?
   * if no 'I'nversion then just return the uptime in seconds
   */
  if (*p == 'I') {
    ++p;
    inversionlevel = 86400;
    if (*p == '/') {
      ++p;
      p = str2uint64(p, &inversionlevel);
      if (!p) goto error;
    } else {
      *p = 0;
    }
  } else {
    ++p;
    if (*p) goto error;
    inversionlevel = 0;
  }
  /* our output (est. max. 120) plus key must fit in buffer */
  if ((p - knoerre_global_arg1 + 120) > knoerre_global_bufsize) goto error;

  i = open("/proc/uptime", O_RDONLY);
  if (__unlikely(i < 0)) goto error;

  /* start output at this cursor, see knoerre.h */
  p = knoerre_global_buf_cursor;

  /* copy key (argv[1]) to output buffer */
  p = stpcpy(p, knoerre_global_arg1);
  if (*(p-1) != ' ') {
    *p = ' ';
    ++p;
  }

  /* avoid buffer overflow, read up to bufsize minus small offset */
  s = read(i, p, knoerre_global_bufsize - 100);
  close(i);
  if (__unlikely(s < 2)) goto error;
  p[s] = 0;

  val64 = 0;
  if (str2uint64(p, &val64)) {
    if (inversionlevel) {
      if (val64 >= inversionlevel) val64 = 0;
      else val64 = inversionlevel - val64;
    }
  }

  /* last line */
  p = sputlongs(p + s, val64);
#ifdef KNOERRED
  return p - knoerre_global_buf;
#else
  write1(knoerre_global_buf, p - knoerre_global_buf);
  _exit(0);
#endif

  error:
#ifndef KNOERRED
  check_error_exit(0);
#endif
  return 0;
}
#endif
