#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_WC_L

#ifdef NULLDEF
static void *kn_memchr(const void *s, int i, size_t n) {
  register const char *p = s;
  register char c = (char) i;
  register const char *t = p + n;

  for(;;) {
    if (p == t) return 0; if (*p == c) return (void *) p; ++p;
    if (p == t) return 0; if (*p == c) return (void *) p; ++p;
    if (p == t) return 0; if (*p == c) return (void *) p; ++p;
    if (p == t) return 0; if (*p == c) return (void *) p; ++p;
    if (p == t) return 0; if (*p == c) return (void *) p; ++p;
    if (p == t) return 0; if (*p == c) return (void *) p; ++p;
    if (p == t) return 0; if (*p == c) return (void *) p; ++p;
    if (p == t) return 0; if (*p == c) return (void *) p; ++p;
  }
  return 0;
}
#endif

/**
 * This function ...
 *
 * This function counts newlines in files like "wc -l".
 *
 * This function is called by knoerre/knoerred main().
 * $Id: do_check_wc-l.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param buf The http data buf for returning data.
 * @param buflen The size of the http data buf.
 * @param bufp The pointer to char after http-header in buf.
 * @param arg1 The file/directory which lines to count
 */

/*
config KNOERRE_CHECK_WC_L
    bool "check_wc-l"
    default y
    help
      This knoerre module offers counting of newlines in a file like "wc -l" does.
      More information in man-page knoerre(1).

config KNOERRE_CHECK_WC_L_KEY
    string "key string for check_wc-l"
    default "wc-l/"
    depends on KNOERRE_CHECK_WC_L
    help
      The key string for check_wc-l as described in man-page knoerre(1).
*/

int do_check_wcl(register char *buf, const int buflen, char *bufp, char *arg1) {
  register char *p;
  register uint32_t linecount;
  register char *u;
  ssize_t rb;
  loff_t lo;
  char pathname[PATHNAME_SIZE+1];
  char rbuf[BUF_SIZE+1];
  int fd;
  int ret;


  ret = 127;
  p = arg1;
  p[strlen(p)-1] = '\0';
  stpcpy(pathname, p);
  p = pathname;
  linecount = 0;


  fd = open(pathname, O_RDONLY);
  if (fd < 0) { ret=1; goto error; }

  for(;;) {
    p = rbuf;
    rb = read(fd, p, BUF_SIZE);
    if (__unlikely(rb < 0)) { ret=4; goto error; }
    if (__unlikely(rb == 0)) break;
    u = p + rb;
    *u = '\0';
    
    /* TODO: use custom memchr(), diet memchr() does only "byte-scan", gnu-memchr() does "dw-scan" */
#ifdef NULLDEF
    while ((p = memchr(p, '\n', u - p))) {
      ++linecount; ++p;
    }
    /* better with unroll: */
    for(;;) {
      p = kn_memchr(p, '\n', u - p); if (!p) break; ++linecount; ++p;
      p = kn_memchr(p, '\n', u - p); if (!p) break; ++linecount; ++p;
      p = kn_memchr(p, '\n', u - p); if (!p) break; ++linecount; ++p;
      p = kn_memchr(p, '\n', u - p); if (!p) break; ++linecount; ++p;
    }
    if (p == t) return 0; if (*p == c) return (void *) p; ++p;
#endif
    for(;;) {
      if (p == u) break; if (*p == '\n') ++linecount; ++p;
      if (p == u) break; if (*p == '\n') ++linecount; ++p;
      if (p == u) break; if (*p == '\n') ++linecount; ++p;
      if (p == u) break; if (*p == '\n') ++linecount; ++p;
      if (p == u) break; if (*p == '\n') ++linecount; ++p;
      if (p == u) break; if (*p == '\n') ++linecount; ++p;
      if (p == u) break; if (*p == '\n') ++linecount; ++p;
      if (p == u) break; if (*p == '\n') ++linecount; ++p;
    }

  }

  lo = lseek(fd, 0, SEEK_CUR);
  lo -= CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS; if (lo < 0) lo = 0;
  lseek(fd, lo, SEEK_SET);
  p = bufp;
  rb = read(fd, p, CONFIG_KNOERRE_LEN_OF_BADLINE_IN_NAGIOS);
  close(fd);
  if (rb > 0) {
    p[rb] = 0;
    /* for(;;) { if (!*p) break; if (*p == '\n') break; *p = ' '; ++p; } */
    for(;;) { if (!*p) break; if (*p == '\n') *p = ' '; ++p; }
    --p;
    *p = '\n'; ++p;
  }

  p = sputlongs(p, linecount);
#ifdef KNOERRED
  return p - buf;
#endif
  write1(buf, p-buf);
  _exit(0);

  error:
  if (fd >= 0) close(fd);
  p = stpcpy(bufp, pathname);
  *p++ = '\n';
  p = stpcpy(p, ERROR_COMMON_VALUE_STRING);
#ifdef KNOERRED
  return p - buf;
#endif
  write1(buf, p-buf);
  _exit(ret);
}
#endif
