#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_MAILQSIZE

static const char sd_active[] = "active/";
static const char sd_deferred[] = "deferred/";
static const char sd_maildrop[] = "maildrop/";
static const char sd_incoming[] = "incoming/";

/**
 * The postfix spooldir-path used by do_postfix_mailqsize().
 */
static const char *postfix_spooldir=CONFIG_KNOERRE_CHECK_MAILQSIZE_SPOOLDIR;

static const char *postfix_spooldir_subsubdirs[] = {
  "0/", "1/", "2/", "3/", "4/", "5/", "6/", "7/",
  "8/", "9/", "A/", "B/", "C/", "D/", "E/", "F/",
  NULL
};

/**
 * This function gets the size of postfix mailqueue and writes it as http data out.
 *
 * This function is called by knoerre/knoerred main().
 * If we are called by knoerred then output is limited to two lines.
 * $Id: do_postfix_mailqsize.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 */

/*
config KNOERRE_CHECK_MAILQSIZE
    bool "check_mailqsize"
    default y
    help
      This knoerre module offers shows the size of a postfix mailqueue.
      For more information see man-page knoerre(1). If unsure say "Y".

config KNOERRE_CHECK_MAILQSIZE_KEY
    string "key string for check_mailqsize"
    default "mailqsize"
    depends on KNOERRE_CHECK_MAILQSIZE
    help
      The key string for check_mailqsize as described in man-page knoerre(1).

config KNOERRE_CHECK_MAILQSIZE_SPOOLDIR
    string "spooldir path for check_mailqsize"
    default "/var/spool/postfix/"
    depends on KNOERRE_CHECK_MAILQSIZE
    help
      This is the path of the spooldir where this check counts mails.
      If subdirectories exist in spooldir they will also examined.
*/

int do_postfix_mailqsize() {
  register uint32_t countsub;
  register uint32_t countsubsub;
  uint32_t count;
  DIR *dptr;
  const char **ssd;
  const char **sd;
  int ret;
  char d[256];
  char *sp;
  char *spd;
  const char *postfix_spooldir_subdirs[] = { 0, 0, 0, 0, 0 };
  int dirs_given;
  int max_mode;
  uint32_t max_count;

  max_mode = 0;
  max_count = 0;
  ret = 3;
  if (*knoerre_global_arg1 == ' ') {
    postfix_spooldir_subdirs[0] = sd_active;
    postfix_spooldir_subdirs[1] = sd_deferred;
    dirs_given = 0;
  } else {
    if (*knoerre_global_arg1 != '/') goto error;
    dirs_given = 1;
    ++knoerre_global_arg1;
    countsub = count = 0;
    for(;;) {
      if (countsub == 5) goto error;
      switch (knoerre_global_arg1[count]) {
        case 'a': postfix_spooldir_subdirs[countsub] = sd_active; ++countsub; break;
        case 'd': postfix_spooldir_subdirs[countsub] = sd_deferred; ++countsub; break;
        case 'm': postfix_spooldir_subdirs[countsub] = sd_maildrop; ++countsub; break;
        case 'i': postfix_spooldir_subdirs[countsub] = sd_incoming; ++countsub; break;
        case 'M': max_mode = 1; break;
        case ' ': goto subdirsbuild;
        default: goto error; break;
      }
      ++count;
    }
  }
  subdirsbuild:
  if (postfix_spooldir_subdirs[0] == 0) check_error_exit(4);
  ret = countsub = count = 0;
  sp = stpcpy(d, postfix_spooldir);

  for (sd = postfix_spooldir_subdirs; (*sd) != NULL; ++sd) {

    countsub = 0;

    spd = stpcpy(sp, *sd);
    if (dirs_given) { knoerre_global_buf_cursor = stpcpy(knoerre_global_buf_cursor, *sd) - 1; *knoerre_global_buf_cursor = '='; ++knoerre_global_buf_cursor; }
    dptr = opendir(d);
    if (!dptr) { ret = 99999999; goto error; }
    for (; readdir(dptr) != NULL; ++countsub);
    closedir(dptr);
    if (countsub > 1) countsub -= 2; /* . and .. dirs */

    for (ssd = postfix_spooldir_subsubdirs; (*ssd) != NULL; ++ssd) {

      stpcpy(spd, *ssd);
      dptr = opendir(d);
      /* if (!dptr) { ret=1; goto out; } */
      if (!dptr) continue;
      if (countsub > 0) countsub--;
      countsubsub = 0;
      for (; readdir(dptr) != NULL; ++countsubsub);
      if (countsubsub > 1) countsubsub-=2; /* . and .. dirs */
      countsub += countsubsub;
      closedir(dptr);
    }

    if (dirs_given) { knoerre_global_buf_cursor = sputlongs(knoerre_global_buf_cursor, countsub) - 1; *knoerre_global_buf_cursor = ','; ++knoerre_global_buf_cursor; }
    if (max_mode && (countsub > max_count)) max_count = countsub;
    count += countsub;

  }

  if (max_mode) count = max_count;

  if (ret) goto error;

  sp = knoerre_global_buf_cursor;
  if (dirs_given) {
    --sp;
    if (*sp != ',') ++sp;
    else { *sp = '\n'; ++sp; }
  }
  sp = sputlongs(sp, count);

#ifdef KNOERRED
  return sp - knoerre_global_buf;
#else
  write1(knoerre_global_buf, sp - knoerre_global_buf);
  _exit(0);
#endif

  error:
#ifndef KNOERRED
  check_error_exit(ret);
#endif
  return 0;
}
#endif
