#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_MOUNTS
#ifndef CONFIG_KNOERRE_GETFSSTATS
#define CONFIG_KNOERRE_GETFSSTATS
#endif
#endif
#ifdef CONFIG_KNOERRE_CHECK_DISKUSAGELOCAL
#ifndef CONFIG_KNOERRE_GETFSSTATS
#define CONFIG_KNOERRE_GETFSSTATS
#endif
#endif
#ifdef CONFIG_KNOERRE_GETFSSTATS

uint16_t get_all_local_fs_stats(FS_STAT *top1stat, FS_STAT *top2stat) {
  register char *p;
  FS_STAT tempstat;
  char mountlist[BUF_SIZE+1];
  int32_t s;
  int i;

  /* read in list of mounts */
  i = open("/proc/mounts", O_RDONLY);
  if (__unlikely(i < 0))
#ifdef KNOERRED
    return 0;
#else
    goto error;
#endif
  p = mountlist;
  s = (int32_t) read(i, p, BUF_SIZE);
  close(i);
  if (__unlikely(s < 2))
#ifdef KNOERRED
    return 0;
#else
    goto error;
#endif
  p[s] = '\0';

  tempstat.percent_used = top1stat->percent_used = top2stat->percent_used = 0;
  tempstat.path = top1stat->path = top2stat->path = 0;
  tempstat.used = top1stat->used = top2stat->used = 0;
  tempstat.free = top1stat->free = top2stat->free = 0;
  tempstat.all = top1stat->all = top2stat->all = 0;

  /* loop through lines */
  for (;;) {

    if (!*p) break;
    s = field_index(p, 1);
    if (s == -1) break;
    p += s;
    tempstat.path = p;
    s = field_index(p, 1);
    if (s == -1) break;
    p += s - 1;
    *p = 0; ++p;
    s = 0;

    for (;;) {

      if (!localrwfs[s]) break;
      if (!strncmp(p, localrwfs[s], strlen(localrwfs[s]))) {
        /* fs match, get stats */
        if (get_fs_stats(&tempstat) == -1) goto next_fs;
        if ((tempstat.percent_used >= top1stat->percent_used) || (top1stat->path == 0)) {
          top2stat->percent_used = top1stat->percent_used;
          top2stat->path = top1stat->path;
          top2stat->used = top1stat->used;
          top2stat->free = top1stat->free;
          top2stat->all = top1stat->all;
          top1stat->percent_used = tempstat.percent_used;
          top1stat->path = tempstat.path;
          top1stat->used = tempstat.used;
          top1stat->free = tempstat.free;
          top1stat->all = tempstat.all;
        } else {
          if ((tempstat.percent_used >= top2stat->percent_used) || (top2stat->path == 0)) {
            top2stat->percent_used = tempstat.percent_used;
            top2stat->path = tempstat.path;
            top2stat->used = tempstat.used;
            top2stat->free = tempstat.free;
            top2stat->all = tempstat.all;
          }
        }
      }
      next_fs: ++s;

    }

    p = strchr(p, '\n');
    if (!p) break;
    ++p;

  }

  return top1stat->percent_used;

#ifndef KNOERRED
  error:
  check_error_exit(0);
#endif
  return 0;
}
#endif
