#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_MOUNTS
#ifndef CONFIG_KNOERRE_GETFSSTATS
#define CONFIG_KNOERRE_GETFSSTATS
#endif
#endif
#ifdef CONFIG_KNOERRE_CHECK_DISKUSAGELOCAL
#ifndef CONFIG_KNOERRE_GETFSSTATS
#define CONFIG_KNOERRE_GETFSSTATS
#endif
#endif
#ifdef CONFIG_KNOERRE_GETFSSTATS

/**
 * List of local rw fs types to be recognized
 */
const char const *localrwfs[] = {
  "ext4",
  "devtmpfs",
  "tmpfs",
  "btrfs",
  "ext3",
  "gfs2",
  "ext2",
  "xfs",
  "acfs",
  "reiserfs",
  "jfs",
  "hfs",
  "hfsplus",
  "jffs2",
/*  "squashfs",*/
  0
};

int16_t get_fs_stats(FS_STAT *fs_stat) {
  register int64_t s;
  uint64_t avail, all;
  int64_t all100;
  struct statfs stat_buf;

  if (statfs(fs_stat->path, &stat_buf) != 0) return -1;

  avail = stat_buf.f_bavail; /* avail for non-root */
  s = stat_buf.f_bfree - avail; /* (free for root) - avail = reserved */
  all = stat_buf.f_blocks - s; /* all for non-root */

  /* percentage, rounding like df does */
  if (all > 50000) {
    /* fs normal sizes */
    all100 = all / 10000;
    s = all - avail; /* used */
    if (s < 0) {
      s = 0;
    } else {
      s /= 100;
      s = (s / all100) + 1;
    }
  } else {
    /* fs extreme small sizes */
    s = all - avail; /* used */
    if (s < 0) {
      s = 0;
    } else {
      s *= 100;
      if (all != 0) s = (s / all) + 1; else s = 100;
    }
  }
  if (s > 100) s = 100;
  fs_stat->percent_used = s;

  avail = avail / 262144 * stat_buf.f_bsize; /* 4 mega blocks * blocksize */
  all = all / 262144 * stat_buf.f_bsize; /* 4 mega blocks * blocksize */

  fs_stat->all = all / 4096; /* GB */
  fs_stat->used = (all - avail) / 4096; /* GB */
  fs_stat->free = avail / 4096; /* GB */

  return fs_stat->percent_used;
}
#endif
