#include "knoerred.h"

/**
 * This function converts an http request to a key.
 *
 * It reads data one time from the given file
 * descriptor. If the data doesn't contain a valid
 * http request for knoerre then return with a value
 * <= 0. The key is the URL without the leading slash.
 * It will be stored in the given key buffer.
 * This function is used by knoerre and knoerrehd.
 *
 * $Id: http_to_key.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param readsock The fd to be read from
 * @param key The buffer where the key will be stored
 * @param keysize The size of the key buffer
 * @return keylen or <=0 on error
 */
int http_to_key(int readsock, char key[], int keysize) {
  register char *p;
  char *u;
  char http_buf[MAX_HTTP_REQUEST_SIZE+1];
  int i;
  char *request;
  char c;

  p = http_buf;
  i = (int)read(readsock, p, MAX_HTTP_REQUEST_SIZE);
  if (__unlikely(i<15))
    return 0; /* data too short for HTTP-request, don't answer it */

  if (__unlikely(strncmp(p, "GET /", 5))) return 0;
  p += 5;

  if (__unlikely(*p == ' ')) return -1;
  request = p;
  ++p;
  for (c=' ';;) {
    if (!*p) return -1;
    if (*p == ' ') { u = ++p; break; };
    if ((c == '.') && (*p == '.')) return -1;
    c = *p++;
  }
  if (__unlikely(strncmp(p, "HTTP/1.", 7))) return -1;
  p += 7;
  if (__unlikely(*p != '1')) {
    if (*p != '0') return -1;
  }
  *u = '\0';
  i = u - request;
  if (__unlikely(i >= keysize)) return -1;
  p = stpcpy(key, request);
  return i;
}
