#include "knoerred.h"

/**
 * $Id: interval_measure.c 691 2017-06-04 23:12:04Z fwbergm $
 */

static int count_overruns(uint64_t values[], int cursor) {
  int count=0;
  int end_cursor;
  uint64_t last_val;

  end_cursor = cursor;

  ++cursor;
  if (cursor > MEASURE_INTERVAL_COUNT) cursor=0;

  for (;;) {
    last_val = values[cursor];
    ++cursor;
    if (cursor > MEASURE_INTERVAL_COUNT) cursor=0;
    if (last_val > values[cursor]) ++count;
    if (cursor == end_cursor) break;
  }

  return count;
}


static uint64_t diff_values(uint64_t new, uint64_t old, int overrun_add) {
    uint64_t diff;

    /* we lose on 64 bit overrun but this will most likely never happen :-) */
    if (overrun_add) {
      new += (overrun_add * 4294967296ULL);
    } else {
      if (new >= old) return new - old;
      /* try adding 2^32 until new is bigger than old */
      for (;;) {
        /* new += UINT32_MAX + 1; */
        new += 4294967296ULL;
        if (new >= old) break;
      }
    }
    diff = new - old;
    return diff;
}


// #ifdef __dietlibc__
// int *interval_measure(void *notused) {
// #else
int interval_measure(void *notused) {
// #endif
  uint64_t user, system, idle, wait, irq, softirq, total, ctxt,
          nettraf0recv, nettraf0send, nettraf1recv, nettraf1send;
  uint64_t interval;
  int i,j;
  int tmpcursor;

  measure_cursor = 0;
  for (current_values_cursor=0;;) {
    cpu_user_percent[current_values_cursor] =
      cpu_system_percent[current_values_cursor] =
      cpu_idle_percent[current_values_cursor] =
      cpu_wait_percent[current_values_cursor] =
      cpu_irq_percent[current_values_cursor] =
      cpu_softirq_percent[current_values_cursor] =
      ctxt_switches_persecond[current_values_cursor] =
      nettraf0_kb_persecond[current_values_cursor] =
      nettraf1_kb_persecond[current_values_cursor] =
      0;
    loadavg[current_values_cursor][0] = 
      swaps[current_values_cursor][0] =
      hostname[current_values_cursor][0] =
      multiresults[current_values_cursor][0] =
      0;
    statfs_root_err[current_values_cursor] = -1;
    fsstat_top[current_values_cursor].all = 0;
    netlinksdown[current_values_cursor] = 0;
    if (current_values_cursor == 1) break;
    ++current_values_cursor;
  }

  for (j=0; j <= MEASURE_INTERVAL_COUNT; ++j) {
    record_measure_values();
    usleep(10000);
  }

  usleep(400000);

  for (;;) {

    /* record all values which must be diff'd */
    record_measure_values();

    interval = measure_timestamp[measure_cursor] - measure_timestamp[measure_oldest];

    tmpcursor = !current_values_cursor;

    /* diff the measure-values */

    user = diff_values(measure_cpu_user[measure_cursor], measure_cpu_user[measure_oldest],0);
    system = diff_values(measure_cpu_system[measure_cursor], measure_cpu_system[measure_oldest],0);
    idle = diff_values(measure_cpu_idle[measure_cursor], measure_cpu_idle[measure_oldest],0);
    wait = diff_values(measure_cpu_wait[measure_cursor], measure_cpu_wait[measure_oldest],0);
    irq = diff_values(measure_cpu_irq[measure_cursor], measure_cpu_irq[measure_oldest],0);
    softirq = diff_values(measure_cpu_softirq[measure_cursor], measure_cpu_softirq[measure_oldest],0);
    total = user + system + idle + wait + irq + softirq;
    if (!total) total = 1; /* unlikely */

    ctxt = diff_values(measure_ctxt[measure_cursor], measure_ctxt[measure_oldest],0);

    i = count_overruns(measure_nettraf0recv, measure_cursor);
    nettraf0recv = diff_values(measure_nettraf0recv[measure_cursor], measure_nettraf0recv[measure_oldest],i);
    i = count_overruns(measure_nettraf0send, measure_cursor);
    nettraf0send = diff_values(measure_nettraf0send[measure_cursor], measure_nettraf0send[measure_oldest],i);
    i = count_overruns(measure_nettraf1recv, measure_cursor);
    nettraf1recv = diff_values(measure_nettraf1recv[measure_cursor], measure_nettraf1recv[measure_oldest],i);
    i = count_overruns(measure_nettraf1send, measure_cursor);
    nettraf1send = diff_values(measure_nettraf1send[measure_cursor], measure_nettraf1send[measure_oldest],i);

    user = user * 100 / total; if (user > 100) user = 100;
    system = system * 100 / total; if (system > 100) system = 100;
    idle = idle * 100 / total; if (idle > 100) idle = 100;
    wait = wait * 100 / total; if (wait > 100) wait = 100;
    irq = irq * 100 / total; if (irq > 100) irq = 100;
    softirq = softirq * 100 / total; if (softirq > 100) softirq = 100;
    cpu_user_percent[tmpcursor] = user;
    cpu_system_percent[tmpcursor] = system;
    cpu_idle_percent[tmpcursor] = idle;
    cpu_wait_percent[tmpcursor] = wait;
    cpu_irq_percent[tmpcursor] = irq;
    cpu_softirq_percent[tmpcursor] = softirq;
    ctxt_switches_persecond[tmpcursor] = ctxt * 1000 / interval;
    nettraf0_kb_persecond[tmpcursor] = ((nettraf0recv + nettraf0send) / 1024) * 1000 / interval;
    nettraf1_kb_persecond[tmpcursor] = ((nettraf1recv + nettraf1send) / 1024) * 1000 / interval;

    /* diffs done, now record values which will be "unmodified" */
    record_multi_values(tmpcursor);

    /* update the cursor to point to the latest values */
    current_values_cursor = tmpcursor;

    sleep(MEASURE_INTERVAL_LENGTH);

  }
  return 0;
}
