#ifdef KNOERREHD
#include "knoerrehd.h"
#else
#include "knoerred.h"
#endif
#ifdef CONFIG_KNOERREHD
#undef CONFIG_KNOERRED
#define CONFIG_KNOERRED
#endif
#ifdef CONFIG_KNOERRED

/**
 * $Id: knoerred.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * knoerre[h]d - udp or http (KNOERREHD) standalone version of knoerre
 *
 * Compile it with: diet gcc -O2 -Wall ... ; sstrip knoerre
 * License is GNU GPL v2.
 *
 * Frank Bergmann, http://www.tuxad.com
 *
 */

#ifdef KNOERREHD
/**
 * This is the default type 200 http header to be returned.
 *
 * After the header a line with the remote check value will be sent.
 * This value will be added by any of the internal do_check_*() functions.
 */
const char str200[]="HTTP/1.0 200 OK\r\nServer: knoerrehd/"VERSION"\r\nContent-Type: text/plain\r\n\r\n";

static const char version_string[] = "version_string: knoerrehd, $Id: knoerred.c 691 2017-06-04 23:12:04Z fwbergm $\n";
#else
static const char version_string[] = "version_string: knoerred, $Id: knoerred.c 691 2017-06-04 23:12:04Z fwbergm $\n";
#endif

char *knoerre_global_buf;
const int knoerre_global_bufsize = DAEMON_NET_BUFSIZE;
char *knoerre_global_buf_cursor;
char *knoerre_global_arg1;

#define MAX_ALLOWED_IP_ADDRESSES_CLIENTS 20

/*
int count=48;
*/
char stack[STACKSIZE];

int measure_cursor;
int measure_oldest;
uint64_t measure_timestamp[MEASURE_INTERVAL_COUNT+1];
uint64_t measure_cpu_user[MEASURE_INTERVAL_COUNT+1];
uint64_t measure_cpu_system[MEASURE_INTERVAL_COUNT+1];
uint64_t measure_cpu_idle[MEASURE_INTERVAL_COUNT+1];
uint64_t measure_cpu_wait[MEASURE_INTERVAL_COUNT+1];
uint64_t measure_cpu_irq[MEASURE_INTERVAL_COUNT+1];
uint64_t measure_cpu_softirq[MEASURE_INTERVAL_COUNT+1];
uint64_t measure_ctxt[MEASURE_INTERVAL_COUNT+1];
uint64_t measure_nettraf0recv[MEASURE_INTERVAL_COUNT+1];
uint64_t measure_nettraf0send[MEASURE_INTERVAL_COUNT+1];
uint64_t measure_nettraf1recv[MEASURE_INTERVAL_COUNT+1];
uint64_t measure_nettraf1send[MEASURE_INTERVAL_COUNT+1];
int cpu_user_percent[2];
int cpu_system_percent[2];
int cpu_idle_percent[2];
int cpu_wait_percent[2];
int cpu_irq_percent[2];
int cpu_softirq_percent[2];
uint32_t ctxt_switches_persecond[2];
uint32_t nettraf0_kb_persecond[2];
uint32_t nettraf1_kb_persecond[2];
char hostname[2][STRING_SIZE+1];
char loadavg[2][STRING_SIZE+1];
char swaps[2][STRING_SIZE+1];
struct statfs statfs_root[2];
int statfs_root_err[2];
FS_STAT fsstat_top[2];
int netlinksdown[2];
char multiresults[2][DAEMON_NET_BUFSIZE+1];

int current_values_cursor;



int main(int argc, char *argv[]) {

  register char *p;
  int tmpint;
  int sockfd;
#ifdef KNOERREHD
  int connsock;
#else
  ssize_t bytecount;
#endif
  struct sockaddr_in servin;
  struct sockaddr_in peerin;
  socklen_t peerin_len;
  char net_buf_rcv[DAEMON_NET_BUFSIZE+1];
  char net_buf_send[DAEMON_NET_BUFSIZE+1];
  char *child_stack;
  uint32_t flags;
  in_addr_t client_in_addr_allowed[MAX_ALLOWED_IP_ADDRESSES_CLIENTS + 1];
  uint16_t net_servport;
  uint64_t value;

  if ((argc >= 2) && (argv[1][0] == '-') && (argv[1][1] == 'd')) {
    pid_t pid;
    if ((pid=fork())<0) {
      _exit(1);
    } else if (pid!=0) {
      _exit(0);
    }
    setsid();
    umask(027);
    chdirv("/");
    close(0); close(1); close(2);
    argv[1] = argv[0];
    execve(argv[1],argv+1,0);
    _exit(2); /* dummy: init doesn't care; */
  }

  if (argc < 2) {
    net_servport = 80;
    write1(version_string, sizeof version_string - 1);
  } else {

    if (!str2uint64(argv[1], &value)) _exit(1);
    net_servport = (uint16_t) value;

    if (argc > 2) {

      tmpint = 0;
      for (;;) {
        client_in_addr_allowed[tmpint] = inet_addr(argv[tmpint+2]);
        ++tmpint;
        client_in_addr_allowed[tmpint] = -1;
        if ((tmpint + 2) == argc) break;
        if (tmpint == MAX_ALLOWED_IP_ADDRESSES_CLIENTS) _exit(2);
      }

    }
  }

#ifdef KNOERREHD
  connsock = -1;
  sockfd = socket(PF_INET, SOCK_STREAM, 0);
#else
  sockfd = socket(PF_INET, SOCK_DGRAM, 0);
#endif
  if (sockfd == -1) goto err_exit;

  tmpint = fcntl(sockfd, F_GETFL);
  tmpint |= O_NONBLOCK;
  tmpint = fcntl(sockfd, F_SETFL, tmpint);
  if (tmpint) goto err_close_exit;

  tmpint = 1;
  tmpint = setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &tmpint, sizeof tmpint);
  if (tmpint) goto err_close_exit;

  /* bind */
  memset(&servin, 0, sizeof servin);
  servin.sin_family = AF_INET;
  servin.sin_port = htons(net_servport);
  tmpint = bind(sockfd, (struct sockaddr *) &servin, sizeof servin);
  if (tmpint) goto err_close_exit;

#ifdef KNOERREHD
  tmpint = listen(sockfd, KNOERREHD_BACKLOG);
  if (tmpint) goto err_close_exit;
#endif

  tmpint = fcntl(sockfd, F_GETFL);
  tmpint ^= O_NONBLOCK;
  tmpint = fcntl(sockfd, F_SETFL, tmpint);
  if (tmpint) goto err_close_exit;
  
  tmpint = 65536;
  tmpint = setsockopt(sockfd, SOL_SOCKET, SO_RCVBUF, &tmpint, sizeof tmpint);
  if (tmpint) goto err_close_exit;

  peerin_len = sizeof peerin;
  memset(&peerin, 0, peerin_len);
  knoerre_global_buf = net_buf_send;

  child_stack = stack + STACKSIZE;
  flags =
    CLONE_FILES |
    CLONE_FS |
    CLONE_IO |
    CLONE_SIGHAND |
    CLONE_SYSVSEM |
    CLONE_THREAD |
    CLONE_VM;
  clone(interval_measure, child_stack, flags, 0);
  /* NOTE: dietlibc defines 'void *(*)(void *)' but glibc defines 'int (*)(void *)' as first arg */

#ifdef KNOERREHD
  goto next_request;

  close_conn:
  (void) close(connsock);
#endif

  /* read loop */
  next_request:
  p = net_buf_rcv;

#ifdef KNOERREHD
  while ((connsock = accept(sockfd, (struct sockaddr *) &peerin, &peerin_len)) == -1);
  if (http_to_key(connsock, p, DAEMON_NET_BUFSIZE) <= 0) goto close_conn;
#else
  bytecount = recvfrom(sockfd, p, sizeof net_buf_rcv - 1, 0, (struct sockaddr *) &peerin, &peerin_len);
  if (bytecount < 1) goto err_close_exit;
  p[bytecount] = 0;
  if (p[bytecount-1] != '\n') goto next_request;
  p[bytecount-1] = ' '; /* knoerre compatibility */
#endif

  /* client allowed? */
  for(tmpint = 0;;++tmpint) {
    if (client_in_addr_allowed[tmpint] == peerin.sin_addr.s_addr) goto request_allowed;
    if (client_in_addr_allowed[tmpint] == -1)
#ifdef KNOERREHD
      goto close_conn;
#else
      goto next_request;
#endif
  }

  /* sendto */
  request_allowed:

#ifdef KNOERREHD
  knoerre_global_buf_cursor = stpcpy(net_buf_send, str200);
#else
  knoerre_global_buf_cursor = net_buf_send;
#endif

  if (!strncmp(p, CONFIG_KNOERRE_CHECK_LOAD1_KEY, sizeof CONFIG_KNOERRE_CHECK_LOAD1_KEY)) {
    tmpint = do_check_load1(0);
    goto check_answer;
  }
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_LOAD5_KEY, sizeof CONFIG_KNOERRE_CHECK_LOAD5_KEY)) {
    tmpint = do_check_load5(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor);
    goto check_answer;
  }
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_LOAD15_KEY, sizeof CONFIG_KNOERRE_CHECK_LOAD15_KEY)) {
    tmpint = do_check_load15(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor);
    goto check_answer;
  }

#ifdef CONFIG_KNOERRE_CHECK_LOADMULTI
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_LOADMULTI_KEY, sizeof CONFIG_KNOERRE_CHECK_LOADMULTI_KEY - 1)) {
    tmpint = do_check_loadmulti(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, p + sizeof CONFIG_KNOERRE_CHECK_LOADMULTI_KEY - 2);
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_CPU
  /* CPU percentage for user, nice, ... */
  if (!strncmp(p, "cpu", 3)) {
    knoerre_global_arg1 = p;
    p += 3;
    if (*p == 'u') {
      ++p;
      if (*p != ' ')
#ifdef KNOERREHD
        goto close_conn;
#else
        goto next_request;
#endif
      tmpint = do_check_cpu(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, CPUUSERCURR, p);
      goto check_answer;
    }
    if (*p == 's') {
      ++p;
      if (*p != ' ')
#ifdef KNOERREHD
        goto close_conn;
#else
        goto next_request;
#endif
      tmpint = do_check_cpu(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, CPUSYSTEMCURR, p);
      goto check_answer;
    }
    if (*p == 'i') {
      ++p;
      if (*p != ' ')
#ifdef KNOERREHD
        goto close_conn;
#else
        goto next_request;
#endif
      tmpint = do_check_cpu(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, CPUIDLECURR, p);
      goto check_answer;
    }
    if (*p == 'w') {
      ++p;
      if (*p != ' ')
#ifdef KNOERREHD
        goto close_conn;
#else
        goto next_request;
#endif
      tmpint = do_check_cpu(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, CPUWAITCURR, p);
      goto check_answer;
    }
    if (*p == 'I') {
      ++p;
      if (*p != ' ')
#ifdef KNOERREHD
        goto close_conn;
#else
        goto next_request;
#endif
      tmpint = do_check_cpu(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, CPUIDLEINVERSECURR, p);
      goto check_answer;
    }
#ifdef KNOERREHD
    goto close_conn;
#else
    goto next_request;
#endif
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_CACHEDVALUE
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_CACHEDVALUE_KEY, sizeof CONFIG_KNOERRE_CHECK_CACHEDVALUE_KEY - 1)) {
    knoerre_global_arg1 = p + sizeof CONFIG_KNOERRE_CHECK_CACHEDVALUE_KEY - 1;
    tmpint = do_check_cachedvalue();
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_CAT
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_CAT_KEY, sizeof CONFIG_KNOERRE_CHECK_CAT_KEY - 1)) {
    knoerre_global_arg1 = p + sizeof CONFIG_KNOERRE_CHECK_CAT_KEY - 2;
    tmpint = do_check_cat();
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_DISKUSAGE
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_DISKUSAGE_KEY, sizeof CONFIG_KNOERRE_CHECK_DISKUSAGE_KEY - 1)) {
    knoerre_global_arg1 = p + sizeof CONFIG_KNOERRE_CHECK_DISKUSAGE_KEY - 2;
    tmpint = do_check_diskusage();
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_DISKUSAGELOCAL
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_DISKUSAGELOCAL_KEY, sizeof CONFIG_KNOERRE_CHECK_DISKUSAGELOCAL_KEY - 1)) {
    tmpint = do_check_diskusagelocal();
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_DISKINODES
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_DISKINODES_KEY, sizeof CONFIG_KNOERRE_CHECK_DISKINODES_KEY - 1)) {
    tmpint = do_check_diskinodes(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, p + sizeof CONFIG_KNOERRE_CHECK_DISKINODES_KEY - 2);
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_TIMEDIFF
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_TIMEDIFF_KEY, sizeof CONFIG_KNOERRE_CHECK_TIMEDIFF_KEY - 1)) {
    tmpint = do_check_timediff(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, p + sizeof CONFIG_KNOERRE_CHECK_TIMEDIFF_KEY - 1);
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_SWAPS
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_SWAPS_KEY, sizeof CONFIG_KNOERRE_CHECK_SWAPS_KEY - 2)) {
    tmpint = do_check_swaps(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor);
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_DMESG
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_DMESG_KEY, sizeof CONFIG_KNOERRE_CHECK_DMESG_KEY - 2)) {
    tmpint = do_check_dmesg(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor);
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_NETLINKSDOWN
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_NETLINKSDOWN_KEY, sizeof(CONFIG_KNOERRE_CHECK_NETLINKSDOWN_KEY) - 1)) {
    tmpint = do_check_netlinksdown();
    goto check_answer;
  }
#endif
#ifdef CONFIG_KNOERRE_CHECK_NETTRAF
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_NETTRAF_KEY, sizeof(CONFIG_KNOERRE_CHECK_NETTRAF_KEY) - 1)) {
    knoerre_global_arg1 = p + sizeof CONFIG_KNOERRE_CHECK_NETTRAF_KEY - 1;
    tmpint = do_check_nettraf();
    goto check_answer;
  }
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_NETTRAF_KEY, sizeof(CONFIG_KNOERRE_CHECK_NETTRAF_KEY) - 2)) {
    knoerre_global_arg1 = p + sizeof CONFIG_KNOERRE_CHECK_NETTRAF_KEY - 2;
    tmpint = do_check_nettraf();
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_PROCCOUNTTG
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_PROCCOUNTTG_KEY, sizeof CONFIG_KNOERRE_CHECK_PROCCOUNTTG_KEY - 1)) {
    tmpint = do_check_proccounttg(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor);
    goto check_answer;
  }
#endif
#ifdef CONFIG_KNOERRE_CHECK_PROCCOUNT
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_PROCCOUNT_KEY, sizeof CONFIG_KNOERRE_CHECK_PROCCOUNT_KEY - 2)) {
    tmpint = do_check_proccount(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor);
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_MAILQSIZE
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_MAILQSIZE_KEY, sizeof CONFIG_KNOERRE_CHECK_MAILQSIZE_KEY - 1)) {
    knoerre_global_arg1 = p + sizeof CONFIG_KNOERRE_CHECK_MAILQSIZE_KEY - 1;
    tmpint = do_postfix_mailqsize();
    goto check_answer;
  }
#endif
#ifdef CONFIG_KNOERRE_CHECK_TCP
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_TCP_KEY, sizeof CONFIG_KNOERRE_CHECK_TCP_KEY - 1)) {
    /*knoerre_global_arg1 = p + sizeof CONFIG_KNOERRE_CHECK_TCP_KEY - 1;*/
    knoerre_global_arg1 = p;
    tmpint = do_check_tcp();
    goto check_answer;
  }
#endif
#ifdef CONFIG_KNOERRE_CHECK_UPTIME
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_UPTIME_KEY, sizeof CONFIG_KNOERRE_CHECK_UPTIME_KEY - 1)) {
    /*knoerre_global_arg1 = p + sizeof CONFIG_KNOERRE_CHECK_UPTIME_KEY - 1;*/
    knoerre_global_arg1 = p;
    tmpint = do_check_uptime();
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_FILETIMESTAMP
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_FILETIMESTAMP_KEY, sizeof CONFIG_KNOERRE_CHECK_FILETIMESTAMP_KEY - 1)) {
    tmpint = do_check_filetimestamp(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, p + sizeof CONFIG_KNOERRE_CHECK_FILETIMESTAMP_KEY - 2);
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_PROCESS
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_PROCESS_KEY, sizeof CONFIG_KNOERRE_CHECK_PROCESS_KEY - 2)) {
    knoerre_global_arg1 = p + sizeof CONFIG_KNOERRE_CHECK_PROCESS_KEY - 2;
    tmpint = do_check_process();
    goto check_answer;
  }
#endif
#ifdef CONFIG_KNOERRE_CHECK_PROCESSD
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_PROCESSD_KEY, sizeof CONFIG_KNOERRE_CHECK_PROCESSD_KEY - 1)) {
    knoerre_global_arg1 = p + sizeof CONFIG_KNOERRE_CHECK_PROCESSD_KEY - 3;
    tmpint = do_check_process();
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_WC_L
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_WC_L_KEY, sizeof CONFIG_KNOERRE_CHECK_WC_L_KEY - 1)) {
    tmpint = do_check_wcl(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, p + sizeof CONFIG_KNOERRE_CHECK_WC_L_KEY - 2);
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_CMP
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_CMP_KEY, sizeof CONFIG_KNOERRE_CHECK_CMP_KEY - 1)) {
    tmpint = do_check_cmp(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, p + sizeof CONFIG_KNOERRE_CHECK_CMP_KEY - 1);
    goto check_answer;
  }
#endif

#ifdef CONFIG_KNOERRE_CHECK_CMDLINE
  if (!strncmp(p, CONFIG_KNOERRE_CHECK_CMDLINE_KEY, sizeof CONFIG_KNOERRE_CHECK_CMDLINE_KEY - 1)) {
    tmpint = do_check_cmdline(knoerre_global_buf, sizeof net_buf_send - 1, knoerre_global_buf_cursor, p + sizeof CONFIG_KNOERRE_CHECK_CMDLINE_KEY - 1);
    goto check_answer;
  }
#endif

#ifdef KNOERREHD
  goto close_conn;
#else
  goto next_request;
#endif

  check_answer:
#ifdef KNOERREHD
  if (!tmpint) goto close_conn; /* error */
  if (tmpint > sizeof net_buf_send - 1) goto close_conn;
#else
  if (!tmpint) goto next_request; /* error */
  if (tmpint > sizeof net_buf_send - 1) goto next_request;
#endif

  /* send answer */
#ifdef KNOERREHD
  tmpint = write(connsock, knoerre_global_buf, tmpint);
  goto close_conn;
#else
  tmpint = sendto(sockfd, knoerre_global_buf, tmpint, 0, (struct sockaddr *) &peerin, peerin_len);
  goto next_request;
#endif

  err_close_exit:
#ifdef KNOERREHD
  close(connsock);
#else
  close(sockfd);
#endif

  err_exit:
  write2("error\n", 6);
  exit(tmpint);
}
#endif
