#include "knoerred.h"
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed
#endif
#include <sys/time.h>

/**
 * This function is called periodically to record cpu jiffies.
 *
 * interval_measure() calls this function periodically to
 * record the cpu jiffies, context switches and more for
 * calculating CPU% for e.g. system or user.
 *
 * $Id: record_measure_values.c 691 2017-06-04 23:12:04Z fwbergm $
 */
void record_measure_values() {
  register char *p;
  int i;
  ssize_t s;
  uint64_t colval;
  uint64_t user, nice, system, idle, wait, irq, softirq, ctxt,
           nettraf0recv, nettraf0send, nettraf1recv, nettraf1send;
  char procfile[BUF_SIZE+1];
  int tmpcursor;
  struct timespec st_tspec;

  tmpcursor = measure_cursor + 1;
  if (tmpcursor > MEASURE_INTERVAL_COUNT) tmpcursor = 0;
  measure_oldest = tmpcursor + 1;
  if (measure_oldest > MEASURE_INTERVAL_COUNT) measure_oldest = 0;

  i = open("/proc/stat", O_RDONLY);
  if (i < 0) goto error_out;

  p = procfile;
  s = read(i, p, BUF_SIZE);
  st_tspec.tv_nsec = 0;
  st_tspec.tv_sec = 0;
  if (clock_gettime(CLOCK_REALTIME, &st_tspec) != 0) {
    st_tspec.tv_nsec = 0;
    st_tspec.tv_sec = time(0);
  }
  measure_timestamp[tmpcursor] = (((uint64_t)st_tspec.tv_sec) * 1000) + (st_tspec.tv_nsec / 1000000);
  close(i);
  if (s < 2) goto error_out;
  p[s] = '\0';

  if (*p != 'c') goto error_out;
  ++p;
  if (*p != 'p') goto error_out;
  ++p;
  if (*p != 'u') goto error_out;
  ++p;
  if (*p != ' ') goto error_out;
  ++p;

  user = nice = system = idle = wait = irq = softirq = ctxt =
         nettraf0recv = nettraf0send = nettraf1recv = nettraf1send = 0;

  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) goto error_out;
  p = str2uint64(p, &colval); if (!p) goto error_out;
  user = colval;

  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) goto error_out;
  p = str2uint64(p, &colval); if (!p) goto error_out;
  nice = colval;

  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) goto error_out;
  p = str2uint64(p, &colval); if (!p) goto error_out;
  system = colval;

  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) goto error_out;
  p = str2uint64(p, &colval); if (!p) goto error_out;
  idle = colval;

  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) goto error_out;
  if (*p == '\n') goto readcpudone; /* kernel 2.4 ? */
  p = str2uint64(p, &colval);
  if (p) wait = colval;

  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) goto error_out;
  if (*p == '\n') goto readcpudone; /* older kernel 2.6 ? */
  p = str2uint64(p, &colval);
  if (p) irq = colval;

  for(;;) { if (*p != ' ') break; p++; }
  if (!*p) goto error_out;
  if (*p == '\n') goto readcpudone; /* older kernel 2.6 ? */
  p = str2uint64(p, &colval);
  if (p) softirq = colval;

  /*
   * steal and guest and possible more ignored
   */

  readcpudone:

  i = str_index2(p, "\nctxt ");
  if (i == -1) goto error_out;
  p += i;
  p = str2uint64(p, &colval); if (!p) goto error_out;
  ctxt = colval;

  i = open("/proc/net/dev", O_RDONLY);
  if (i < 0) goto nettraferror;
  p = procfile;
  s = read(i, p, BUF_SIZE);
  close(i);
  if (s < 2) goto nettraferror;
  p[s] = 0;

  /*
    lo:118983402 1307291    0    0    0     0          0         0 118983402 1307291    0    0    0     0       0          0
  eth0:2104787106 223573269    2    0    0     1          0      4314 1922834560 217901029    0    0    0     0       0          0
  eth1:215615193 3016942    0    0    0     0          0      1938 84721398 1062218    0    0    0     0       0          0
  */
  p = strstr(p, " eth0:");
  if (!p) goto nettraferror;
  p += 6;
  for(;;++p) if (*p != ' ') break;
  /* p now points to receive bytes */
  p = str2uint64(p, &colval);
  if (!p) goto nettraferror;
  nettraf0recv = colval;
  p = str_field(p, 8, ' ', 1);
  /* p now points to transmit bytes */
  p = str2uint64(p, &colval);
  if (!p) goto nettraferror;
  nettraf0send = colval;

  p = strstr(procfile, " eth1:");
  if (!p) goto nettraferror;
  p += 6;
  for(;;++p) if (*p != ' ') break;
  /* p now points to receive bytes */
  p = str2uint64(p, &colval);
  if (!p) goto setandout;
  nettraf1recv = colval;
  p = str_field(p, 8, ' ', 1);
  /* p now points to transmit bytes */
  p = str2uint64(p, &colval);
  if (!p) goto setandout;
  nettraf1send = colval;

  goto setandout;

  nettraferror:
  nettraf0recv = nettraf0send = nettraf1recv = nettraf1send = 0;


  setandout:
  measure_cpu_user[tmpcursor] = user;
  measure_cpu_system[tmpcursor] = system;
  measure_cpu_idle[tmpcursor] = idle;
  measure_cpu_wait[tmpcursor] = wait;
  measure_cpu_irq[tmpcursor] = irq;
  measure_cpu_softirq[tmpcursor] = softirq;
  measure_ctxt[tmpcursor] = ctxt;
  measure_nettraf0recv[tmpcursor] = nettraf0recv;
  measure_nettraf0send[tmpcursor] = nettraf0send;
  measure_nettraf1recv[tmpcursor] = nettraf1recv;
  measure_nettraf1send[tmpcursor] = nettraf1send;

  measure_cursor = tmpcursor;
  return;

  error_out:
  user = nice = system = idle = wait = irq = softirq = ctxt =
         nettraf0recv = nettraf0send = nettraf1recv = nettraf1send = 0;
  goto setandout;

  return;
}
