#include "knoerred.h"
#ifndef CONFIG_KNOERRE_PROC_ACCESS
#error PROC_ACCESS needed
#endif
#include <sys/time.h>


/**
 * This function is called periodically to record multiple system values.
 *
 * interval_measure() calls this function periodically to
 * record multiple system values like load, swaps, diskfree and more.
 *
 * $Id: record_multi_values.c 691 2017-06-04 23:12:04Z fwbergm $
 */
void record_multi_values(int tmpcursor) {
  register char *p;
  int i;
  ssize_t s;
  DIR *dptr;
  struct dirent *eptr;
  char devcarrier[STRING_SIZE+28];
  char *devptr;
  int fd_carrier;
  char carrierbuf[2];
  FS_STAT tempstat;

  statfs_root_err[tmpcursor] = statfs("/", &statfs_root[tmpcursor]);
  (void) get_all_local_fs_stats(&fsstat_top[tmpcursor], &tempstat);

  p = multiresults[tmpcursor];
  i = open(CONFIG_KNOERRED_MULTIRESULTS, O_RDONLY);
  if (__unlikely(i < 0)) goto multiresultserror;
  s = read(i, p, DAEMON_NET_BUFSIZE * 2 / 3);
  close(i);
  if (__unlikely(s < 2)) goto multiresultserror;
  if (p[s-1] != '\n') {
    p[s] = '\n';
    ++s;
  }
  p[s] = 0;

  goto multiresultsdone;

  multiresultserror:
  *p = 0;

  multiresultsdone:
  p = hostname[tmpcursor];
  i = open("/proc/sys/kernel/hostname", O_RDONLY);
  if (__unlikely(i < 0)) goto hostnameerror;
  s = read(i, p, STRING_SIZE);
  close(i);
  if (__unlikely(s < 2)) goto hostnameerror;
  --s;
  p[s] = 0;

  goto hostnamedone;

  hostnameerror:
  *p = 0;

  hostnamedone:
  devptr = stpcpy(devcarrier, "/sys/class/net/");
  dptr = opendir(devcarrier);
  if (!dptr) goto netlinkserror;
  i = 0;
  for (; (eptr=readdir(dptr));) {
    if (*(eptr->d_name) == '.') continue;
    *devptr = 0;
    strncat(devptr, eptr->d_name, STRING_SIZE);
    strncat(devptr, "/carrier", STRING_SIZE);
    fd_carrier = open(devcarrier, O_RDONLY);
    if (fd_carrier >= 0) {
      if (read(fd_carrier, carrierbuf, 1) > 0) {
        if (*carrierbuf == '0') ++i;
      }
      close(fd_carrier);
    }
  }
  closedir(dptr);
  netlinksdown[tmpcursor] = i;
  goto netlinksdone;

  netlinkserror:
  netlinksdown[tmpcursor] = 0;

  netlinksdone:
  p = loadavg[tmpcursor];
  i = open("/proc/loadavg", O_RDONLY);
  if (__unlikely(i < 0)) goto loaderror;
  s = read(i, p, STRING_SIZE);
  close(i);
  if (__unlikely(s < 2)) goto loaderror;
  p[s] = 0;

  goto loadavgdone;

  loaderror:
  *p = 0;

  loadavgdone:
  p = swaps[tmpcursor];
  i = open("/proc/swaps", O_RDONLY);
  if (__unlikely(i < 0)) goto swapserror;
  s = read(i, p, STRING_SIZE);
  close(i);
  if (__unlikely(s < 2)) goto swapserror;
  p[s] = 0;

  goto getout;

  swapserror:
  *p = 0;

  getout:
  return;

}
