#ifdef KNOERRED
#include "knoerred.h"
#else
#include "knoerre.h"
#endif
#ifdef CONFIG_KNOERRE_CHECK_FILESIZES

/**
 * Step recursively into a directory and get the maximum file size.
 *
 * $Id: walkdir_filesizes.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * This is a helper function of do_check_filesizes.
 */
int walkdir_filesizes(
    const char *name,
    off_t *maxfilesize,
    char *maxfilesize_pathname,
    uint16_t pathname_size,
    int16_t depth,
    int16_t curdepth,
    const char *subdir,
    const char *pattern,
    int mode,
    time_t ts) {

  register char *nptr;
  register int dotdirs;
  register int ret;
  register off_t fsize;
  char pathname[PATHNAME_SIZE*3];
  struct stat statbuf;
  register struct dirent *eptr;
  register DIR *dptr;

  dptr = opendir(name);
  if (!dptr) return 1;
  ret = 0;
  dotdirs = 0;
  for (;;) {
    eptr = readdir(dptr); if (!eptr) break;
    nptr = eptr->d_name;
    if ((dotdirs < 2) && (*nptr == '.')) {
      if (nptr[1] == '\0') { dotdirs += 1;continue; }
      if (nptr[1] == '.') {
        if (nptr[2] == '\0') { dotdirs += 1;continue; }
      }
    }
    strncpy(pathname, name, PATHNAME_SIZE);
    strcat(pathname, "/");
    strcat(pathname, nptr);
    if (subdir) {
      strcat(pathname, "/");
      strcat(pathname, subdir);
    }
    if (lstat(pathname, &statbuf) == 0){

      if ((statbuf.st_mode&S_IFMT) == S_IFDIR) {
        if (depth >= 0) {
          if (curdepth < depth) {
            ret += walkdir_filesizes(pathname, maxfilesize, maxfilesize_pathname, pathname_size, depth, curdepth+1, NULL, pattern, mode, ts);
          }
        } else { /* depth */
          ret += walkdir_filesizes(pathname, maxfilesize, maxfilesize_pathname, pathname_size, -1, 0, NULL, pattern, mode, ts);
        } /* depth */

      } else { /* statbuf */

        if ((statbuf.st_mode&S_IFMT) == S_IFREG) {

          /* prefix mode does not work as it should: test whole path instead of filename */
          if ((mode == 0) ||
              ((mode == 1) && (str_diff_right(pathname, pattern) == 0)) ||
              ((mode == 2) && (strncmp(pathname, pattern, strlen(pattern)) == 0)) ||
              ((mode == 3) && strstr(pathname, pattern)) ||
              ((mode == 4) && strstr(pathname, pattern) && (statbuf.st_mtime >= ts))) {

            fsize = statbuf.st_size / 1024;
            if (fsize > *maxfilesize) {
              *maxfilesize = fsize;
              strcpy(maxfilesize_pathname, pathname);
            }

          } /* mode */

        } /* S_IFREG */

      } /* S_IFDIR */

    } else { /* lstat */
      ++ret;
    } /* lstat */
  }
  closedir(dptr);
  return ret;
}
#endif
