/**
 * $Id: alloc.c 691 2017-06-04 23:12:04Z fwbergm $
 */

/**
 * PD by Daniel J. Bernstein:
 * alloc()/alloc_free()
 */
#define ALIGNMENT 16 /* XXX: assuming that this alignment is enough */
#define SPACE 314572800 /* must be multiple of ALIGNMENT */

typedef union { char irrelevant[ALIGNMENT]; double d; } aligned;
static aligned realspace[SPACE / ALIGNMENT];
#define space ((char *) realspace)
static unsigned int avail = SPACE; /* multiple of ALIGNMENT; 0<=avail<=SPACE */

/*@null@*//*@out@*/void *alloc(size_t n) {
  char *x;
  n = ALIGNMENT + n - (n & (ALIGNMENT - 1)); /* XXX: could overflow */
  if (n <= avail) { avail -= n; return space + avail; }
  x = malloc(n);
  /* if (!x) errno = error_nomem; */
  if (!x) errno = 127;
  return x;
}

void alloc_free(char *x) {
  if (x >= space)
    if (x < space + SPACE)
      return; /* XXX: assuming that pointers are flat */
  free(x);
}
