#include "buffer.h"

/**
 * $Id: buffer_filecmp.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#define BUF_SIZE 16384

/**
 * buffer_filecmp: compare two files like "cmp"
 *
 * @param filename1 The name of the first file.
 * @param filename2 The name of the second file.
 * @return -1 on error. 0 if both files have same content, 1 if files differ.
 */
int buffer_filecmp(const char *filename1, const char *filename2) {
  register char c1;
  register char c2;
  int ret_value = -1;
  char buf1[BUF_SIZE+1];
  char buf2[BUF_SIZE+1];

  buffer b1 = { buf1, buf1 + BUF_SIZE, buf1 + BUF_SIZE, 0, BUF_SIZE };
  buffer b2 = { buf2, buf2 + BUF_SIZE, buf2 + BUF_SIZE, 0, BUF_SIZE };

  ret_value = buffer_openread(&b1, filename1);
  if (ret_value == -1) goto out;
  ret_value = buffer_openread(&b2, filename2);
  if (ret_value == -1) goto out;
  ret_value = -1;

  for (;;) {
    c1 = buffer_getchar(&b1);
    if (c1 == -2) goto out;
    c2 = buffer_getchar(&b2);
    if (c2 == -2) goto out;
    if (c1 != c2) goto differ;
    if (c1 == EOF) goto equal;
  }

  equal:
  ret_value = 0;
  goto out;

  differ:
  ret_value = 1;

  out:
  buffer_close(&b1);
  buffer_close(&b2);
  return ret_value;
}

