#include "libfwb.h"

/**
 * Return index to a field in a string with given separator.
 *
 * This is a helper function for do_check_longuserprocesses() and more.
 * It searches a string for a field specified by number
 * 'index' and returns the string index position.
 * If the end of the string is reached without finding
 * field number 'index' then -1 is returned.
 *
 * $Id: field_indexc.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param s The string to be searched through.
 * @param index The number of the field index to search for.
 * @param sep The field separator.
 * @param multiple True, if successive separators should count as one.
 * @return The index position of the field in the string or -1 on error.
 */
int32_t field_indexc(const char *s,int index, char sep, int multiple) {
  register const char* t = s;

  if (!t) return 0;
  if (index < 1) return 0;
  for (;;) {
    if (!*t) return -1;
    if (*t == sep) {
      --index; ++t;
      if (multiple) while (*t == sep) ++t;
      if (!index) return t-s;
    } else {
      ++t;
    }
  }
}
