#include "libfwb.h"

/**
 * $Id: file2buf.c 691 2017-06-04 23:12:04Z fwbergm $
 */

/**
 * Open a file and read (one time only) data to specified buf.
 *
 * @param filename The name of the file the buf should be read from.
 * @param buf The buf to be filled from file.
 * @param size The size of the buf.
 * @return -1 on error. 0 if no bytes were read. On success return bytes read.
 */
ssize_t file2buf(const char *filename, register char *buf, int size) {
  register int c = -1;
  register ssize_t bytes_read;

  if (!buf) return c;
  *buf = 0;
  if (!filename) return c;
  if (!*filename) return c;
  if (size < 1) return c;
  
  c = open(filename, O_RDONLY);
  if (c == -1) return c;

  bytes_read = read(c, buf, size);
  (void) close(c);
  if (bytes_read > 0) buf[bytes_read] = 0;
  return bytes_read;
}

