/*
 * lib/get_home_from_passwd.c
 * FWB www.tuxad.com
 *
 * Get home directory from /etc/passwd.
 * Example usage:
 *
 * int main() {
 *   register char *p;
 *   const char *pwdbuf;
 *   off_t pwdlen;
 *   uint32_t ulen;
 * 
 *   p = (char *) map_passwd_file(&pwdbuf, &pwdlen);
 *   if (p == (char *) -1) return 1;
 *   p = (char *) get_home_from_passwd(pwdbuf, pwdlen, 1000, &ulen);
 *   if (!p) { munmap((void *) pwdbuf, pwdlen); return 2; }
 *   write(1, p, ulen);
 *   write(1, "\n", 1);
 *   munmap((void *) pwdbuf, pwdlen);
 *   return 0;
 * }
 *
 * $Id: get_home_from_passwd.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#include "libfwb.h"
#include <stdint.h>
#include <string.h>

const char *get_home_from_passwd(const char *pwdbuf, off_t pwdlen, uint32_t uid, uint32_t *userhome_len) {
  register char *p;
  char *s;
  char uid_s[12];
  int uid_s_len;
  const char *t;

  if ((!pwdbuf) || (!userhome_len)) return 0;
  t = pwdbuf + pwdlen;
  p = sputuint32(uid_s, uid);
  *p = ':'; ++p;
  uid_s_len = p - uid_s;
  p = (char *) pwdbuf;

  for(;;) {
    s = memchr(p, '\n', t-p);
    p = strchr(pwdbuf, ':');
    if (!p) return 0;
    ++p;
    p = strchr(p, ':');
    if (!p) return 0;
    ++p;
    if (!strncmp(p, uid_s, uid_s_len)) {
      p += uid_s_len;
      p = memchr(p, ':', s - p);
      if (!p) return 0;
      ++p;
      p = memchr(p, ':', s - p);
      if (!p) return 0;
      ++p;
      s = memchr(p, ':', s - p);
      if (!s) return 0;
      *userhome_len = (int) (s - p);
      return p;
    }
    p = memchr(p, '\n', t-p);
    if (!p) return 0;
    ++p;
    for(;*p == '\n';++p); /* broken passwd files */
    pwdbuf = p;
  }
}
