#include "libfwb.h"

/**
 * Convert hex number from string to (unsigned) integer.
 *
 * The given string is parsed for ciphers '0' - '9' and
 * alpha chars 'a' - 'f' and 'A' - 'F'.
 * If the first char is not a hex cipher then 0 will be returned.
 * Parsing will stop at the first non-hexcipher char.
 * No boundary and carry checks are made.
 * The second arg is a pointer to the var to save the result in.
 * This pointer may be 0. This can be used to just return a
 * pointer to the char after the number.
 * $Id: hexstr2uint32.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param s The string to parse for a hex number.
 * @param val a pointer to the variable to store the value or NULL
 * @return a pointer to the first char after number or 0 on error
 */
char *hexstr2uint32(register char *s, uint32_t *val) {
  register char c;
  uint32_t i;

  if ((!s) || (!val)) return 0;
  c = *s;
  if ((c < '0') || ((c > '9') && (c < 'A')) || ((c > 'F') && (c < 'a')) || (c > 'f')) return 0;
  ++s;
  c -= '0';
  if (c > 9) c -= 7; 
  if (c > 15) c -= 32; 
  i = c;
  for(;;) {

    c = *s;
    if ((c < '0') || ((c > '9') && (c < 'A')) || ((c > 'F') && (c < 'a')) || (c > 'f')) {
      if (val) *val = (uint32_t) i;
      return s;
    } else {
      c -= '0';
      if (c > 9) c -= 7; 
      if (c > 15) c -= 32; 
      i = (i * 16) + c;
      ++s;
    }

    /* loop unroll 2 */
    c = *s;
    if ((c < '0') || ((c > '9') && (c < 'A')) || ((c > 'F') && (c < 'a')) || (c > 'f')) {
      if (val) *val = (uint32_t) i;
      return s;
    } else {
      c -= '0';
      if (c > 9) c -= 7; 
      if (c > 15) c -= 32; 
      i = (i * 16) + c;
      ++s;
    }

  }
}
