/*
 * lib/map_file.c
 * FWB www.tuxad.com
 *
 * $Id: map_file.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#include "libfwb.h"

/**
 * mmap() a file
 *
 * Open file, get its length by lseek(), call
 * mmap(), close the file and return a pointer to the
 * mmapped area (*fbuf) like mmap() does or -1 on
 * any error. The benefit of this function is to check
 * for any errors and a mmapped area without an open
 * file descriptor.
 * Returned errors should be checked with something like
 * i.e.
 * p = map_file(...); if (p == (char *) -1) goto error;
 *
 * @param fname The file to be mapped.
 * @param fbuf The handle for the mmapped data.
 * @param flen The pointer to the off_t to hold the length.
 * @return The pointer to the data (*fbuf) or -1 on error.
 */
const char *map_file(const char *fname, const char **fbuf, off_t *flen) {
  int fd;
  int sv_errno;

  if ((!fbuf) || (!flen)) goto error;
  fd = open(fname, O_RDONLY);
  if (fd < 0) goto error;
  *flen = lseek(fd, 0, SEEK_END); /* 'stead of stat */
  if (*flen == (off_t) -1) { close(fd); goto error; }

  *fbuf = mmap((void *) 0, *flen, PROT_READ, MAP_PRIVATE, fd, 0);
  sv_errno = errno;
  close(fd);
  errno = sv_errno;
  if (*fbuf == (char *) -1) goto error;

  return *fbuf;

  error:
  return (char *) -1;
}
