/*
 * lib/map_passwd.c
 * FWB www.tuxad.com
 *
 * $Id: map_passwd.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#include "libfwb.h"

#ifndef _PATH_PASSWD
#define _PATH_PASSWD "/etc/passwd"
#endif

/**
 * mmap() file /etc/passwd
 *
 * Open /etc/passwd, get its length by lseek(), call
 * mmap(), close the file and return a pointer to the
 * mmapped area (*pwdbuf) like mmap() does or -1 on
 * any error. The benefit of this function is to check
 * for any errors and a mmapped area without an open
 * file descriptor.
 * Returned errors should be checked with something like
 * i.e.
 * p = map_passwd_file(...); if (p == (char *) -1) goto error;
 *
 * $Id: map_passwd.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param pwdbuf The handle for the mmapped data.
 * @param pwdlen The pointer to the off_t to hold the length.
 * @return The pointer to the data (*pwdbuf) or -1 on error.
 */
const char *map_passwd_file(const char **pwdbuf, off_t *pwdlen) {
  int fd;
  int sv_errno;

  if ((!pwdbuf) || (!pwdlen)) goto error;
  fd = open(_PATH_PASSWD, O_RDONLY);
  if (fd < 0) goto error;
  *pwdlen = lseek(fd, 0, SEEK_END); /* 'stead of stat */
  if (*pwdlen == (off_t) -1) { close(fd); goto error; }

  *pwdbuf = mmap((void *) 0, *pwdlen, PROT_READ, MAP_PRIVATE, fd, 0);
  sv_errno = errno;
  close(fd);
  errno = sv_errno;
  if (*pwdbuf == (char *) -1) goto error;

  return *pwdbuf;

  error:
  return (char *) -1;
}
