#include "libfwb.h"

/**
 * Convert number from string to (unsigned) integer.
 *
 * The given string is parsed for ciphers '0' - '9'.
 * If the first char is not a cipher then -1 will be returned.
 * Parsing will stop at the first non-cipher char.
 * No boundary and carry checks are made.
 *
 * $Id: str2uint63.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * @param s The string to parse for a number.
 * @return The recognized unsigned integer or -1 on error.
 */
int64_t str2uint63(register const char *s) {
  register int64_t i;
  register char c;

  c = *s;
  if ((c < '0') || (c > '9')) return -1;
  ++s;
  i = c - '0';
  for(;;) {
    c = *s;
    if ((c < '0') || (c > '9')) {
      return i;
    } else {
      i = (i * 10) + c - '0';
      ++s;
    }
  }
}
