#include "libfwb.h"

/**
 * Return pointer to a field in a string with given separator.
 *
 * This is a helper function for do_check_longuserprocesses() and more.
 * It searches a string for a field specified by number
 * 'index' and returns a pointer to the field.
 * If the end of the string is reached without finding
 * field number 'index' then 0 is returned.
 *
 * @param s The string to be searched through.
 * @param index The number of the field index to search for.
 * @param sep The field separator.
 * @param multiple must be true, if successive separators should be counted as one.
 * @return The field position of the field in the string or 0 on error.
 */
char *str_field(const char *s, int index, char sep, int multiple) {
  register const char* t=s;

  if (index < 1) return (char *) s;
  for (;;) {
    if (!*t) return 0;
    if (*t == sep) {
      --index; ++t;
      if (multiple) while (*t == sep) ++t;
      if (!index) return (char *) t;
    } else {
      ++t;
    }
  }
}
