/**
 * monitoringfs.c
 *
 * nagios remote monitoring filesystem for libfuse
 * www.tuxad.com
 */

#include "monitoringfs.h"

const char usage[] =
  "usage: monitoringfs [--port PORT] [FUSE-opts] mountpoint <hosts-and-addresses-plus-servicechecks\n"
  "example: echo 'localhost 127.0.0.1'|monitoringfs mnt/\n";
static void printusage() { write(1, usage, sizeof usage - 1); _exit(1); }

int port;

const char *monitoringfs_hosts[MAX_HOSTS + 1];
const char *monitoringfs_ipaddresses[MAX_HOSTS + 1];
const char *monitoringfs_keys[] = {
  "cat/proc/loadavg",
  "cat/proc/sys/kernel/hostname",
  "ctxtswitch",
  "cpuI",
  "cpui",
  "cpuu",
  "cpus",
  "cpuw",
  "diskusage/home/www",
  "load1",
  "load5",
  "load15",
  "loaduser/10000/2000000",
  "loaduser/630/65530",
  "mailqsize",
  "nettraf0",
  "nettraf1",
  "proccount",
  "proccounttg",
  "swaps",
  "timediff/0",
  "uptime",
  0
};
uint16_t monitoringfs_hosts_count;
uint16_t monitoringfs_keys_count;

KD keyroot = {
  0,
  0,
  ".",
  ".",
  1,
  2
};

buffer buf0;
char buf0space[READBUFSIZE+1];

#ifdef DEBUG
static void print_kd_dir(KD *start, int recursive) {
  KD *e;

  if (start) {
    e = start;
  } else {
    e = &keyroot;
  }
  for (;;) {
    if (!e) break;
    printf("%s %s", e->fullpath, e->entry);
    if (e->subdir || e->is_dir) {
      printf(" d\n");
      if (recursive && e->subdir) print_kd_dir(e->subdir, recursive);
    } else {
      printf(" -\n");
    }
    e = e->next;
  }
}
#endif


static struct fuse_operations monitoringfs_oper = {
    .getattr   = monitoringfs_getattr,
    .readdir = monitoringfs_readdir,
    .open   = monitoringfs_open,
    .read   = monitoringfs_read,
};


char *str2uint16(register char *s, uint16_t *val) {
  register char c;
  uint16_t i;

  c = *s;
  if ((c < '0') || (c > '9')) return 0;
  ++s;
  i = c - '0';
  for(;;) {

    c = *s;
    if ((c < '0') || (c > '9')) {
      if (val) *val = i;
      return s;
    } else {
      i = (i * 10) + c - '0';
      ++s;
    }

  }
}

/**
 *    main()
 *
 */
int main(int argc, char *argv[]) {
  register char *p;
  char *entry;
  char inputline[STRINGSIZE+1];
  int l;
  uint16_t val16;
  KD *kptr;
  char **margv;

  if (argc < 2) printusage();
  port = 8888;
#include <stdio.h>
/*
for(l=0;l<argc;++l) printf("X%sX ", argv[l]); printf("\n");
*/
  if (argc >= 4) {
    if (!strcmp(argv[1], "--port")) {
      if (!str2uint16(argv[2], &val16)) printusage();
      port = val16;
      /* delete option from list */
      l = 3;
      for (;;) {
        if (l >= argc) break;
        argv[l-2] = argv[l];
        ++l;
      }
      argc -= 2;
    }
  }
  /* add directio-option */
  argc += 2;
  margv = alloc(argc * sizeof(char*));
  if (!margv) _exit(2);
  margv[0] = argv[0];
  margv[1] = "-o"; margv[2] = "direct_io";
  for (l=3;l<argc;++l) margv[l] = argv[l];
  argv = margv;

  monitoringfs_hosts_count = monitoringfs_keys_count = 0;

  buf0.s = buf0space;
  buf0.l = READBUFSIZE;
  buf0.t = buf0space + buf0.l;
  buf0.c = buf0.t;

  *monitoringfs_hosts = *monitoringfs_ipaddresses = 0;

  /* set up root of FS with entries "." and ".." */
  kptr = create_kd_entry("..");
  if (!kptr) return EX_SOFTWARE;
  keyroot.next = kptr;
  kptr->is_dir = 1;
  kptr->lnk_count = 0;

  /* read host/address list from stdin */
  l = bgetline(inputline, STRINGSIZE);

  for (;;) {

    if (l == EOF) break;
    p = inputline;
    p[l-1] = 0;
    ++l; /* for alloc() */

    /**
     * host format: <HOST> <SPACE> <IPADDRESS>
     * key format: key
     */
    p = strchr(p, ' ');
    if (!p) {

      /* treat line as key and insert it */
      kptr = search_kd_entry(inputline, 0, 1);
      if (!kptr) return EX_SOFTWARE;
      ++monitoringfs_keys_count;

    } else {

      /* treat line as host definition */
      *p = 0; /* end of host_name string */
      ++p; /* points to address */
      entry = alloc(l);
      p = stpcpy(entry, p); /* copy address */
      /* storing addresses before names because of alignment :-) */
      monitoringfs_ipaddresses[monitoringfs_hosts_count] = entry;
  
      ++p; /* point to first free char after address */
      strcpy(p, inputline); /* copy host_name */
      monitoringfs_hosts[monitoringfs_hosts_count] = p;
  
      ++monitoringfs_hosts_count;
      monitoringfs_ipaddresses[monitoringfs_hosts_count] = 0;
      monitoringfs_hosts[monitoringfs_hosts_count] = 0;
      if (monitoringfs_hosts_count >= MAX_HOSTS) break;

    }

    l = bgetline(inputline, STRINGSIZE);

  }

  /* no key / service-check per stdin, use builtin defaults */
  if (!monitoringfs_keys_count) {
    l = 0;
    for (;;) {
      if (!monitoringfs_keys[monitoringfs_keys_count]) break;
      kptr = search_kd_entry(monitoringfs_keys[monitoringfs_keys_count], 0, 1);
      ++monitoringfs_keys_count;
    }
  }

/* debug, to be removed */
printf("monitoringfs_hosts_count = %u, monitoringfs_keys_count = %u\n", monitoringfs_hosts_count, monitoringfs_keys_count);

  count_dir_entries(0, 0);

  /* array of host_names and ipaddresses is filt, register mount */
  return fuse_main(argc, argv, &monitoringfs_oper, NULL);
}

