#ifndef MONITORINGFS_H
#define MONITORINGFS_H

/**
 * monitoringfs.h
 * $Id: monitoringfs.h 691 2017-06-04 23:12:04Z fwbergm $
 */

#define _FILE_OFFSET_BITS 64

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <features.h>
#include <sysexits.h>

#define FUSE_USE_VERSION  26
#include <fuse.h>
 
#include <stdint.h>
#include <poll.h>
#include <setjmp.h>
#include <time.h>

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <unistd.h>


/*
 * timeout in ms for receiving data
 */
#define UDP_RECV_TIMEOUT 2000
#define HTTP_RECV_TIMEOUT 2000

/*
 * max no of hosts to be served by monitoringfs
 * on raising you maybe should also raise SPACE
 */
#define MAX_HOSTS 10000



#define STRINGSIZE 1024
#define UDPREADBUFSIZE 4096
#define UDPREMOTEPORT 8888
#define HTTPREADBUFSIZE 4096
extern int port;
 

extern const char *monitoringfs_hosts[];
extern const char *monitoringfs_ipaddresses[];
extern const char *monitoringfs_keys[];
extern uint16_t monitoringfs_hosts_count;

typedef struct key_dir_entry KD;
typedef struct key_dir_entry {
  KD *next;            /* points to next entry in dir */
  KD *subdir;          /* points to subdir if type is dir */
  char *fullpath;      /* relative full path within monitoringfs */
  char *entry;         /* entry within this dir, is last part of fullpath */
  uint16_t is_dir;     /* marks "." and ".." as dir, subdir is 0 in this case */
  uint16_t lnk_count;  /* link count */
} KDstruct;
extern KD keyroot;


#define READBUFSIZE 4096
#ifndef EOF
#define EOF (-1)
#endif
typedef struct buffer {
  char *s;    /* first char in bufspace */
  char *c;    /* cursor */
  char *t;    /* first char after bufspace */
  uint32_t l; /* for read buffers */
} buffer;
extern buffer buf0;
extern char buf0space[READBUFSIZE+1];

int bgetchar();
int bgetline(char *r, int size);

KD *create_kd_entry(const char *path);
KD *insert_kd_dir(KD *start, const char *fullpath, const char *entry);
KD *search_kd_direntry(const char *entry, KD *start, KD **last_entry);
KD *search_kd_dir(const char *kdirpath, KD *start, int insert);
KD *search_kd_entry(const char *kdirpath, KD *start, int insert);
void count_dir_entries(KD *start, uint16_t countparentdot);

int monitoringfs_get_remote_by_http(const char *ip_address, const char *key, char *resultstring, int resultmaxlen);
int monitoringfs_get_remote_by_udp(const char *ip_address, const char *key, char *resultstring, int resultmaxlen);
int monitoringfs_getattr(const char *path, struct stat *stbuf);
int monitoringfs_open(const char *path, struct fuse_file_info *fi);
int monitoringfs_read(const char *, char *, size_t , off_t , struct fuse_file_info *);
int monitoringfs_readdir(const char *, void *, fuse_fill_dir_t, off_t, struct fuse_file_info *);

void *alloc(size_t n);

#endif
