#include "monitoringfs.h"

#define STR_HTTP_GET1 "GET /"
#define STR_HTTP_GET2 " HTTP/1.1\nHost: "
#define STR_HTTP_GET3 "\n\n"

int monitoringfs_get_remote_by_http(const char *ip_address, const char *key, char *resultstring, int resultmaxlen) {
  register char *p;
  int tmpint;
  ssize_t bytecount;
  int sockfd;
  struct sockaddr_in servin;
  struct pollfd st_poll;
  char http_buf[HTTPREADBUFSIZE+1];
  int resultlen;

  if (resultmaxlen > HTTPREADBUFSIZE) resultmaxlen = HTTPREADBUFSIZE;
  --resultmaxlen; /* just paranoid */
  *resultstring = 0;
  resultlen = 0;

  sockfd = -1;

  /* setup sockaddr */
  memset(&servin, 0, sizeof servin);
  servin.sin_family = AF_INET;
  servin.sin_port = htons(port);
  if (inet_aton(ip_address, &servin.sin_addr) == 0) {
    goto out;
  }

  /* socket */
  sockfd = tmpint = socket(PF_INET, SOCK_STREAM, 0);
  if (sockfd == -1) {
    goto out;
  }

  tmpint = connect(sockfd, (struct sockaddr *) &servin, sizeof servin);
  if (tmpint == -1) {
    goto out;
  }

  /* send */
  p = stpcpy(http_buf, STR_HTTP_GET1);
  p = stpcpy(p, key);
  p = stpcpy(p, STR_HTTP_GET2);
  p = stpcpy(p, ip_address);
  p = stpcpy(p, STR_HTTP_GET3);

  tmpint = write(sockfd, http_buf, p - http_buf);
  if (tmpint == -1) {
    goto out;
  }

  /* recv */
  st_poll.fd = sockfd;
  st_poll.events = POLLIN;
  tmpint = poll(&st_poll, 1, HTTP_RECV_TIMEOUT); /* no check for POLLIN */
  if (!tmpint) { /* timeout */
    /* only while developing! */
    strcpy(resultstring, "TIMEOUT\n");
    resultlen = 8;
    goto out;
  }
  if (tmpint < 0) { /* error */
    /* only while developing! */
    strcpy(resultstring, "ERROR\n");
    resultlen = 6;
    goto out;
  }

  bytecount = read(sockfd, http_buf, sizeof http_buf);
  if (bytecount < 1) goto out;
  http_buf[bytecount] = 0;

  /* check HTTP code: HTTP/1.0 200 */
  if (strncmp(http_buf, "HTTP/1.", 7)) goto out;
  if (strncmp(http_buf+8, " 200 ", 5)) goto out;

  /* get last line */
  for (tmpint = bytecount;;) {
    --tmpint;
    if (!tmpint) break;
    if ((unsigned char) http_buf[tmpint] >= 32) break;
  }
  p = http_buf + tmpint + 1;
  *p = 0;
  for (;;) {
    --tmpint;
    if (tmpint < 0) break;
    if ((unsigned char) http_buf[tmpint] < 32) break;
  }
  /* skip blanks and ctrl chars */
  for (;;) {
    ++tmpint;
    if ((unsigned char) http_buf[tmpint] > 32) break;
  }

  p = http_buf + tmpint;
  strncpy(resultstring, p, resultmaxlen - 1);
  resultlen = strlen(resultstring);
  resultstring[resultlen] = 0;

  out:
  if (sockfd != -1) close(sockfd);
  return resultlen;
}
