#include "monitoringfs.h"

int monitoringfs_get_remote_by_udp(const char *ip_address, const char *key, char *resultstring, int resultmaxlen) {
  register char *p;
  int tmpint;
  ssize_t bytecount;
  int sockfd;
  struct sockaddr_in servin;
  struct sockaddr_in clntin;
  struct pollfd st_poll;
  char udp_buf[UDPREADBUFSIZE+1];
  int resultlen;

  if (resultmaxlen > UDPREADBUFSIZE) resultmaxlen = UDPREADBUFSIZE;
  --resultmaxlen; /* just paranoid */
  *resultstring = 0;
  resultlen = 0;

  sockfd = -1;

  /* setup sockaddr */
  memset(&servin, 0, sizeof servin);
  servin.sin_family = AF_INET;
  servin.sin_port = htons(port);
  if (inet_aton(ip_address, &servin.sin_addr) == 0) {
    goto out;
  }

  /* socket */
  sockfd = tmpint = socket(PF_INET, SOCK_DGRAM, 0);
  if (sockfd == -1) {
    goto out;
  }

  memset(&clntin, 0, sizeof clntin);
  clntin.sin_family = AF_INET;
  /* let os choose port */
  clntin.sin_port = 0;
  tmpint = bind(sockfd, (struct sockaddr *) &clntin, sizeof clntin);
  if (tmpint) {
    goto out;
  }

  tmpint = connect(sockfd, (struct sockaddr *) &servin, sizeof servin);
  if (tmpint == -1) {
    goto out;
  }

  /* send */
  p = stpcpy(udp_buf, key);
  *p = '\n'; ++p;

  tmpint = send(sockfd, udp_buf, p - udp_buf, 0);
  if (tmpint == -1) {
    goto out;
  }

  /* recv */
  st_poll.fd = sockfd;
  st_poll.events = POLLIN;
  tmpint = poll(&st_poll, 1, UDP_RECV_TIMEOUT); /* no check for POLLIN */
  if (!tmpint) { /* timeout */
    /* only while developing! */
    strcpy(resultstring, "TIMEOUT\n");
    resultlen = 8;
    goto out;
  }
  if (tmpint < 0) { /* error */
    /* only while developing! */
    strcpy(resultstring, "ERROR\n");
    resultlen = 6;
    goto out;
  }

  bytecount = recv(sockfd, udp_buf, sizeof udp_buf, 0);
  if (bytecount < 1) {
    goto out;
  }
  udp_buf[bytecount] = 0;

  /* get last line */
  for (tmpint = bytecount;;) {
    --tmpint;
    if (!tmpint) break;
    if ((unsigned char) udp_buf[tmpint] >= 32) break;
  }
  p = udp_buf + tmpint + 1;
  *p = 0;
  for (;;) {
    --tmpint;
    if (tmpint < 0) break;
    if ((unsigned char) udp_buf[tmpint] < 32) break;
  }
  /* skip blanks and ctrl chars */
  for (;;) {
    ++tmpint;
    if ((unsigned char) udp_buf[tmpint] > 32) break;
  }

  p = udp_buf + tmpint;
  strncpy(resultstring, p, resultmaxlen - 1);
  resultlen = strlen(resultstring);
  resultstring[resultlen] = 0;

  out:
  if (sockfd != -1) close(sockfd);
  return resultlen;
}
