#include "monitoringfs.h"

int monitoringfs_getattr(const char *path, struct stat *stbuf) {
  register char *p;
  int h;
  time_t now;
  KD *kd;

  now = time(0);
  memset(stbuf, 0, sizeof(struct stat));

  /* defaults :-) */
  stbuf->st_mode = S_IFDIR | 0555;
  stbuf->st_nlink = keyroot.lnk_count;
  stbuf->st_size = 0;
  stbuf->st_atime = now;
  stbuf->st_mtime = now;
  stbuf->st_ctime = now;

  if(strcmp(path, "/") == 0) {
    stbuf->st_nlink = 2 + monitoringfs_hosts_count;
    return 0;
  }
  ++path;

  h = 0;
  for(;;++h) {
    int hlen;

    p = (char *) monitoringfs_hosts[h];
    if (!p) return -ENOENT;

    /* host-dir */
    hlen = strlen(p);
    if (!strncmp(path, p, hlen)) {
      /* matches at least beginning of host_name */
      p = (char *) path + hlen;
      if (!*p) return 0; /* does end with 0 */
      if (*p != '/') continue;
      ++p;
      if (!*p) return 0;

      /* now points to key-dir */
      path = p;
      kd = search_kd_entry(path, 0, 0);
      if (!kd) return -ENOENT;
      if (kd->is_dir || kd->subdir) {
        stbuf->st_nlink = kd->subdir->lnk_count;
        return 0;
      }
      stbuf->st_mode = S_IFREG | 0444;
      stbuf->st_nlink = 1;
      return 0;
    }
    
  }

  return -ENOENT;
}
