#!/bin/bash

# /etc/nagios-cache.d/apacheovz.sh
# FWB 2010-07-09
# $Id: apacheovz.sh 691 2017-06-04 23:12:04Z fwbergm $

# files in /nagios-cache :
# /nagios-cache/processd/apache2           result of the host itself
# /nagios-cache/vps/0/processd/apache2     if host is OpenVZ master (CTID=0) then this is a copy
# /nagios-cache/vps/1079/processd/apache2  result of virtual private server with CTID=1079
# /nagios-cache/vps/processd/apache2       summary/max result of all CTIDs (0 included)


# do some checks, set up environment, cd /nagios-cache :
test -e /etc/nagios-cache.d/.ncrc || { echo "ERROR: no /etc/nagios-cache.d/.ncrc found" | tee /dev/stderr; exit 1; }
. /etc/nagios-cache.d/.ncrc

# nagios-check "apacheovz" only for OpenVZ master (CTID=0)
KEY=processd/apache2
TMPSUFFIX=$(date +%M)
KEYDIR=${KEY%/*}
mkdir -p $KEYDIR 2>/dev/null
KEYRESULT=$KEY
[ -d "$KEYRESULT" ] && KEYRESULT="$KEYRESULT"_
LOCKFILE="$KEYRESULT".lock

if [ -e $LOCKFILE ]
then
  # there's still a nc-process running
  echo 999999999999999997 >>$KEYRESULT
  AGE=$(($(date +%s)-$(date -r $LOCKFILE +%s 2>/dev/null || echo 0)))
  [ $AGE -gt 1800 ] && /bin/rm -f $LOCKFILE
  exit 0
fi

echo $$ >$LOCKFILE

# check the host itself?
if [ "$OVZ_CHECK_MASTER_ID" ]
then
  knoerre $KEY >>$KEY.$TMPSUFFIX 2>/dev/null \
    || echo 999999999999999999 >>$KEY.$TMPSUFFIX
  /bin/mv $KEY.$TMPSUFFIX $KEYRESULT 2>/dev/null
fi

# is host an OpenVZ master (CTID=0)?
if [ -d "$OVZ_PRIVATE_DIR" ]
then

  # OpenVZ master: check all virtual private servers, too

  mkdir -p vps/0/$KEYDIR vps/$KEYDIR
  # copy master result?
  [ "$OVZ_CHECK_MASTER_ID" ] && /bin/cp -af $KEYRESULT vps/0/$KEYRESULT

  # collect results of all vps
  # id 1200 has apache under daemontools, you don't need to check this! :-)
  /bin/rm -fr vps/1200/$KEYDIR
  for f in $(cd "$OVZ_PRIVATE_DIR";ls|grep -v 1200)
  do
    mkdir -p vps/$f/$KEYDIR
    # no lockfile per ovm instance, just check age of lockfile
    AGE=$(($(date +%s)-$(date -r $LOCKFILE +%s 2>/dev/null || echo 0)))
    [ $AGE -gt 55 ] && exit 0
    vzctl exec $f /opt/nagios-fwb/bin/knoerre $KEY >>vps/$f/$KEY.$TMPSUFFIX 2>/dev/null \
      || echo 999999999999999999 >>vps/$f/$KEY.$TMPSUFFIX
    grep -q 99999 vps/$f/$KEY.$TMPSUFFIX && \
      { vzctl exec $f /opt/nagios-fwb/bin/knoerre process/apache2 >vps/$f/$KEY.$TMPSUFFIX 2>/dev/null \
        || echo 999999999999999999 >>vps/$f/$KEY.$TMPSUFFIX; }
    /bin/mv vps/$f/$KEY.$TMPSUFFIX vps/$f/$KEYRESULT 2>/dev/null
  done

  # search for max result, include CTID=0
  RET=0
  HIGHVALUEVPS=""
  for f in $(cd "$OVZ_PRIVATE_DIR";ls) $OVZ_CHECK_MASTER_ID
  do
    test -e vps/$f/$KEYRESULT || continue
    echo $f $(cat vps/$f/$KEYRESULT 2>/dev/null) >>vps/$KEY.$TMPSUFFIX
    VALUE=$(tail -n 1 vps/$f/$KEYRESULT 2>/dev/null)
    # simple error check to catch integer-test-errors
    [ "$VALUE" -ge 0 ] 2>/dev/null || VALUE="999999999999999999"
    # new max value?
    [ "$VALUE" -gt "$RET" ] && { HIGHVALUEVPS="$f $(cat vps/$f/$KEYRESULT 2>/dev/null)"; RET=$VALUE; }
  done
  [ "$HIGHVALUEVPS" ] && echo $HIGHVALUEVPS >>vps/$KEY.$TMPSUFFIX

  # print max of all results
  echo $RET >>vps/$KEY.$TMPSUFFIX
  /bin/mv vps/$KEY.$TMPSUFFIX vps/$KEYRESULT 2>/dev/null

fi

/bin/rm -f $LOCKFILE
exit 0
