#!/bin/bash

# /etc/nagios-cache.d/daemons.sh
# $Id: daemons.sh 691 2017-06-04 23:12:04Z fwbergm $

# do some checks, set up environment, cd /nagios-cache :
test -e /etc/nagios-cache.d/.ncrc || { echo "ERROR: no /etc/nagios-cache.d/.ncrc found" | tee /dev/stderr; exit 1; }
. /etc/nagios-cache.d/.ncrc

# nagios-check "daemons"
KEY=daemons
ulimit -t 30
ulimit -i 1
# check master, too; set OVZ_CHECK_MASTER_ID and set it to 0:
#OVZ_CHECK_MASTER_ID=0
TMPSUFFIX=$(date +%M)
KEYDIR=${KEY%/*}
[ "$KEY" = "$KEYDIR" ] && KEYDIR=.
mkdir -p $KEYDIR 2>/dev/null
KEYRESULT=$KEY
[ -d "$KEYRESULT" ] && KEYRESULT="$KEYRESULT"_
LOCKFILE="$KEYRESULT".lock

if [ -e $LOCKFILE ]
then
  # there's still a nc-process running
  echo 999999999999999997 >>$KEYRESULT
  AGE=$(($(date +%s)-$(date -r $LOCKFILE +%s 2>/dev/null || echo 0)))
  [ $AGE -gt 1800 ] && /bin/rm -f $LOCKFILE
  exit 0
fi

echo $$ >$LOCKFILE

# collect results of all daemons, background jobs and related checks
for f in \
  /etc/nagios-cache.d/daemons-cron.sh \
  /etc/nagios-cache.d/daemons-ftp.sh \
  /etc/nagios-cache.d/daemons-kugelfisch.sh \
  /etc/nagios-cache.d/daemons-loadlogger.sh \
  /etc/nagios-cache.d/daemons-ssh.sh \
  /etc/nagios-cache.d/daemons-syslog.sh \
  /etc/nagios-cache.d/kugelfutex.sh \
  /etc/nagios-cache.d/loadlogger.sh
do
  $f
done

# search for max result
RET=0
HIGHVALUEVPS=""
rm -f $KEY.$TMPSUFFIX
for f in \
  /nagios-cache/filetimestamp/M/var/kugelfisch/log/futex_log \
  /nagios-cache/cmdline/loadlogger.py \
  /nagios-cache/processd/cron \
  /nagios-cache/processd/vsftpd \
  /nagios-cache/process/kugelfisch \
  /nagios-cache/processd/sshd \
  /nagios-cache/processd/syslog-ng \

do
  echo $(tac $f 2>/dev/null) >>$KEY.$TMPSUFFIX
  VALUE=$(tail -n 1 $f 2>/dev/null)
  # simple error check to catch integer-test-errors
  [ "$VALUE" -ge 0 ] 2>/dev/null || VALUE="999999999999999999"
  # new max value?
  [ "$VALUE" -gt "$RET" ] && { HIGHVALUEVPS="$VALUE $f"; RET=$VALUE; }
done
[ "$HIGHVALUEVPS" ] && echo $HIGHVALUEVPS >>$KEY.$TMPSUFFIX

# print max of all results
echo $RET >>$KEY.$TMPSUFFIX
/bin/mv $KEY.$TMPSUFFIX $KEYRESULT 2>/dev/null


/bin/rm -f $LOCKFILE
exit 0
