#!/bin/bash

# /etc/nagios-cache.d/duvarlibvzprivate.sh
# $Id: duvarlibvzprivate.sh 691 2017-06-04 23:12:04Z fwbergm $


# do some checks, set up environment, cd /nagios-cache :
test -e /etc/nagios-cache.d/.ncrc || { echo "ERROR: no /etc/nagios-cache.d/.ncrc found" | tee /dev/stderr; exit 1; }
. /etc/nagios-cache.d/.ncrc

# nagios-check "duvarlibvzprivate"
KEY=duvarlibvzprivate
ulimit -t 3600
ulimit -i 1
# check master, too; set OVZ_CHECK_MASTER_ID and set it to 0:
OVZ_CHECK_MASTER_ID=0
TMPSUFFIX=$(date +%M)
KEYDIR=${KEY%/*}
[ "$KEY" = "$KEYDIR" ] && KEYDIR=.
mkdir -p $KEYDIR 2>/dev/null
KEYRESULT=$KEY
[ -d "$KEYRESULT" ] && KEYRESULT="$KEYRESULT"_
LOCKFILE="$KEYRESULT".lock

if [ -e $LOCKFILE ]
then
  # there's still a nc-process running
  echo 999999999999999997 >>$KEYRESULT
  AGE=$(($(date +%s)-$(date -r $LOCKFILE +%s 2>/dev/null || echo 0)))
  [ $AGE -gt 1800 ] && /bin/rm -f $LOCKFILE
  exit 0
fi

echo $$ >$LOCKFILE

# check the host itself?
if [ "$OVZ_CHECK_MASTER_ID" ]
then
  du -smx /var/lib/vz/private/* >>$KEY.$TMPSUFFIX 2>/dev/null \
    || echo 0 >>$KEY.$TMPSUFFIX
  sort -n $KEY.$TMPSUFFIX >$KEY.$TMPSUFFIX.2 2>/dev/null
  /bin/cp -af $KEY.$TMPSUFFIX.2 $KEY.$TMPSUFFIX 2>/dev/null
  tail -n 1 $KEY.$TMPSUFFIX.2 2>/dev/null |awk '{print$1}' 2>/dev/null >>$KEY.$TMPSUFFIX 2>/dev/null
  /bin/rm -f $KEY.$TMPSUFFIX.2
  /bin/mv $KEY.$TMPSUFFIX $KEYRESULT 2>/dev/null
fi

# is host an OpenVZ master (CTID=0)?
if [ -d "$OVZ_PRIVATE_DIR" ]
then

  # OpenVZ master: check all virtual private servers, too

  mkdir -p vps/0/$KEYDIR vps/$KEYDIR
  # copy master result?
  [ "$OVZ_CHECK_MASTER_ID" ] && /bin/cp -af $KEYRESULT vps/0/$KEYRESULT

  # collect results of all vps
  for f in $(cd "$OVZ_PRIVATE_DIR";ls)
  do
    mkdir -p vps/$f/$KEYDIR
    # no lockfile per ovm instance, just check age of lockfile
    #AGE=$(($(date +%s)-$(date -r $LOCKFILE +%s 2>/dev/null || echo 0)))
    #[ $AGE -gt 55 ] && exit 0
    # lockfile age-check disabled due to expensive snapshot-mounts
    vzctl exec $f du -smx /var/lib/vz/private/* >>vps/$f/$KEY.$TMPSUFFIX 2>/dev/null \
      || echo 0 >>vps/$f/$KEY.$TMPSUFFIX
    sort -n vps/$f/$KEY.$TMPSUFFIX >vps/$f/$KEY.$TMPSUFFIX.2 2>/dev/null
    /bin/mv vps/$f/$KEY.$TMPSUFFIX.2 vps/$f/$KEY.$TMPSUFFIX 2>/dev/null
    /bin/mv vps/$f/$KEY.$TMPSUFFIX vps/$f/$KEYRESULT 2>/dev/null
  done

  # search for max result, include CTID=0
  RET=0
  HIGHVALUEVPS=""
  for f in $(cd "$OVZ_PRIVATE_DIR";ls) $OVZ_CHECK_MASTER_ID
  do
    echo $f $(sort -n vps/$f/$KEYRESULT 2>/dev/null |tail -n 1 2>/dev/null) >>vps/$KEY.$TMPSUFFIX
    VALUE=$(sort -n vps/$f/$KEYRESULT 2>/dev/null|tail -n 1 2>/dev/null |awk '{print$1}' 2>/dev/null)
    # simple error check to catch integer-test-errors
    [ "$VALUE" -ge 0 ] 2>/dev/null || VALUE="999999999999999999"
    # new max value?
    [ "$VALUE" -gt "$RET" ] && { HIGHVALUEVPS="$f $(cat vps/$f/$KEYRESULT 2>/dev/null)"; RET=$VALUE; }
  done
  [ "$HIGHVALUEVPS" ] && echo $HIGHVALUEVPS >>vps/$KEY.$TMPSUFFIX

  # print max of all results
  echo $RET >>vps/$KEY.$TMPSUFFIX
  /bin/mv vps/$KEY.$TMPSUFFIX vps/$KEYRESULT 2>/dev/null

fi

/bin/rm -f $LOCKFILE
exit 0
