#!/bin/bash

# /etc/nagios-cache.d/loadlogger.sh
# $Id: loadlogger.sh 691 2017-06-04 23:12:04Z fwbergm $

# files in /nagios-cache :
# /nagios-cache/cmdline/loadlogger           result of the host itself
# /nagios-cache/vps/0/cmdline/loadlogger     if host is OpenVZ master (CTID=0) then this is a copy
# /nagios-cache/vps/1079/cmdline/loadlogger  result of virtual private server with CTID=1079
# /nagios-cache/vps/cmdline/loadlogger       summary/max result of all CTIDs (0 included)


# do some checks, set up environment, cd /nagios-cache :
test -e /etc/nagios-cache.d/.ncrc || { echo "ERROR: no /etc/nagios-cache.d/.ncrc found" | tee /dev/stderr; exit 1; }
. /etc/nagios-cache.d/.ncrc

# nagios-check "loadlogger" only for OpenVZ master (CTID=0)
KEY=cmdline/loadlogger
KEYDIR=${KEY%/*}

# NO check for the host itself
#mkdir -p $KEYDIR 2>/dev/null
#knoerre $KEY >>$KEY.0 2>/dev/null \
#  || echo 999999999999999999 >>$KEY.0
#/bin/mv $KEY.0 $KEY 2>/dev/null

# is host an OpenVZ master (CTID=0)?
if [ -d "$OVZ_PRIVATE_DIR" ]
then

  # OpenVZ master: check all virtual private servers, too

  mkdir -p vps/$KEYDIR

  # collect results of all vps
  for f in $(cd "$OVZ_PRIVATE_DIR";ls)
  do
	mkdir -p vps/$f/$KEYDIR
	vzctl exec $f /opt/nagios-fwb/bin/knoerre $KEY >>vps/$f/$KEY.0 2>/dev/null \
      || echo 999999999999999999 >>vps/$f/$KEY.0
    /bin/mv vps/$f/$KEY.0 vps/$f/$KEY 2>/dev/null
  done

  # search for max result, include CTID=0
  RET=0
  HIGHVALUEVPS=""
  for f in $(cd "$OVZ_PRIVATE_DIR";ls)
  do
    echo $f $(cat vps/$f/$KEY) >>vps/$KEY.0
	VALUE=$(tail -n 1 vps/$f/$KEY)
	# simple error check to catch integer-test-errors
	[ "$VALUE" -ge 0 ] 2>/dev/null || VALUE="999999999999999999"
	# new max value?
	[ "$VALUE" -gt "$RET" ] && { HIGHVALUEVPS="$f $(cat vps/$f/$KEY)"; RET=$VALUE; }
  done
  [ "$HIGHVALUEVPS" ] && echo $HIGHVALUEVPS >>vps/$KEY.0

  # print max of all results
  echo $RET >>vps/$KEY.0
  /bin/mv vps/$KEY.0 vps/$KEY 2>/dev/null

fi
