#!/bin/bash

# /etc/nagios-cache.d/myerrlog.sh
# $Id: myerrlog.sh 691 2017-06-04 23:12:04Z fwbergm $
# Examine all (mysqld|mysqld.err) files in $LOGDIR/$HOST
# with "knoerre myerrlog" for errors and write a
# summary

# files in /nagios-cache :
# /nagios-cache/myerrlog/_          summary of results, list the three topmost HOSTS
# /nagios-cache/myerrlog/db018      result of db1018

# do some checks, set up environment, cd /nagios-cache :
test -e /etc/nagios-cache.d/.ncrc || { echo "ERROR: no /etc/nagios-cache.d/.ncrc found" | tee /dev/stderr; exit 1; }
. /etc/nagios-cache.d/.ncrc

# nagios-check "myerrlog"
# mysqlerr/var/log/mysql/mysqld.err mysqlerr/var/log/cluster/mysql/db1053/mysqld
LOGDIR=/var/log/cluster
KEY=myerrlog/
ulimit -t 300
ulimit -i 1
TMPSUFFIX=$(date +%M)
KEYDIR=${KEY%/*}
[ "$KEY" = "$KEYDIR" ] && KEYDIR=.
mkdir -p $KEYDIR 2>/dev/null
KEYRESULT=$KEY
[ -d "$KEYRESULT" ] && KEYRESULT="$KEYRESULT"_
LOCKFILE="$KEYRESULT".lock

[ -d $LOGDIR ] || exit 0

if [ -e $LOCKFILE ]
then
  # there's still a nc-process running
  echo 999999999999999997 >>$KEYRESULT
  AGE=$(($(date +%s)-$(date -r $LOCKFILE +%s 2>/dev/null || echo 0)))
  [ $AGE -gt 1800 ] && /bin/rm -f $LOCKFILE
  exit 0
fi

echo $$ >$LOCKFILE
/bin/rm -f $KEYRESULT.$TMPSUFFIX

find $LOGDIR/ -maxdepth 3 \( -name mysqld.err -o -name mysqld \) -type f 2>/dev/null | while read LOGFILE
do
  HOST=$(dirname $LOGFILE|sed 's,.*\/,,')
  #echo $LOGFILE $HOST $TMPSUFFIX
  knoerre mysqlerr$LOGFILE >>$KEYDIR/$HOST.$TMPSUFFIX 2>/dev/null \
    || echo 999999999999999999 >>$KEYDIR/$HOST.$TMPSUFFIX
  /bin/mv $KEYDIR/$HOST.$TMPSUFFIX $KEYDIR/$HOST 2>/dev/null
  echo $(tail -n 1 $KEYDIR/$HOST 2>/dev/null || echo 9999) $HOST
done \
| sort -rn \
| head -n 3 \
| echo $(while read VALUE HOST
do
  echo $HOST=$VALUE
done) >>$KEYRESULT.$TMPSUFFIX

HIGHHOST=$(cut -d\  -f1 $KEYRESULT.$TMPSUFFIX|cut -d= -f1)
HIGHVALUE=$(cut -d\  -f1 $KEYRESULT.$TMPSUFFIX|cut -d= -f2)
echo $HIGHVALUE >>$KEYRESULT.$TMPSUFFIX

cat $KEYDIR/$HIGHHOST 2>/dev/null|sed 's,^,'$HIGHHOST' ,' >$KEYRESULT
cat $KEYRESULT.$TMPSUFFIX >>$KEYRESULT 2>/dev/null
/bin/rm $KEYRESULT.$TMPSUFFIX 2>/dev/null

/bin/rm -f $LOCKFILE
exit 0
