/*
 * check-nagiosprocess.c
 * counts the number of nagios daemons
 * 0 daemons will output 9999
 * FWB 8/2009
 */

#define NAGIOS_PROCESS_NAME "nagios3"
/*#define NAGIOS_PROCESS_NAME "nagios"*/

#undef RUN_AS_CGI_PROGRAM
#define RUN_AS_CGI_PROGRAM

#include <unistd.h>
#include <sys/vfs.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include "../lib/libfwb.h"
#define BUFSIZE 512
#define NAGIOS_PROCESS_STRING " ("NAGIOS_PROCESS_NAME") "

int main(int argc,char *const argv[]) {
  register char *p;
  register char *r;
  DIR *dptr;
  struct dirent *eptr;
  int pstat;
  char pathname[64];   /* /proc/2147483648/stat */
  char statbuf[BUFSIZE+1];
  uint32_t i;

  /**
   * step through /proc/ files
   */
  dptr = opendir("/proc");
  if (dptr == NULL) _exit(1);
  for (i=0; (eptr=readdir(dptr))!=NULL; ) {
    if ((*(eptr->d_name) < '0') || (*(eptr->d_name) > '9')) continue;
    p = stpcpy(pathname, "/proc/");
    p = stpcpy(p, eptr->d_name);
    strcpy(p, "/stat");
    pstat = open(pathname, O_RDONLY);
    if (pstat > 0) {
      if (read(pstat, statbuf, BUFSIZE) > 2) {
        statbuf[BUFSIZE] = '\0';
        r = strchr(statbuf, ' ');
        /* if (!r) continue; we trust the kernel ;-) */
        if (!strncmp(r, NAGIOS_PROCESS_STRING, sizeof(NAGIOS_PROCESS_STRING)-1)) { 
          *r = 0;
          p = r + sizeof(NAGIOS_PROCESS_STRING);
          ++p;
          // ppid == 1 ?
          if ((*p++ == '1') && (*p++ == ' ')) {
            p = strchr(p, ' ') + 1;
            r = strchr(p, ' ');
            *r = 0;
            /* session leader? */
            if (!strcmp(statbuf, p)) i++;
          }
        }
      }
      close(pstat);
    }
  };
  closedir(dptr);

  /**
   * write the data into buffer and print it out
   */
  p = pathname;
#ifdef RUN_AS_CGI_PROGRAM
  p = stpcpy(p, "Content-Type: text/plain\n\n");
#endif
  if (i) {
    p = sputlongs(p,i);
  } else {
    p = stpcpy(p, "9999");
  }
  write1(pathname, p-pathname);
  _exit(0);
}
