/*
 * $Id: nagios-check-longuserprocesses.c 691 2017-06-04 23:12:04Z fwbergm $
 * shows time in mins of longest running process of given minimum uid
 */

#if __GNUC__ > 5
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
#endif

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <features.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <stdint.h>
#ifndef _USE_DEPRECATED_STR2UINT63
#define _USE_DEPRECATED_STR2UINT63
#endif
#include "../lib/libfwb.h"


static const char msg[]="missing arg (min UID)\n";
static const char uid_string[]="Uid:\011";
/* uid_max > 65535 doesn't work on 32-Bit systems */
static const uint32_t uid_max=200000;

#define PATHNAME_SIZE 4096
#define BUF_SIZE 8192

int str_index2(const char* a, const char* b) {
  register const char* s=a;
  register int t=strlen(b);
  register const char* u=s+strlen(s)-t;
  for (;;) {
    if (s>u) break;
    if (!strncmp(s,b,t)) return (s-a)+t;
    ++s;
  }
  return -1;
}

int field_index(const char *s,int index) {
  register const char* t=s;

  if(index<1) return 0;
  for(;;) {
    if(*t=='\0') return -1;
    if(*t++==' ') {
      index--;
      if(!index) return t-s;
    }
  }
}

int main(int argc,char *const argv[]) {
  register char *p;
  register int64_t lval;
  register int16_t c;
  register uint32_t uid_min;
  register int rf;
  register int64_t uptime;
  register uint32_t mins;
  register uint32_t pid;
  register uint32_t maxtime_mins;
  register uint32_t maxtime_pid;
  char name[2*PATHNAME_SIZE];
  char filebuf[BUF_SIZE+1];
  DIR *dptr;
  struct dirent *eptr;


  if (argc<2) { write2(msg,strlen(msg)); _exit(2); }
  maxtime_mins=maxtime_pid=0;
  uid_min=str2uint63(argv[1]);

  /* args boundary checks */
  c=2;
  for(;c<argc;c++) {
    if (strlen(argv[c]) > PATHNAME_SIZE) goto err_exit;
  }

  /* get uptime */
  p = name;
  stpcpy(p, "/proc/uptime");
  rf=open(p, O_RDONLY);
  if(rf<1) goto err_exit;
  p = filebuf;
  if(read(rf,p,BUF_SIZE)<2) { close(rf); goto err_exit; }
  p[BUF_SIZE]='\0';
  uptime=str2uint63(p)*100;
  close(rf);

  dptr=opendir("/proc");
  if(dptr==NULL) goto err_exit;

  for(;(eptr=readdir(dptr))!=NULL;) {

    if ((*eptr->d_name < '0') || (*eptr->d_name > '9')) continue;

    /* don't use stat/lstat(), read .../status, works on very old linux and 64-Bit too */
    p = stpcpy(name, "/proc/");
    p = stpcpy(p, eptr->d_name);
    p = stpcpy(p, "/status");
    p = name;

    rf = open(p, O_RDONLY);
    if (rf <= 0) continue;
    p = filebuf;
    if (read(rf, p, BUF_SIZE) <2 ) goto next_file;
    p[BUF_SIZE] = '\0';
    if ((c = str_index2(p, uid_string)) < 0) goto next_file;
    p += c; /* points to real uid */
    c = field_index(p, 1);
    lval = str2uint63(p+c); /* effective user id */
    if ((lval < uid_min) || (lval > uid_max)) goto next_file;
    close(rf);

    p = stpcpy(name, "/proc/");
    p = stpcpy(p, eptr->d_name);
    p = stpcpy(p, "/stat");
    p = name;

    /* read process stat */
    rf=open(p, O_RDONLY);
    if(rf>0) {
      p = filebuf;
      if(read(rf,p,BUF_SIZE)<2) goto next_file;
      p[BUF_SIZE]='\0';
      c=2;
      for(;c<argc;c++) {
        p = stpcpy(name, " (");
        p = stpcpy(p, argv[c]);
        p = stpcpy(p, ") ");
        if(str_index2(filebuf,name)>=0) goto next_file;
      }
      p = filebuf;
      pid=str2uint63(p); /* no error check */
      c=field_index(p,21);
      lval=str2uint63(p+c);
      mins=(uptime-lval)/6000;
      if(mins>maxtime_mins) { maxtime_mins=mins; maxtime_pid=pid; }

      next_file:
      close(rf);
    }

  }
  closedir(dptr);
  putlong(maxtime_mins);
  putlong(maxtime_pid);
  _exit(0);

  err_exit:
  putlong(9999);
  _exit(1);
}
