/*
 * $Id: postfix-mailqsize.c 691 2017-06-04 23:12:04Z fwbergm $
 * postfix-mailqsize.c
 * a very fast and simple postfix mailq size display
 * Frank Bergmann
 * v0.3 03/2008: don't check maildrop, just active, deferred and incoming
 * v0.2 02/2008: also check for files in subdirs not only subsubdirs,
 *               needed on different postfix version on spmXX
 * v0.1 12/2007
 */

#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include "../lib/libfwb.h"

static const char *spooldir = "/var/spool/postfix/";
static const char *subdirs[]  =  {
  "active/",
  "deferred/",
/*  "maildrop/", */
/*  "incoming/", */
  0
};
static const char *subsubdirs[]  =  {
  "0/", "1/", "2/", "3/", "4/", "5/", "6/", "7/",
  "8/", "9/", "A/", "B/", "C/", "D/", "E/", "F/",
  0
};

int main() {
  register uint32_t countsub;
  register uint32_t countsubsub;
  register DIR *dptr;
  register const char **ssd;
  register const char **sd;
  register int ret;
  char d[80];

  ret = countsub = 0;
  for (sd = subdirs;*sd;++sd) {
    *d = '\0';
    strcat(d, spooldir);
    strcat(d, *sd);
    dptr = opendir(d);
    if (dptr != 0) {
      for (;readdir(dptr);++countsub);
      if (countsub > 1) countsub -= 2; /* . and .. dirs */
      closedir(dptr);
    } else {
      if (errno == EACCES) ret = 1;
    }
    for (ssd = subsubdirs;*ssd;++ssd) {
      *d = '\0';
      strcat(d, spooldir);
      strcat(d, *sd);
      strcat(d, *ssd);
      dptr = opendir(d);
      if (dptr != 0) {
        if (countsub > 0) --countsub;
        countsubsub = 0;
        for (;readdir(dptr);++countsubsub);
        if (countsubsub > 1) countsubsub -= 2; /* . and .. dirs */
        countsub += countsubsub;
        closedir(dptr);
      } else {
        if (errno == EACCES) ret = 1;
      }
    }
  }
  if (ret == 0) putlong(countsub);
  _exit(ret);
}
