/**
 * $Id: check_mounts.c 691 2017-06-04 23:12:04Z fwbergm $
 * check_mounts.c
 *
 * compile with i.e.
 *   diet gcc -O2 -Wall -o check_mounts check_mounts.c;sstrip check_mounts
 */

#if __GNUC__ > 5
#pragma GCC diagnostic ignored "-Wunused-const-variable"
#endif

#define TESTMULTIREQUESTS
#undef TESTMULTIREQUESTS

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <features.h>

#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <poll.h>
#include <setjmp.h>
#include <time.h>
#include <string.h>
#include <stdint.h>

#include "lib/libfwb.h"
#include "knoerre/knoerred.h"

#define READBYTES 4096

#define NAGIOSRESULTPREFIXSTRING "MOUNTS "
#define NAGIOSRESULTOKSTRING "OK "
#define NAGIOSRESULTWARNSTRING "WARNING "
#define NAGIOSRESULTCRITSTRING "CRITICAL "
#define NAGIOSRESULTUNKNOWNSTRING "UNKNOWN "
enum { OK, WARNING, CRITICAL, UNKNOWN };

char *knoerre_global_buf;
char *knoerre_global_buf_cursor;

static const char version_string[] = "version_string: $Id: check_mounts.c 691 2017-06-04 23:12:04Z fwbergm $";

static const char usage[]="usage: check_mounts [-A(0|1|2|3|4)] -w warnlevel -c critlevel [-u UNIT]\n";
static void printusage() { write1(usage, sizeof usage - 1); exit(1); }

static void print(const char *str) {
  register uint32_t i;

  if (!str) return;
  if (!*str) return;
  i = 1;
  for (;;) { if (!str[i]) break; ++i; }
  write1(str, i);
}

int main(int argc,char *const argv[]) {
  register char *p;
  int tmpint;
  ssize_t bytecount;
  char resultbuf[READBYTES+1];
  char linebuf[READBYTES+1];
  long warn,crit;
  uint64_t value;
  int retvalue;
  char *unit;
  int warni,criti;
  char *lastline;
  int mode_show_additional_data;

#ifdef TESTMULTIREQUESTS
  int testmultirequests;

  for(testmultirequests=0;testmultirequests<20000;testmultirequests++) {
#endif

  knoerre_global_buf = resultbuf;
  knoerre_global_buf_cursor = knoerre_global_buf;
  unit = NULL;
  warn = warni = crit = criti = 0;
  mode_show_additional_data = 1;
  retvalue = 2;

  if (argc < 2) printusage();
  for(tmpint=1; tmpint<argc; ++tmpint) {
    if (*argv[tmpint] == '-') {
/*
      if (argv[tmpint][2] != '\0') printusage();
*/
      switch (argv[tmpint][1]) {
        case 'h': printusage(); break;
        case 'w': ++tmpint; warni=tmpint; if(!str2uint64(argv[tmpint], &value)) printusage(); warn=(long) value; break;
        case 'c': ++tmpint; criti=tmpint; if(!str2uint64(argv[tmpint], &value)) printusage(); crit=(long) value; break;
        case 'u': ++tmpint; unit=argv[tmpint]; break;
        case 'A': mode_show_additional_data = argv[tmpint][2] - '0';
                  if (mode_show_additional_data < 0) printusage();
                  if (mode_show_additional_data > 4) printusage();
                  break;
        default: printusage(); break;
      }
    }
  }

  do_check_mounts();
  bytecount = knoerre_global_buf_cursor - knoerre_global_buf;

  /* get last line */
  for (tmpint = bytecount;;) {
    --tmpint;
    if (!tmpint) break;
    if ((unsigned char) resultbuf[tmpint] >= 32) break;
  }
  p = resultbuf + tmpint + 1;
  *p = 0;
  for (;;) {
    --tmpint;
    if (tmpint < 0) break;
    if ((unsigned char) resultbuf[tmpint] < 32) break;
  }
  /* skip blanks and ctrl chars */
  for (;;) {
    ++tmpint;
    if ((unsigned char) resultbuf[tmpint] > 32) break;
  }
  p = resultbuf + tmpint;
  lastline = p;

  /* construct output line */
  retvalue = OK;
  value = (uint64_t) *((uint32_t*)(resultbuf+bytecount+1));
  if (value > warn) retvalue = WARNING;
  if (value > crit) retvalue = CRITICAL;
  switch (retvalue) {
    case 0: p = stpcpy(linebuf, NAGIOSRESULTPREFIXSTRING NAGIOSRESULTOKSTRING "- "); break;
    case 1: p = stpcpy(linebuf, NAGIOSRESULTPREFIXSTRING NAGIOSRESULTWARNSTRING "- ");break;
    case 2: p = stpcpy(linebuf, NAGIOSRESULTPREFIXSTRING NAGIOSRESULTCRITSTRING "- "); break;
    default: p = stpcpy(linebuf, NAGIOSRESULTPREFIXSTRING NAGIOSRESULTUNKNOWNSTRING "- "); retvalue=3; break;
  }

  p = stpcpy(p, lastline);
  if (unit != NULL) {
    p = stpcpy(p, unit);
  }
  /* append additional data if retvalue >= mode_show_additional_data */
  if (retvalue >= mode_show_additional_data) {
    /* get line before last line */
    for (tmpint = lastline - resultbuf;;) {
      if (!tmpint) break;
      --tmpint;
      if ((unsigned char) resultbuf[tmpint] >= 32) break;
    }

    if (lastline != (resultbuf+tmpint)) {
      resultbuf[tmpint + 1] = 0;
      for (;;) {
        --tmpint;
        if (tmpint < 0) break;
        if ((unsigned char) resultbuf[tmpint] < 32) break;
      }
      /* skip blanks and ctrl chars */
      for (;;) {
        ++tmpint;
        if ((unsigned char) resultbuf[tmpint] > 32) break;
      }
    }
    *p = ' '; ++p;
    *p = '('; ++p;
    p = stpcpy(p, resultbuf + tmpint);
    *p = ')'; ++p;
  }
  /* performance data */
  p = stpcpy(p, " | mounts=");
  p = stpcpy(p, lastline);
  if (unit != NULL) {
    while (*unit == ' ') ++unit;
    for (tmpint = 0;; ++tmpint)
      if (unit[tmpint] <= 32) break;
    unit[tmpint] = 0;
    p = stpcpy(p, unit);
  }
  *p = ';'; ++p;
  p = stpcpy(p, argv[warni]);
  *p = ';'; ++p;
  p = stpcpy(p, argv[criti]);
  *p = '\n'; ++p;
  *p = 0;
  print(linebuf);
#ifdef TESTMULTIREQUESTS
  }
#endif
  _exit(retvalue);
}
