#!/usr/bin/env python

"""
This small "webserver" executes specified commands on localhost.
It is designed to be queried by a nagios check_remote_by_http plugin.

Its resource config file is '/etc/pyrrerc' and it must contain an array
named 'cmds' with URLs to server and corresponding commands.
Example:

cmds = {
    '/loadavg' : 'cat /proc/loadavg',
    '/load1' : '',
    '/mailqs1': '/usr/sbin/postfix-mailqsize',
}

FWB 2008-06-22
"""

HOSTS_ALLOW = [ 
    "127.0.0.1",
    "192.168.1.1"
]

cmds = {
    '/digitemp': 'digitemp -t0'
}


import imp
import os
import re
import BaseHTTPServer

class PyrreHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):

    def do_GET(self):
        """ Serve a GET request. """
        path = self.path
        if path == '/' or not path in ['/'] + self.cmds.keys():
            #
            #   HTTP-Redirect for invalid URLs
            #
            self.send_response(302, "Found")
            self.send_header("Location", "http://www.tuxad.com/")
            self.end_headers()
            self.wfile.write("""<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html><head>
<title>302 Found</title>
</head><body>
<h1>Found</h1>
<p>The document has moved <a href="http://www.tuxad.com/">here</a>.</p>
<hr>
</body></html>""")
            return None
        self.send_response(200)
        self.send_header("Content-type", 'text/plain')
        self.end_headers()
        #
        #   serve internal commands
        #
        cmd = self.cmds[self.path]
        if path == '/digitemp':
            digitemp = os.popen(cmd).read()
            re_celsius = re.compile(" C: ")
            celsius = digitemp[re_celsius.search(digitemp).end():].split(".")[0]
            nagios = ''.join([ digitemp, "\n", celsius, "\n" ])
            self.wfile.write(nagios)
            return None
        self.wfile.write(os.popen(cmd).read())
        return None


def verify_request(request, client_address):
    ip = client_address[0]
    if ip in HOSTS_ALLOW:
        return True
    return False


def main(HandlerClass = PyrreHTTPRequestHandler, ServerClass = BaseHTTPServer.HTTPServer):
    HandlerClass.server_version = 'Pyrre/0.8'
    HandlerClass.sys_version = ''
    #execfile("/etc/pyrrerc")
    HandlerClass.cmds = cmds
    os.chdir("c:\digitemp")
    pyrreserver = ServerClass(('', 10488), HandlerClass)
    # don't define class, just override one method:
    pyrreserver.verify_request = verify_request
    pyrreserver.serve_forever()


if __name__ == '__main__':
    main()
