#!/usr/bin/env python

"""
This small "webserver" executes specified commands on localhost.
It is designed to be queried by a nagios check_remote_by_http plugin.

Its resource config file is '/etc/pyrrerc' and it must contain an array
named 'cmds' with URLs to server and corresponding commands.
Example:

cmds = {
    '/loadavg' : 'cat /proc/loadavg',
    '/load1' : '',
    '/mailqs1': '/usr/sbin/postfix-mailqsize',
}

FWB 2008-06-22
"""

HOSTS_ALLOW = [ 
    "127.0.0.1",
    "172.16.12.84"
]

cmds = {
    'load1' : '',
    'load5' : '',
    'load15' : '',
    'diskusagelocal': '',
    'timediff' : '',
    'cmp' : '',
    'cmdline' : '',
}

ERROR_COMMON_VALUE_STRING = '999999999999999999'

import imp
import BaseHTTPServer, os
import time
import sys
import re

class PyrreHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):

    def do_check_diskusagelocal(self):
        max_percent = 0
        df = os.popen('/bin/df -l -k')
        df.readline()
        for l in df.readlines():
            ls = l.split()
            if len(ls) == 1:
                olddev = ls[0]
            else:
                if len(ls) < 6: ls.insert(0,olddev)
            percent = int(ls[4].rstrip('%'))
            if percent > max_percent:
                max_l = ls
                max_percent = percent
        df.close()
        return '%s %d%%, %d/%d/%d GB all/used/free\n%d\n' % \
            (max_l[5], max_percent, int(max_l[1])/1024/1024, int(max_l[2])/1024/1024, int(max_l[3])/1024/1024, max_percent)


    def getload(self):
        #load = os.getloadavg()
        lfile = open("/proc/loadavg", "r")
        load = lfile.read().split(" ")
        lfile.close()
        return load

    def main_error_redirect_exit(self):
        #
        #   HTTP-Redirect for invalid URLs
        #
        self.send_response(302, "Found")
        self.send_header("Location", "http://www.meyle-mueller.de/")
        self.end_headers()
        self.wfile.write("""<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html><head>
<title>302 Found</title>
</head><body>
<h1>Found</h1>
<p>The document has moved <a href="http://www.meyle-mueller.de/">here</a>.</p>
<hr>
</body></html>""")
        return None


    def do_GET(self):
        """ Serve a GET request. """
        if self.path == '/': self.main_error_redirect_exit()
        #
        #   serve external commands
        #
        args = self.path.split('/')[1:]
        cmd = 'NONE'
        try:
            cmd = self.cmds[args[0]]
        except:
            self.main_error_redirect_exit()
        self.send_response(200)
        self.send_header("Content-type", 'text/plain')
        self.end_headers()
        #
        #   serve internal commands
        #
	if args[0] == 'load1':
            self.wfile.write('%d\n' % int(os.getloadavg()[0]))
            return None
	if args[0] == 'load5':
            self.wfile.write('%d\n' % int(os.getloadavg()[1]))
            return None
	if args[0] == 'load15':
            self.wfile.write('%d\n' % int(os.getloadavg()[2]))
            return None
	if args[0] == 'diskusagelocal':
            self.wfile.write(self.do_check_diskusagelocal())
            return None
	if args[0] == 'timediff':
            servertime = abs(int(args[1]))
            mytime = time.time()
            self.wfile.write('%d\n%d\n' % (mytime, abs(int(mytime - servertime))))
            return None
	if args[0] == 'cmp':
            if args[2:] == ['proc', 'sys', 'kernel', 'hostname']:
                # fake linux proc interface
                #myshorthostname = re.sub('\..*', '', os.uname()[1])
                #myshorthostname = os.uname()[1]
                # on system startup the hostname will be set by configd (see scutil(8)) as fqdn
                myshorthostname = os.uname()[1].split(".")[0]
                if args[1] == myshorthostname:
                    self.wfile.write('%s\n0\n' % myshorthostname)
                else:
                    self.wfile.write('%s\n1\n' % myshorthostname)
                return None
            self.main_error_redirect_exit()
	if args[0] == 'cmdline':
            cmdline = "/".join(args[1:])
            cmd = "ps axuw|grep -v grep|grep -c %s" % (cmdline)
            try:
                fd_popen = os.popen(cmd)
                output = fd_popen.read()
                fd_popen.close()
            except:
                self.wfile.write('error on %s/%s\n%s\n' % (args[0], args[1], ERROR_COMMON_VALUE_STRING))
            if output == '':
                self.wfile.write('no output from %s\n%s\n' % (args[0], ERROR_COMMON_VALUE_STRING))
            else:
                count = int(output)
                if count == 0: count = 999999999999999999
                output = "%s\n%d\n" % (cmdline, count)
                self.wfile.write(output)
	    return None
        #
        #   serve external commands
        #
        try:
            fd_popen = os.popen(cmd)
            output = fd_popen.read()
            fd_popen.close()
        except:
            self.wfile.write('error on %s\n%s\n' % (args[0], ERROR_COMMON_VALUE_STRING))
	    return None
        if output == '':
            self.wfile.write('no output from %s\n%s\n' % (args[0], ERROR_COMMON_VALUE_STRING))
        else:
            self.wfile.write(output)
	return None


def verify_request(request, client_address):
    ip = client_address[0]
    if ip in HOSTS_ALLOW:
        return True
    return False


def main(HandlerClass = PyrreHTTPRequestHandler, ServerClass = BaseHTTPServer.HTTPServer):
    HandlerClass.server_version = 'Pyrre/0.8'
    HandlerClass.sys_version = ''
    try:
        execfile("/etc/pyrrerc")
    except:
        print "WARNING: no /etc/pyrrerc found, using defaults"
        HandlerClass.cmds = cmds
    pyrreserver = ServerClass(('', 8889), HandlerClass)
    # don't define class, just override one method:
    pyrreserver.verify_request = verify_request
    pyrreserver.serve_forever()


if __name__ == '__main__':
    main()
