/**
 * actionwhenfileold.c
 * Exec a command if a file reaches a maximum age (mtime).
 *
 * Compile with:
 * diet gcc -O2 -Wall -o actionwhenfileold actionwhenfileold.c && sstrip actionwhenfileold
 * Frank Bergmann, www.tuxad.com, 11/2008
 * $Id: actionwhenfileold.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>
#include <time.h>
#include <stdint.h>

#ifndef write1
#include <sys/syscall.h>
#define write1(b,c) (syscall(__NR_write, 1, b, c))
#endif
#ifndef write2
#include <sys/syscall.h>
#define write2(b,c) (syscall(__NR_write, 2, b, c))
#endif

#define SLEEP_INTERVAL 10

static const char usage[]="usage: actionwhenfileold /path/to/file max-age-in-minutes /path/to/command [arg(s)]\n";
static void printusage() { write1(usage, sizeof usage - 1); _exit(1); }

static char *str2uint32(register char *s, uint32_t *val) {
  register char c;
  uint32_t i;
  c = *s;
  if ((c < '0') || (c > '9')) return 0;
  ++s;
  i = c - '0';
  for (;;) {
    c = *s;
    if ((c < '0') || (c > '9')) {
      if (val) *val = (uint32_t) i;
      return s;
    } else {
      i = (i * 10) + c - '0';
      ++s;
    }
  }
}

int main(int argc,char *argv[]) {
  int childstatus;
  pid_t child;
  uint32_t maxage;
  time_t tp;
  struct stat statbuf;
  
  maxage = 0;

  /* only two checks are made */
  if (argc < 4) printusage();
  if (!str2uint32(argv[2], &maxage)) printusage();

  child = 0;

  /* main loop */
  for (;;) {

    if (!stat(argv[1], &statbuf)) {

      tp = statbuf.st_mtime;
      tp = time(0) - tp;
      if (tp < 0) tp = 0;
      tp = tp / 60;
      if (tp < maxage) goto sleep_and_waitpid;

      child = fork();
      if (child == -1) {
        /* error on fork() */
        write2("can't fork\n", 11);
        child = 0;
        goto sleep_and_waitpid;
      }
      if (!child) {
        /* child */
        execve(argv[3],argv+3,0);
        write2("exec err\n",9);
        _exit(111);
      }

    }

    sleep_and_waitpid:
    sleep(SLEEP_INTERVAL);

    if (child) {
      if (waitpid(child, &childstatus, WNOHANG) == child) {
        if (WIFEXITED(childstatus) || WIFSIGNALED(childstatus)) {
          /* child exited */
          child = 0;
          continue;
        }
      }
      kill(child, 9);
      goto sleep_and_waitpid;
    }

  }

  /* never reached */
  _exit(0);
}
