/*
 * check-nfs.c
 * check NFS access by opening a file in a child and termintate child if it blocks
 *
 * Frank Bergmann, 07/2015
 */

#define _FILE_OFFSET_BITS 64
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <signal.h>
#include <string.h>
#include <setjmp.h>

#ifndef write1
#include <sys/syscall.h>
#define write1(b,c) (syscall(__NR_write, 1, b, c))
#endif
#ifndef write2
#include <sys/syscall.h>
#define write2(b,c) (syscall(__NR_write, 2, b, c))
#endif

static pid_t do_check_nfs_pid;

static void check_nfs_sig_handler(int num) {
  int child_status;

  /* ignore all signals, check_nfs_sig_handler should only be called once */
  signal(SIGCHLD, SIG_IGN);

  /* match do_check_nfs_pid? */
  if (waitpid(do_check_nfs_pid, &child_status, 0) != do_check_nfs_pid) goto leave_handler;
  /* child killed? */
  if (WIFSIGNALED(child_status)) _exit(1);
  /* child exited normally? */
  if (!WIFEXITED(child_status)) goto leave_handler;
  /* return code is 0? */
  if (WEXITSTATUS(child_status) != 0) goto leave_handler;

  /*
   * at this point we are sure, that our child did a good job, terminated normally
   * and had a return value of 0
   */
  _exit(0);

  leave_handler:
  return;
}


int main(int argc, char *argv[]) {

  if (argc < 2) _exit(1);
  
  /* catch a SIGCHILD of a well teminated child */
  if (signal(SIGCHLD, check_nfs_sig_handler) == SIG_ERR) goto error;

  if ((do_check_nfs_pid = fork()) < 0) {
    goto error;
  } else if (do_check_nfs_pid == 0) {
    /* child: try to open nfs-file */
    int i = open(argv[1], O_RDONLY);
    if (i == -1) _exit(1);
    close(i);
    _exit(0);
  }

  /* parent: sleep a short time while child maybe does his job */
  sleep(5);
  /* this kill will not be executed if check_nfs_sig_handler() will exit our process */
  if (kill(do_check_nfs_pid, 9) == 0) sleep(5);
  /*
   * if kill was successful (child did still exist and did not exit) then 
   * sleep a short time while sighandler maybe does his job
   */

  error:
  _exit(1);
}

