/*
 * griller.c
 * "grills" processes which run too long
 *
 * Due to educational purposes all following comments are made in german language
 *
 * $Id: griller.c 691 2017-06-04 23:12:04Z fwbergm $
 */
#include <stdio.h>

#include "griller.h"

static const char usage[]="usage: griller [-s] [-c] [-u] [-U] [-L proclimit] [min-uid [max-uid]]\n";
static inline void printusage() { write1(usage, sizeof usage - 1); _exit(EX_USAGE); }

#define USERS_NAME_BUF_SIZE MAX_USERS*33

static const char uid_marker[]="Uid:\011";
/* minimaler Abstand von uid_marker in /proc/PID/status */
#define UID_MIN_OFFSET 45
/* bringt minimale Beschleunigung ;-) */

/* ab wieviel Prozent des KILL_LEVEL werden die Daten eines Prozesses geloggt? */
#define LOGGING_THRESHOLD_PERCENT 75

/* Schwellwert fuer Laufzeitbegrenzung und Ausgabe der stats */
/*
static const int load_threshold = 5;
*/
static int load_threshold;

uint32_t kill_level;
uint32_t min_uid;
uint32_t max_uid;
int killing_off;
uint32_t maxloadsoft;
uint32_t maxloadhard;
uint32_t procsoftlimit;
uint32_t prochardlimit;
const char *cmdline_excludes[MAX_CMDLINE_EXCLUDES+1];
const char *userignores[MAX_USERIGNORES+1];
const char *userquadtimes[MAX_USERIGNORES+1];
uint32_t rss_softlimit;

static const char *get_username_by_uid(uint32_t uid, const char **user_names, uint32_t *user_ids) {
  register uint32_t i=0;
  for (;;) {
    if (user_ids[i] == uid) return user_names[i]; ++i; if (user_names[i] == 0) return 0;
    if (user_ids[i] == uid) return user_names[i]; ++i; if (user_names[i] == 0) return 0;
    if (user_ids[i] == uid) return user_names[i]; ++i; if (user_names[i] == 0) return 0;
    if (user_ids[i] == uid) return user_names[i]; ++i; if (user_names[i] == 0) return 0;
  }
  return 0;
}

int main(int argc,char *const argv[]) {
  register char *p;
  register char *s;
  uint32_t val32;
  uint64_t val64;
  ssize_t c;
  uint32_t process_age;
  uint64_t uptime;
  uint32_t load1;
  int rf;
  uint32_t pid;
  int pid_s_len;
  uint32_t uid;
  int32_t pagesize;
  char filename[2*BUFSIZE+1];
  char filebuf[BUFSIZE+1];
  char wbufspace[WRITEBUFSIZE+1];
  char cmdline[BUFSIZE+1];
  DIR *procdirp;
  DIR *sdirp;
  int pw_name_len;
  struct dirent *procdirep;
  struct dirent *sdirep;
  uint32_t uidp[MAX_USERS+1];
  uint32_t uidcnt[MAX_USERS];
  uint32_t max1uid, max2uid, max3uid, max1cnt, max2cnt, max3cnt;
  uint32_t proc_count, kill_count, kill_score, load_factor;
  int kill_proc;
  const char *pwdbuf;
  off_t pwdlen;
  int jiffies_per_second=100;
  char users_name_buf[USERS_NAME_BUF_SIZE+1];
  char *user_names[MAX_USERS+1];
  const char *user_name;
  uint16_t user_name_len;
  uint32_t user_ids[MAX_USERS+1];
  uint32_t user_index;
  uint64_t rss;

  /**
   * uptime wird fuer Berechnung von jiffies_per_second
   * benoetigt und sollte als erstes bestimmt werden
   */
  uptime = get_uptime_in_seconds();
  /* HZ bzw. jiffies_per_second = start_time_griller / uptime */
  jiffies_per_second = get_jiffies_per_second(uptime);

  if (argc > 1) {
    if ((argv[1][0] == '-') && (argv[1][1] == 'h')) {
      printusage();
    }
  }

  min_uid = 500;
  max_uid = 0; /* 0 = off */
  pagesize = pw_name_len = 0;
  max1uid = max1cnt = 0;

  get_envvars();
  load_threshold = maxloadsoft / 8;

  /* Ausgabepuffer initialisieren */
  p = wbufspace;
  buffer wbuf = { p, p, p+WRITEBUFSIZE, 1 };

#ifdef OBSOLETE_STUFF
  pagesize = sysconf(_SC_PAGESIZE);
  if (pagesize < 1) pagesize = 4096; /* Ausweichmoeglichkeit bei Fehler */
#else
#if ( defined(__alpha__) || defined(__sparc__) )
  pagesize = 8192;
#else
  pagesize = 4096;
#endif
#endif

  /* Mapping von /etc/passwd zum Konvertieren UID => Username */
  p = (char *) map_passwd_file(&pwdbuf, &pwdlen);
  if (p == (char *) -1) _exit(EX_OSFILE);

  /* lese Daten aus /etc/passwd ein */
  {
    char *bufp = users_name_buf;
    char *buf_end = bufp + USERS_NAME_BUF_SIZE;
    const char *pwd_end = pwdbuf + pwdlen;
    
    user_index = 0;
    for (;;) {

      /* kopiere Accountname in Puffer */
      user_names[user_index] = bufp;
      for (;;) {
        if (p >= pwd_end) goto passwd_read;
        if (bufp >= buf_end) goto passwd_read;
        if (*p == ':') break;
        if (*p == '\n') break;
        *bufp = *p;
        ++bufp;
        ++p;
      }
      *bufp = 0;
      ++bufp;

      /* gehe zu Spalte UID */
      if (*p != ':') goto passwd_read;
      ++p;
      for (;;) {
        if (p >= pwd_end) goto passwd_read;
        if (*p == ':') break;
        if (*p == '\n') break;
        ++p;
      }
      if (*p != ':') goto passwd_read;
      ++p;

      /* kopiere UID */
      p =  str2uint32(p, &val32); /* Effective UID */
      if (!p) goto passwd_read;
      user_ids[user_index] = val32;

      /* gehe zum Zeilenende */
      for (;;) {
        if (p >= pwd_end) goto passwd_read;
        if (*p == '\n') break;
        ++p;
      }
      
      ++p;
      if (bufp >= buf_end) break;
      ++user_index;
      if (user_index >= MAX_USERS) break;
    }
  }
  passwd_read:
  user_names[user_index] = 0;
  user_ids[user_index] = 0;
  munmap((void *) pwdbuf, pwdlen);

  /* gebe Startmeldung mit Konfiguration aus */
  buffer_put(&wbuf, "\nstarting griller, KILL_LEVEL=", 30);
  buffer_putuint64(&wbuf, kill_level);
  buffer_put(&wbuf, " MINUID=", 8);
  buffer_putuint64(&wbuf, min_uid);
  buffer_put(&wbuf, " MAXUID=", 8);
  buffer_putuint64(&wbuf, max_uid);
  buffer_put(&wbuf, " KILLING_OFF=", 13);
  buffer_putuint64(&wbuf, killing_off);
  buffer_put(&wbuf, " MAXLOADSOFT=", 13);
  buffer_putuint64(&wbuf, maxloadsoft);
  buffer_put(&wbuf, " MAXLOADHARD=", 13);
  buffer_putuint64(&wbuf, maxloadhard);
  buffer_put(&wbuf, " PROCSOFTLIMIT=", 15);
  buffer_putuint64(&wbuf, procsoftlimit);
  buffer_put(&wbuf, " PROCHARDLIMIT=", 15);
  buffer_putuint64(&wbuf, prochardlimit);
  buffer_put(&wbuf, " HZ=", 4);
  buffer_putuint64(&wbuf, jiffies_per_second);
  buffer_put(&wbuf, " CMDLINE_EXCLUDES=", 18);
  for(val32=0;;) {
    if (cmdline_excludes[val32]) {
      if (val32) buffer_putc(&wbuf, ';');
      buffer_put(&wbuf, cmdline_excludes[val32], strlen(cmdline_excludes[val32]));
    } else break;
    ++val32;
  }
  buffer_put(&wbuf, " USERIGNORE=", 12);
  for(val32=0;;) {
    if (userignores[val32]) {
      if (val32) buffer_putc(&wbuf, '|');
      buffer_put(&wbuf, userignores[val32], strlen(userignores[val32]));
    } else break;
    ++val32;
  }
  buffer_put(&wbuf, " USERQUADTIME=", 14);
  for(val32=0;;) {
    if (userquadtimes[val32]) {
      if (val32) buffer_putc(&wbuf, '|');
      buffer_put(&wbuf, userquadtimes[val32], strlen(userquadtimes[val32]));
    } else break;
    ++val32;
  }
  buffer_put(&wbuf, " RSS_SOFTLIMIT=", 15);
  buffer_putuint64(&wbuf, rss_softlimit);
  buffer_putc(&wbuf, '\n');
  buffer_flush(&wbuf);


  main_loop:

  /* load per 1 minute average */
  load1 = get_load1();

  if (load1 > load_threshold) {
    /* ueber Schwellwert: Laufzeit wird abhaengig von der Load reduziert */
    load_factor = (100 * load1) / maxloadsoft;
  } else {
    if (load1 == 0) {
      load_factor = 10;
    } else {
      load_factor = 100;
    }
  }

  /* setze Zaehler zurueck */
  uidp[0] = uidcnt[0] = 0;
  kill_count = proc_count = 0;
  uptime = get_uptime_in_seconds() * jiffies_per_second;

  /* gehe durch das /proc Verzeichnis und alle /proc/<PID>/stat Dateien */
  procdirp = opendir("/proc");
  if (procdirp == NULL) goto my_exit;
  for(;(procdirep = readdir(procdirp)) != NULL;) {

    /* PID? */
    if ((*procdirep->d_name > '9') || (*procdirep->d_name < '0')) continue;

    ++proc_count;

    /**
     * verwende kein stat()/lstat(), lese statt dessen .../status;
     * funktioniert bei ganz alten und ganz neuen Linux-Systemen,
     * da Strings tatsaechlich die einzig portable Darstellung 
     * von Zahlen sind: unabhaengig von Typen einer Sprache, unabhaengig
     * Bibliotheken und auch vom Kernel :-)
     */
    s = stpcpy(filename, "/proc/");
    s = stpcpy(s, procdirep->d_name);
    stpcpy(s, "/status");

    /* lese UID */
    rf = open(filename, O_RDONLY);
    if (rf == -1) continue;
    p = filebuf;
    c = read(rf, p, BUFSIZE);
    close(rf);
    if (c < 45) continue;
    p[c] = '\0';

    /* suche String "Uid:" mit einem kleinen Offset */
    if ((c = str_index3(p+UID_MIN_OFFSET, uid_marker, c, sizeof(uid_marker)-1)) < 0) continue;
    /* p+c zeigt auf Real UID */
    p = strchr(p+c+UID_MIN_OFFSET, '\t') + 1;
    val32 = 0;
    str2uint32(p, &val32); /* Effective UID */
    uid = val32;
    if (uid < min_uid) continue;
    if (max_uid && (uid > max_uid)) continue;

    /* Prozess gehoert einem User mit passender UID */

    /* suche nach UID in unserer Liste und zaehle deren Zaehler hoch */
    c = 0;
    for(;;) {
      if (c >= MAX_USERS) break;
      if (uidp[c] == 0) {
        /* noch nicht in Liste */
        uidp[c] = uid; uidcnt[c] = 1;
        uidp[++c] = 0; break;
      }
      if (uidp[c] == val32) {
        uidcnt[c] += 1;
        break;
      }
      ++c;
    }

    /* hole Benutzername und prüfe gegen USERIGNORE-Liste */
    user_name = get_username_by_uid(uid, (const char **) user_names, user_ids);
    user_name_len = strlen(user_name);
    for(val32=0;;++val32) {
      if (userignores[val32]) {

        uint16_t match_end;
        uint16_t match_beginning;
        uint16_t len;

        /* pruefe auf Caret und Dollar im userignore */
        if (userignores[val32][0] == '^') {
          match_beginning = 1;
        } else {
          match_beginning = 0;
        }
        len = strlen(userignores[val32] + match_beginning);
        if (userignores[val32][match_beginning+len-1] == '$') {
          match_end = 1;
          --len;
        } else {
          match_end = 0;
        }

        /* vergleiche Strings */
        /* 1) exakt passend */
        if ((match_end+match_beginning) == 2) {
          if (len != user_name_len) {
            continue;
          }
          if (strncmp(user_name, userignores[val32]+1, len)) {
            continue;
          } else {
            break;
          }
        }
        /* fuer entweder '^' oder '$' darf Suchstring nicht zu lang sein */
        if (len > user_name_len) {
          continue;
        }
        /* 2) passend am Anfang */
        if (match_beginning) {
          if (strncmp(user_name, userignores[val32] + match_beginning, len)) {
            continue;
          } else {
            break;
          }
        }
        /* 3) passend am Anfang */
        if (match_end) {
          if (strncmp(user_name+user_name_len-len, userignores[val32], len)) {
            continue;
          } else {
            break;
          }
        }
        /* 4) passend irgendwo */
        if (strstr(user_name, userignores[val32])) {
          break;
        } else {
          continue;
        }

      } else goto get_cmdline;
    }
    continue; /* Benutzer passt zu Eintrag in USERIGNORE */

    get_cmdline:
    /* Befehlszeile des Prozesses */
    stpcpy(s, "/cmdline");
    p = cmdline;
    *p = '?'; p[1] = 0;
    rf = open(filename, O_RDONLY);
    if (rf >= 0) {
      c = read(rf, p, BUFSIZE);
      close(rf);
      if (c > 0) {
        p[c] = '\0';
        /* aendere jedes \0 zu Leerzeichen, jedes \n zu einem Fragezeichen */
        for(;c > 0;) {
          --c;
          if(!p[c]) p[c] = ' ';
          else if (p[c] == '\n') p[c] = '?';
        }
        val32=0;
        /* Teste alle CMDLINE_EXCLUDES */
        for(;;) {
          if (cmdline_excludes[val32]) {
            if (strstr(p, cmdline_excludes[val32])) {
              /* Befehlszeile des Prozesses enthält diesen Teilstring: Kein Killing! */
              break;
            }
          } else goto cmdline_not_excluded; /* Kein Teilstring passt */
          ++val32;
        }
        continue; /* Bei Match => kein Killing */
      }
    }

    cmdline_not_excluded:
    /* lese Prozess stat-Datei */
    s = stpcpy(filename, "/proc/");
    p = stpcpy(s, procdirep->d_name);
    pid_s_len = p - s;
    stpcpy(p, "/stat");

    rf = open(filename, O_RDONLY);
    if (rf >= 0) {
      p = filebuf;
      c = read(rf, p, BUFSIZE);
      close(rf);
      if (c == -1) continue;
      p[c] = '\0';
      str2uint32(p, &val32); /* keine Fehlerpruefung */
      pid = val32;
      /* der 21ste Wert ist "starttime" in Einheit Jiffies */
      s = field(p, 21);
      if (s) {
        /* bei Fehler so verhalten, als waere Prozess gerade gestartet */
        if(!str2uint64(s, &val64)) val64 = uptime;
        s = p;
      } else {
        /* bei Fehler so verhalten, als waere Prozess gerade gestartet */
        val64 = uptime;
      }
      /* verhindere "negatives" Alter */
      if (val64 > uptime) val64 = uptime;
      process_age = (uptime - val64) / jiffies_per_second;

      /* RSS */
      s = field(p, 23);
      if (s) {
        if (!str2uint64(s, &rss)) rss = 0;
      } else {
        rss = 0;
      }
      rss = rss / 1024 * pagesize; /* in KB */

      /* berechne kill_score */
      kill_proc = 0;
      if (load1 >= maxloadhard) {
        kill_score = kill_level;
        goto begin_output;
      }
      if ((max1uid == uid) && (max1cnt >= procsoftlimit)) {
        /* im letzten Lauf die meisten Prozesse */
        if (max1cnt >= prochardlimit) {
          /* hartes Limit: kill */
          kill_score = kill_level;
          goto begin_output;
        } else {
          /* weiches Limit: Faktor 5 */
          kill_score = 5 * process_age;
        }
      } else {
        kill_score = process_age;
      }
      if (rss_softlimit) {
        if (rss >= rss_softlimit) {
          kill_score *= 3;
        }
      }

      /* prüfe gegen USERQUADTIME-Liste */
      for(val32=0;;++val32) {
        if (userquadtimes[val32]) {
  
          uint16_t match_end;
          uint16_t match_beginning;
          uint16_t len;
  
          /* pruefe auf Caret und Dollar im userignore */
          if (userquadtimes[val32][0] == '^') {
            match_beginning = 1;
          } else {
            match_beginning = 0;
          }
          len = strlen(userquadtimes[val32] + match_beginning);
          if (userquadtimes[val32][match_beginning+len-1] == '$') {
            match_end = 1;
            --len;
          } else {
            match_end = 0;
          }
  
          /* vergleiche Strings */
          /* 1) exakt passend */
          if ((match_end+match_beginning) == 2) {
            if (len != user_name_len) {
              continue;
            }
            if (strncmp(user_name, userquadtimes[val32]+1, len)) {
              continue;
            } else {
              break;
            }
          }
          /* fuer entweder '^' oder '$' darf Suchstring nicht zu lang sein */
          if (len > user_name_len) {
            continue;
          }
          /* 2) passend am Anfang */
          if (match_beginning) {
            if (strncmp(user_name, userquadtimes[val32] + match_beginning, len)) {
              continue;
            } else {
              break;
            }
          }
          /* 3) passend am Anfang */
          if (match_end) {
            if (strncmp(user_name+user_name_len-len, userquadtimes[val32], len)) {
              continue;
            } else {
              break;
            }
          }
          /* 4) passend irgendwo */
          if (strstr(user_name, userquadtimes[val32])) {
            break;
          } else {
            continue;
          }
  
        } else goto load_factor_correction;
      }
      /* Benutzer passt zu Eintrag in USERQUADTIME */
      kill_score /= 4;

      load_factor_correction:
      /* if (!kill_score) kill_score = 1; */
      kill_score *= load_factor;
      kill_score /= 100; /* load_factor Korrektur */

      begin_output:

      /* kein Logging der Daten, wenn kill_score zu niedrig */
      if (kill_score < (kill_level * LOGGING_THRESHOLD_PERCENT / 100)) continue;

      /* Beginn der Ausgabezeile */
      if (kill_score >= kill_level) {
        if (killing_off) {
          kill_proc = 0;
          buffer_put(&wbuf, "pseudokilling ", 14);
        } else {
          kill_proc = 1;
          buffer_put(&wbuf, "killing ", 8);
        }
      }

      buffer_put(&wbuf, "killscore=", 10);
      buffer_putuint64(&wbuf, kill_score);
      buffer_put(&wbuf, " load1=", 7);
      buffer_putuint64(&wbuf, load1);
      buffer_put(&wbuf, " AGE=", 5);
      buffer_putuint64(&wbuf, process_age); /* Prozessalter */
      buffer_put(&wbuf, " PID=", 5);
      buffer_put(&wbuf, procdirep->d_name, pid_s_len); /* PID */
      buffer_put(&wbuf, " USER=", 6);

      if (user_name) {
        buffer_put(&wbuf, user_name, strlen(user_name));
      } else {
        buffer_put(&wbuf, "uid", 3);
        buffer_putuint64(&wbuf, uid); /* alternativ UID */
      }

      buffer_put(&wbuf, " RSS=", 5);
      buffer_putuint64(&wbuf, rss);

      s = field(p, 49);
      if (s) {
        if (!str2uint32(s, &val32)) val32 = 0;
      } else {
        val32 = 0;
      }
      buffer_put(&wbuf, " VPID=", 6);
      buffer_putuint64(&wbuf, val32); /* VPID (OpenVZ) */

      s = field(p, 50);
      if (s) {
        if (!str2uint32(s, &val32)) val32 = 0;
      } else {
        val32 = 0;
      }
      buffer_put(&wbuf, " CTID=", 6);
      buffer_putuint64(&wbuf, val32); /* CTID/FnID (OpenVZ) */

      buffer_put(&wbuf, " PROCNAME=", 10);
      s = strchr(p, ')'); if (!s) s = p + 10;
      *s = 0;
      val32 = s - p;
      s = strchr(p, '(');
      if (!s) s = p + 5;
      ++s;
      val32 -= s - p;
      buffer_put(&wbuf, s, val32); /* Prozessname */

      /* wenn dieser Schwellwert erreicht ist, hole andere proc-Infos nicht */
      if (load1 < maxloadhard) {

        /* Offene Sockets? */
        s = stpcpy(filename, "/proc/");
        s = stpcpy(s, procdirep->d_name);
        s = stpcpy(s, "/fd");

        buffer_put(&wbuf, " S=", 3);
        /* gehe durch das FileDescriptor-Verzeichnis und suche nach Socket-Symlinks */
        sdirp = opendir(filename);
        if (!sdirp) goto nosocket;
        *s = '/'; ++s;
        for(;(sdirep = readdir(sdirp)) != NULL;) {
          stpcpy(s, sdirep->d_name);
          c = readlink(filename, p, BUFSIZE);
          if (c < 0) continue;
          /* p[c] = 0; */
          if (is_socket_link(p)) goto socket;
        }
        closedir(sdirp);
      
        nosocket:
        buffer_putc(&wbuf, '?');
        goto sockfollower;

        socket:
        buffer_put(&wbuf, "socket", 6);
        closedir(sdirp);

        sockfollower:

        /* Laeuft der Prozess in einer chroot-Umgebung? */
        s = stpcpy(filename, "/proc/");
        s = stpcpy(s, procdirep->d_name);
        stpcpy(s, "/root");
        c = readlink(filename, p, BUFSIZE);
        buffer_put(&wbuf, " ROOT=", 6);
        if (c < 0) buffer_putc(&wbuf, '?');
        else {
          buffer_put(&wbuf, p, c); /* (ch)root */
        }

        /* Arbeitsverzeichnis des Prozesses */
        stpcpy(s, "/cwd");
        c = readlink(filename, p, BUFSIZE);
        buffer_put(&wbuf, " CWD=", 5);
        if (c < 0) buffer_putc(&wbuf, '?');
        else {
          buffer_put(&wbuf, p, c);
        }

        /* die ausfuehrbare Programmdatei des Prozesses */
        stpcpy(s, "/exe");
        c = readlink(filename, p, BUFSIZE);
        buffer_put(&wbuf, " EXE=", 5);
        if (c < 0) buffer_putc(&wbuf, '?');
        else {
          buffer_put(&wbuf, p, c);
        }

        /* Befehlszeile des Prozesses */
        buffer_put(&wbuf, " CMD=", 5);
        buffer_put(&wbuf, cmdline, strlen(cmdline));

        /* Umgebungsvariablen: HTTP_USER_AGENT */
        stpcpy(s, "/environ");
        rf = open(filename, O_RDONLY);
        if (rf >= 0) {
          c = read(rf, p, BUFSIZE);
          if (c > 0) {
            val32 = c;
            /* suche HTTP_USER_AGENT */
            for(c=0;c<val32;) {
              if (!strncmp(p+c, "HTTP_USER_AGENT=", 16)) {
                p += c;
                buffer_putc(&wbuf, ' ');
                buffer_put(&wbuf, p, strlen(p));
                break;
              } else {
                c += strlen(p+c) + 1;
              }
            }
          }
          close(rf);
        }

      } /* load1 < Schwellwert maxloadhard */

      buffer_putc(&wbuf, '\n');
      buffer_flush(&wbuf);

      /* alle Daten gelesen, kill? */
      if (kill_proc) {
        kill(pid, 9);
        ++kill_count;
      }

    } /* stat-Datei und mehr Prozessinfos gelesen */


  } /* for-Schleife ueber proc-Dateien */


  /* gehe durch unsere Liste und ermittle die drei Benutzer mit den meisten Prozessen */
  max1uid = max1cnt = max2uid = max2cnt = max3uid = max3cnt = 0;
  c = 0;
  for(;;) {
    if (c >= MAX_USERS) break;
    if (!uidp[c]) break;
    if (uidcnt[c] > max1cnt) {
      max3uid = max2uid; max3cnt = max2cnt;
      max2uid = max1uid; max2cnt = max1cnt;
      max1uid = uidp[c]; max1cnt = uidcnt[c];
    } else {
      if (uidcnt[c] > max2cnt) {
        max3uid = max2uid; max3cnt = max2cnt;
        max2uid = uidp[c]; max2cnt = uidcnt[c];
      } else {
        if (uidcnt[c] > max3cnt) { max3uid = uidp[c]; max3cnt = uidcnt[c]; }
      }
    }
    ++c;
  }

  /* zeige statistische Daten */
  if (load1 > load_threshold) {
    buffer_put(&wbuf, "stats: load1=", 13);
    buffer_putuint64(&wbuf, load1);
    buffer_put(&wbuf, " procs=", 7);
    buffer_putuint64(&wbuf, proc_count);
    buffer_put(&wbuf, " kills=", 7);
    buffer_putuint64(&wbuf, kill_count);
    buffer_putc(&wbuf, ' ');
    if (max1cnt) {
      s = (char *) get_username_by_uid(max1uid, (const char **) user_names, user_ids);
      if (s) {
        buffer_put(&wbuf, s, pw_name_len);
      } else {
        buffer_putuint64(&wbuf, max1uid);
      }
      buffer_putc(&wbuf, '=');
      buffer_putuint64(&wbuf, max1cnt);
      if (max2cnt) {
        buffer_putc(&wbuf, ' ');
        s = (char *) get_username_by_uid(max2uid, (const char **) user_names, user_ids);
        if (s) {
          buffer_put(&wbuf, s, pw_name_len);
        } else {
          buffer_putuint64(&wbuf, max2uid);
        }
        buffer_putc(&wbuf, '=');
        buffer_putuint64(&wbuf, max2cnt);
        if (max3cnt) {
          buffer_putc(&wbuf, ' ');
          s = (char *) get_username_by_uid(max3uid, (const char **) user_names, user_ids);
          if (s) {
            buffer_put(&wbuf, s, pw_name_len);
          } else {
            buffer_putuint64(&wbuf, max3uid);
          }
          *p = '='; ++p;
          buffer_putc(&wbuf, '=');
          buffer_putuint64(&wbuf, max3cnt);
        }
      }
    }
    buffer_putc(&wbuf, '\n');
    buffer_flush(&wbuf);
  }

  closedir(procdirp);

  if ((max1cnt < prochardlimit) && (load1 < maxloadsoft)) sleep(4);

  goto main_loop;


  my_exit:
  /*munmap((void *) pwdbuf, pwdlen);*/
  _exit(EX_OSFILE);

}
