/*
 * ipv4conv.c
 * converts IPv4 network byte order hex ip address to human readable dotted notation
 * $Id: ipv4conv.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#if __GNUC__ > 5
#pragma GCC diagnostic ignored "-Wmisleading-indentation"
#endif

#include <unistd.h>
#include <sysexits.h>

#ifndef write1
#include <sys/syscall.h>
#define write1(b,c) (syscall(__NR_write, 1, b, c))
#endif

static const char usage[]="usage: ipv4conv IPADDRESSHEX\nexample: ipv4conv 010010AC\n172.16.0.1\n";
static void printusage() { write1(usage, sizeof usage - 1); _exit(EX_USAGE); }

static char *putint(char *s, int num) {
  register int i,d;
  register int c,o;

  i = num; o = 0;
  for (d = 10000; d > 1; d /= 10) {
    c = i / d;
    if ((c > 0) || (o)) {
      *s = c + '0';
      ++s;
      i = i - (c * d);
      o = 1;
    }
  }
  *s = i + '0'; ++s;
  return s;
}

int main(int argc,char *const argv[]) {
  register char *p;
  int byte;
  char buffer[256];
  char *s;
  char i;

  if (argc != 2) printusage();

  p = argv[1];
  for (;;) {
    if (!*p) break; ++p;
  }
  if ((p - argv[1]) != 8) printusage();
  --p;

  s = buffer;

  byte = *p - '0'; --p;
  if (byte > 9) byte -= 7;
  i = *p - '0'; --p;
  if (i > 9) i -= 7;
  byte += i * 16;
  s = putint(s, byte);
  *s = '.'; ++s;

  byte = *p - '0'; --p;
  if (byte > 9) byte -= 7;
  i = *p - '0'; --p;
  if (i > 9) i -= 7;
  byte += i * 16;
  s = putint(s, byte);
  *s = '.'; ++s;

  byte = *p - '0'; --p;
  if (byte > 9) byte -= 7;
  i = *p - '0'; --p;
  if (i > 9) i -= 7;
  byte += i * 16;
  s = putint(s, byte);
  *s = '.'; ++s;

  byte = *p - '0'; --p;
  if (byte > 9) byte -= 7;
  i = *p - '0'; --p;
  if (i > 9) i -= 7;
  byte += i * 16;
  s = putint(s, byte);
  *s = '\n'; ++s;

  write1(buffer, s - buffer);

  return 0;
}
