/*
 * killwebprocs.c
 * kill php and imagemagick procs
 * $Id: killwebprocs.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#if __GNUC__ > 5
#pragma GCC diagnostic ignored "-Wmisleading-indentation"
#endif

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#ifndef _FILE_OFFSET_BITS
#define _FILE_OFFSET_BITS 64
#endif
#include <features.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <pwd.h>
#include <stdint.h>
#include <string.h>
#include <signal.h>
#include <sysexits.h>

#define BUFSIZE 16384

#ifndef write2
#include <sys/syscall.h>
#define write2(b,c) (syscall(__NR_write, 2, b, c))
#endif

static const char usage[]="usage: killwebprocs [min-uid [max-uid]]\nkill php and imagemagick procs\n";
static const char uid_marker[]="Uid:\011";

static void printusage() { write2(usage, sizeof usage - 1); _exit(1); }

static inline int str_index3(const char* a, const char* b, int al, register int t) {
  register const char* s;
  register char z;
  register const char* u;

  s = a;
  u = s + al - t;
  z = *b;
  for (;;) {
    if (s > u) break; if ((*s == z) && (!strncmp(s, b, t))) return (s - a) + t; ++s;
    if (s > u) break; if ((*s == z) && (!strncmp(s, b, t))) return (s - a) + t; ++s;
    if (s > u) break; if ((*s == z) && (!strncmp(s, b, t))) return (s - a) + t; ++s;
    if (s > u) break; if ((*s == z) && (!strncmp(s, b, t))) return (s - a) + t; ++s;
  }
  return -1;
}

static int64_t str2uint63(register const char *s) {
  register int64_t i;
  register char c;

  c = *s;
  if ((c < '0') || (c > '9')) return -1;
  ++s;
  i = c - '0';
  for(;;) {
    c = *s;
    if ((c < '0') || (c > '9')) {
      return i;
    } else {
      i = (i * 10) + c - '0';
      ++s;
    }
  }
}

int main(int argc,char *const argv[]) {
  register char *p;
  register char *s;
  int64_t lval;
  ssize_t c;
  uint32_t pid;
  uint32_t uid_min;
  uint32_t uid_max;
  int rf;
  char name[2*BUFSIZE+1];
  char filebuf[BUFSIZE+1];
  DIR *procdirp;
  struct dirent *procdirep;
  uint32_t retval;


  uid_min = uid_max = 0;

  /* cmdline parsing */
  c = 1;
  if (argc > c) {
    lval = str2uint63(argv[c]);
    if (lval < 0) printusage();
    uid_min = (uint32_t) lval;
    ++c;
    if (argc > c) {
      lval = str2uint63(argv[c]);
      if (lval < 0) printusage();
      uid_max = (uint32_t) lval;
    } /* ignore other args */
  }


  procdirp = opendir("/proc");
  if (!procdirp) _exit(EX_OSFILE);


  retval = 0;
  for(;(procdirep = readdir(procdirp));) {

    /* is it a PID? */
    if ((*procdirep->d_name < '0') || (*procdirep->d_name > '9')) continue;

    /*
     * don't use stat/lstat(), read .../status, because this is
     * actually the only portable presentation of numbers: as string
     */
    s = stpcpy(name, "/proc/");
    s = stpcpy(s, procdirep->d_name);
    stpcpy(s, "/status");

    rf = open(name, O_RDONLY);
    if (rf <= 0) continue;
    p = filebuf;
    lval = (int64_t) read(rf, p, BUFSIZE);
    close(rf);
    if (lval < 2) continue;
    p[lval] = 0;

    if ((c = str_index3(p, uid_marker, lval, sizeof(uid_marker)-1)) < 0) continue;
    /* p+c points to real uid */
    p = strchr(p+c, '\t') + 1; /* now points at effective user id */
    lval = str2uint63(p);
    if (lval < uid_min) goto next_file;
    if (uid_max && (lval > uid_max)) goto next_file;

    s = stpcpy(name, "/proc/");
    s = stpcpy(s, procdirep->d_name);
    stpcpy(s, "/stat");

    /* read process stat */
    close(rf);
    rf = open(name, O_RDONLY);
    if (rf > 0) {
      p = filebuf;
      if (read(rf, p, BUFSIZE) < 2) goto next_file;
      p[BUFSIZE] = 0;
      pid = str2uint63(p); /* no error check */

      s = strchr(p, ')'); if (!s) s = p + 10;
      *s = 0;
      s = strchr(p, '(');
      if (!s) s = p + 5;
      ++s;

      if (!strncmp(s, "php", 4)) goto kill_proc;
      if (!strncmp(s, "convert", 8)) goto kill_proc;
      if (!strncmp(s, "identify", 9)) goto kill_proc;
      goto next_file;

      kill_proc:
      retval -= kill(pid, 9);

      next_file:
      close(rf);
    }

  }

  closedir(procdirp);
  _exit(retval);

}
