/*
 * lkill.c
 * kills all procs of a user
 *
 * $Id: lkill.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#include <stdint.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <dirent.h>
#include <signal.h>
#include <paths.h>
#include <sysexits.h>
#include "lib/libfwb.h"

#define STRINGSIZE 1024

static const char usage[]="usage: lkill username  # kills all procs of user username\n";
static const char uid_marker[]="Uid:\011";

static inline void printusage() { write1(usage, sizeof usage - 1); _exit(EX_USAGE); }

static inline int64_t str2uint63(register const char *s) {
  int64_t i;
  char c;

  c = *s;
  if ((c < '0') || (c > '9')) return -1;
  ++s;
  i = c - '0';
  for(;;) {
    c = *s;
    if ((c < '0') || (c > '9')) {
      return i;
    } else {
      i = (i * 10) + c - '0';
      ++s;
    }
  }
}

static inline int str_index3(const char* a, const char* b, int al, register int t) {
  register const char* s;
  char z;
  const char* u;

  s = a;
  u = s + al - t;
  z = *b;
  for (;;) {
    if (s > u) break; if ((*s == z) && (!strncmp(s, b, t))) return (s - a) + t; ++s;
    if (s > u) break; if ((*s == z) && (!strncmp(s, b, t))) return (s - a) + t; ++s;
    if (s > u) break; if ((*s == z) && (!strncmp(s, b, t))) return (s - a) + t; ++s;
    if (s > u) break; if ((*s == z) && (!strncmp(s, b, t))) return (s - a) + t; ++s;
  }
  return -1;
}

int main(int argc,char *const argv[]) {
  register char *p;
  char *t;
  int64_t lval;
  DIR *procdirp;
  struct dirent *procdirep;
  int rf;
  ssize_t c;
  const char *pwdbuf;
  off_t pwdlen;
  char name[STRINGSIZE+14];
  char filebuf[STRINGSIZE+1];
  uint32_t retval;
  char uid_s[STRINGSIZE+1];
  int s_len;

  if ((argc < 2) || (*argv[1] == '-')) printusage();
  s_len = strlen(argv[1]);

  /* open /etc/passwd for uid-mapping */
  p = (char *) map_passwd_file(&pwdbuf, &pwdlen);
  if (p == (char *) -1) _exit(EX_OSFILE);
  t = p + pwdlen - s_len;

  /* get UID of user */
  p = (char *) get_uid_from_passwd(pwdbuf, pwdlen, argv[1], s_len, &s_len);
  if (!p) { munmap((void *) pwdbuf, pwdlen); _exit(EX_NOUSER); }
  if (s_len > STRINGSIZE) { /* slightly paranoid */
    munmap((void *) pwdbuf, pwdlen);
    _exit(EX_DATAERR);
  }
  for (c = 0; c < s_len; ++c) uid_s[c] = p[c];
  p = uid_s + s_len;

  *p = 9; /* separator TAB in /proc/PID/status */
  *++p = 0;
  ++s_len;
  munmap((void *) pwdbuf, pwdlen);
  
  /* check all processes */
  procdirp = opendir("/proc");
  if (!procdirp) _exit(EX_OSFILE);

  retval = 0;
  for(;(procdirep = readdir(procdirp)) != 0;) {

    /* is it a PID? */
    if ((*procdirep->d_name < '0') || (*procdirep->d_name > '9')) continue;

    /*
     * don't use stat/lstat(), read .../status, because this is
     * actually the only portable presentation of numbers: as string
     */
    t = stpcpy(name, "/proc/");
    t = stpcpy(t, procdirep->d_name);
    stpcpy(t, "/status");

    rf = open(name, O_RDONLY);
    if (rf <= 0) continue;
    p = filebuf;
    lval = (int64_t) read(rf, p, STRINGSIZE);
    close(rf);
    if (lval < 2) continue;
    p[lval] = 0;

    if ((c = str_index3(p, uid_marker, lval, sizeof uid_marker - 1)) < 0) continue;
    /* p+c points at real uid */
    p = strchr(p + c, '\t') + 1; /* now points at effective user id */
    if (!strncmp(p, uid_s, s_len)) {
      lval = str2uint63(procdirep->d_name);
      /* last check then kill */
      if (lval > 0) retval -= kill(lval, SIGKILL);
    }
  }

  closedir(procdirp);
  _exit(retval);
}
