/**
 * lns
 * lightweight network stats
 *
 * $Id: lns.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#include "lib/libfwb.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdint.h>
#include <string.h>
#include <sysexits.h>
#include <sys/syscall.h>
#include <unistd.h>

#define MAX_ADDRESSES_GUESSED 1000

#define BUFSIZE 16384
#define WRITEBUFSIZE 8192

/*
static const char usage[]="usage: lns [-c] [-u] [-L socklimit]\n";
*/
static const char usage[]="usage: lns [-t] [-u] [-w]\n";
static inline void printusage() { write1(usage, sizeof usage - 1); _exit(EX_USAGE); }


int main(int argc,char *const argv[]) {
  register char *p;
  char *r;
  uint32_t sc;
  int rf;
  ssize_t br;
  char filebuf[2*BUFSIZE+1];
  int remote;
  int count_timewait;
  char procfilename[15];
  char buf[67];
  char remote_addr[67];
  int c;
  uint32_t mode_count;
  int mode_top;
  int mode_limit;
  int mode_show;
  int stage_ipv6;
  char wbufspace[WRITEBUFSIZE+1];
  uint32_t val32;
  uint32_t localport;
  char a_addresses[MAX_ADDRESSES_GUESSED+1][67];
  uint16_t a_addresscount[MAX_ADDRESSES_GUESSED+1];
  char *max1_address;
  uint16_t max1_count;
  char *max2_address;
  uint16_t max2_count;
  char *max3_address;
  uint16_t max3_count;
  
/*
 * example line from /proc/net/tcp:
 *    9: 3200101C:56B8 3C82141C:0019 06 00000000:00000000 03:0000155C 00000000     0        0 0 2 d6904880
 */
  sc = remote = 0;
  count_timewait = 0;
  a_addresses[0][0] = 0;
  mode_top = mode_limit = mode_count = mode_show = 0;

  for (c = 1; argc > c; ++c) {
    if (*argv[c] == '-') {
      if (argv[c][2] != '\0') printusage();
      switch (argv[c][1]) {
        case 'c': mode_count = 1; break;
        case 't': mode_top = 1; break;
        case 'w': count_timewait = 1; break;
        case 'L': mode_limit = mode_top = 1;
                  ++c;
                  if (!str2uint32(argv[c], &val32)) printusage();
                  break;
        default: printusage();
      }
    } else {
      if (!str2uint32(argv[c], &val32)) _exit(EX_USAGE);
      ++c;
      if (argc > c) {
        if (!str2uint32(argv[c], &val32)) _exit(EX_USAGE);
      }
      break;
    }
  }
  if ((!mode_top) && (!mode_limit) && (!mode_count)) mode_show = 1;

  /* Ausgabepuffer initialisieren */
  p = wbufspace;
  buffer wbuf = { p, p, p+WRITEBUFSIZE, 1 };

  /* loop with two stages: tcp and tcp6 */
  for (stage_ipv6 = 0; stage_ipv6 < 2; ++stage_ipv6) {

    strcpy(procfilename, "/proc/net/");
    strcpy(procfilename+10, "tcp");
    if (stage_ipv6) { procfilename[13] = '6'; procfilename[14] = 0; }
  
    rf = open(procfilename, O_RDONLY);
    /*if (rf < 0) goto err_exit;*/
    if (rf < 0) continue;
  
    read_again:
    p = filebuf;
    br = read(rf, p, 2*BUFSIZE);
    if (br < 1) goto end_file;
    p[br] = 0;
    p = strchr(p, '\n'); if(!p) goto end_file;
  
    next_conn:
    p = strchr(p, ':'); if(!p) goto read_again;
    p += 2; /* local addr */
  
    /* save ptr to local address for delayed output */
    r = p;
  
    p = strchr(p, ':'); if(!p) goto read_again;
    ++p;
    /* local port */
    val32 = 0;
    p = hexstr2uint32(p, &val32);
    /* save for delayed output */
    localport = val32;
    ++p;
  
    /* remote addr */
    procnetipaddr_to_humanreadable((const char *) p, remote_addr);
    if (mode_show) buffer_putstr(&wbuf, remote_addr);
  
    p = strchr(p, ':'); if(!p) goto read_again;
    ++p;
    /* remote port */
    val32 = 0;
    p = hexstr2uint32(p, &val32);
    if (mode_show) {
      buffer_putc(&wbuf, ' ');
      buffer_putuint32(&wbuf, val32);
    }

    procnetipaddr_to_humanreadable((const char *) r, buf);
    if (mode_show) {
      buffer_putc(&wbuf, ' ');
      buffer_putstr(&wbuf, buf);

      buffer_putc(&wbuf, ' ');
      buffer_putuint32(&wbuf, localport);
    }
  
    p = strchr(p, ' '); if(!p) goto read_again;
    ++p;
    /* state */
    val32 = 0;
    hexstr2uint32(p, &val32);

    /* TCP_TIME_WAIT won't be counted if count_timewait is not set */
    if (count_timewait || (val32 != 6)) {

      ++sc;

      if (mode_top) {
        /* search for address in our list and increment counter for this address */
        c = 0;
        for(;;) {
          if (c >= MAX_ADDRESSES_GUESSED) break;
          if (a_addresses[c][0] == 0) {
            strncpy(a_addresses[c], remote_addr, 67);
            a_addresscount[c] = 1;
            a_addresses[++c][0] = 0; break;
          }
          if (!strncmp(a_addresses[c], remote_addr, 67)) {
            a_addresscount[c] += 1;
            break;
          }
          c++;
        }
      }

    }

    if (mode_show) {
      buffer_putc(&wbuf, ' ');
      buffer_putuint32(&wbuf, val32);
      buffer_putc(&wbuf, '\n');
    }
  
    p = strchr(p, '\n'); if(!p) goto read_again; /* maybe one matched line will be missed */
    goto next_conn;
  
    end_file:
    close(rf);
  }

  if (mode_count) {
    buffer_putuint32(&wbuf, sc);
    buffer_putc(&wbuf, '\n');
  }

  if (mode_top) {
    /* walk through our list and get the 3 top users with most processes running */
    max1_address = max2_address = max3_address = 0;
    max1_count = max2_count = max3_count = 0;
    c = 0;
    for(;;) {
      if (c >= MAX_ADDRESSES_GUESSED) break;
      if (a_addresses[c][0] == 0) { break; }
      if (a_addresscount[c] > max1_count) {
        max3_address = max2_address; max3_count = max2_count;
        max2_address = max1_address; max2_count = max1_count;
        max1_address = a_addresses[c]; max1_count = a_addresscount[c];
      } else {
        if (a_addresscount[c] > max2_count) {
          max3_address = max2_address; max3_count = max2_count;
          max2_address = a_addresses[c]; max2_count = a_addresscount[c];
        } else {
          if (a_addresscount[c] > max3_count) {
            max3_address = a_addresses[c]; max3_count = a_addresscount[c];
          }
        }
      }
      c++;
    }
    if (max1_count == 0) goto flush_and_exit;
    buffer_putstr(&wbuf, max1_address); buffer_putc(&wbuf, '=');
    buffer_putuint32(&wbuf, max1_count); buffer_putc(&wbuf, '\n');
    if (max2_count == 0) goto flush_and_exit;
    buffer_putstr(&wbuf, max2_address); buffer_putc(&wbuf, '=');
    buffer_putuint32(&wbuf, max2_count); buffer_putc(&wbuf, '\n');
    if (max3_count == 0) goto flush_and_exit;
    buffer_putstr(&wbuf, max3_address); buffer_putc(&wbuf, '=');
    buffer_putuint32(&wbuf, max3_count); buffer_putc(&wbuf, '\n');
  }

  flush_and_exit:
  buffer_flush(&wbuf);
  _exit(0);

/*
  err_exit:
  _exit(1);
*/
}
