/*
 * nagios-show-addresses.c
 * read nagios objects.cache and print hostnames and addresses
 * derived from nagios-check-time
 * usage: nagios-show-addresses <objects.cache
 *
 * excerpt of file:
 * define host {
 *         host_name       sysimg
 *         alias   sysimg
 *         address 172.16.4.15
 *
 * output format:
 * <HOSTNAME> <SPACE> <IP_ADDRESS> <NEWLINE>
 *
 * compile with:
 * diet gcc -O2 -Wall ...
 * Frank Bergmann <something@tuxad.com> 8/2009, 7/2011
 */

const char VERSION[]="nagios-show-addresses $Id: nagios-show-addresses.c 691 2017-06-04 23:12:04Z fwbergm $";

#define HOST "define host {"
#define HOST_NAME "\thost_name\t"
#define ADDRESS "\taddress\t"

#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#define DEBUG
#undef DEBUG
#ifdef DEBUG
#include <stdio.h>
#endif

#ifndef EOF
#define EOF (-1)
#endif

#ifndef write1
#include <sys/syscall.h>
#define write1(b,c) (syscall(__NR_write, 1, b, c))
#endif

#define READBUFSIZE 4096
#define WRITEBUFSIZE 4096
typedef struct buffer {
  char *s; /* first char in bufspace */
  char *c; /* cursor */
  char *t; /* first char after bufspace */
  uint32_t l; /* for read buffers */
} buffer;
buffer buf0;
buffer buf1;
char buf0space[READBUFSIZE+1];
char buf1space[WRITEBUFSIZE+1];

#define STRINGSIZE 1024

void numtrim(register char *n) {
  register char *p;
  register int c = 0;
  char *s;

  p = n;
  s = p;
  for(;*p;++p) {
    if (*p == '.') continue;
    if (!c) {
      if (*p == '0') continue;
      c = 1;
    }
     *n++ = *p;
  }
  if (n == s) {
    *n = '0';
    ++n;
  }
  *n=0;
}

int bgetchar() {
  register char *p;
  register ssize_t l;

  p = buf0.c;
  if (!p) return EOF;
  if (p < buf0.t) goto get;

  p = buf0.s;
  l = read(0, p, buf0.l);
  if (l<=0) {
    buf0.c = 0; /* EOF */
    if (l<0) return -2;
    return EOF;
  }
  buf0.t = p + l;

  get:
  buf0.c = p + 1;
  return (int)(unsigned char)(*p);
}

int bgetline(char *r, int size) {
  register char *p;
  register ssize_t l;

  p = buf0.c;
  if (!p) return EOF;
  if (p < buf0.t) goto get;

  p = buf0.s;
  l = read(0, p, buf0.l);
  if (l <= 0) {
    buf0.c = 0; /* EOF */
    if (l < 0) return -2;
    return EOF;
  }
  buf0.t = p + l;
  buf0.c = p;

  get:
  p = strchr(p, '\n');
  if (p) goto foundln;
  l = buf0.t - buf0.c;
  p = buf0.c;

  if (size < l) l = size;
  memcpy(r, p, l);
  buf0.c += l;
  r += l;
  *r = 0;
  if (size == l) return l;
  size -= l;
  int l2 = bgetline(r, size);
  if (l2 != EOF) l += l2;
  return l;

  foundln:
  ++p;
  l = p - buf0.c;
  if (size < l) l = size;
  memcpy(r, buf0.c, l);
  buf0.c += l;
  r += l;
  *r = 0;
  return l;
}

int bflush() {
  if (buf1.c == buf1.s) return 0;
  return write1(buf1.s, buf1.c - buf1.s);
}

int bputc(register const char a) {
  register char *p;
  register char *s;

  p = buf1.c;
  if (p < buf1.t) goto put;

  s = buf1.s;
  if (write1(s, p - s) < 0) return -1;
  p = s;

  put:
  *p++ = a;
  buf1.c = p;
  return 0;
}

int bputstr(register const char *a) {
  register char *p;
  register char *s;
  register const char *t;
  register uint32_t l;

  l = strlen(a);
  if (!l) return 0;
  p = buf1.c;
  t = buf1.t;
  if ((p+l) <= t) goto put;

  s = buf1.s;
  if (write1(s, p-s) < 0) return -1;
  p = s;
  if ((p+l) > t) {
    buf1.c = p;
    return write1(a, l);
  }

  put:
  t = a + l;
  for(;;) {
    *p++ = *a++; if (a >= t) break;
    *p++ = *a++; if (a >= t) break;
    *p++ = *a++; if (a >= t) break;
    *p++ = *a++; if (a >= t) break;
  }
  buf1.c = p;
  return 0;
}

void bputlong(uint32_t num) {
  register uint32_t i,d;
  register int c,o;
  register char *pbuf;
  char buf[15];

  i=num; o=0;
  pbuf=buf;
  for(d=1000000000;d>1;d/=10) {
    c=i/d;
    if((c>0)||(o)) {
      *pbuf++=c+'0';
      i=i-(c*d);
      o=1;
    }
  }
  *pbuf++=i+'0';
  *pbuf++=0;
  bputstr(buf);
}

int main() {
  register char *p;
  register int i;
  char s[STRINGSIZE+1];
  char host_name[STRINGSIZE+1];
  char address[STRINGSIZE+1];
  int host = 0;
  int l;

  buf1.s = buf1space;
  buf1.t = buf1space + WRITEBUFSIZE;
  buf1.c = buf1space;

  buf0.s = buf0space;
  buf0.l = READBUFSIZE;
  buf0.t = buf0space + buf0.l;
  buf0.c = buf0.t;

  i = 0;

  l = bgetline(s, STRINGSIZE);
  while (l != EOF) {

    if (host) {

      /* for nagios3/objects_cache */
      if (strchr(s, '}')) {
        host = 0;
        /*numtrim(check_execution_time);*/
        bputstr(host_name); bputc(' ');
        bputstr(address); bputc('\n');
        goto next_line;
      }
      p = 0;
      if (!strncmp(s, HOST_NAME, sizeof HOST_NAME - 1)) {
        i = sizeof HOST_NAME - 1;
        p = host_name;
      }
      if (!strncmp(s, ADDRESS, sizeof ADDRESS - 1)) {
        i = sizeof ADDRESS - 1;
        p = address;
      }
      if (p) {
        strncpy(p, s + i, STRINGSIZE);
        p[strlen(p)-1] = 0;
      }

    } else {

      if (!strncmp(s, HOST, sizeof HOST - 1)) {
        host = 1;
      }

    }

    next_line:
    l = bgetline(s, STRINGSIZE);

  }

  bflush();
  return 0;
}
