/**
 * nts2hrf.c
 * nagios timestamp to human readable format
 * usage: tail /var/log/nagios3/nagios.log |tools/nts2hrf
 * $Id: nts2hrf.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#include "../lib/buffer.h"
#include "../lib/libfwb.h"
#include <time.h>

#define READBUFSIZE 4096
char buf0space[READBUFSIZE+1];
#define WRITEBUFSIZE 4096
char buf1space[WRITEBUFSIZE+1];

int main() {
  register char *p;
  register int c;
  char num[13];
  int t;
  uint32_t ts32;
  time_t ts;
  struct tm lts;

  p = buf0space; buffer buf0 = { p, p + READBUFSIZE, p + READBUFSIZE, 0, READBUFSIZE };
  p = buf1space; buffer buf1 = { p, p, p + WRITEBUFSIZE, 1 };

  c = buffer_getchar(&buf0);
  for(;;) {

    /* start of line */
    if (c == '[') {

      /* assuming it's the beginning of the timestamp */
      for(t = 0; t < 11; ++t) {
        c = buffer_getchar(&buf0);
        if (c == EOF) goto got_eof;
        if (c == ']') break;
        num[t] = c;
      }
      /* ts string read */
      num[t] = 0;

      if(str2uint32(num, &ts32)) {
        ts = ts32;
        /* buffer_putstr(&buf1, ctime((const time_t *)&ts)); */
        if (!localtime_r((const time_t *) &ts, &lts)) goto ts_error;
        buffer_putuint32(&buf1, 1900 + lts.tm_year);
        buffer_putc(&buf1, '-');
        if (lts.tm_mon < 10) buffer_putc(&buf1, '0');
        buffer_putuint32(&buf1, lts.tm_mon+1);
        buffer_putc(&buf1, '-');
        if (lts.tm_mday < 10) buffer_putc(&buf1, '0');
        buffer_putuint32(&buf1, lts.tm_mday);
        buffer_putc(&buf1, ' ');
        if (lts.tm_hour < 10) buffer_putc(&buf1, '0');
        buffer_putuint32(&buf1, lts.tm_hour);
        buffer_putc(&buf1, ':');
        if (lts.tm_min < 10) buffer_putc(&buf1, '0');
        buffer_putuint32(&buf1, lts.tm_min);
        buffer_putc(&buf1, ':');
        if (lts.tm_sec < 10) buffer_putc(&buf1, '0');
        buffer_putuint32(&buf1, lts.tm_sec);
      } else {
        ts_error:
        buffer_putstr(&buf1, num);
      }

      c = buffer_getchar(&buf0);
    }

    /* print chars til end of line */
    for(;;) {
      if (c == EOF) goto got_eof;
      buffer_putc(&buf1, c);
      c = buffer_getchar(&buf0);
      if (c == '\n') break;
    }
    buffer_putc(&buf1, c);
    c = buffer_getchar(&buf0);
  }

  got_eof:
  buffer_flush(&buf1);
  return 0;
}
