/*
 * postfix-saslusernames.c
 * $Id: postfix-saslusernames.c 691 2017-06-04 23:12:04Z fwbergm $
 * http://www.tuxad.com
 */

#include "../lib/libfwb.h"
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <sysexits.h>

#define SASLUSERNAMESTRING "sasl_username="
#define AUTHENTICATEDSENDERSTRING "(Authenticated sender: "
#define RECEIVEDSTRING "Received: "

#define PATHNAMELEN 80
#define BUFSIZE 16384
#define WRITEBUFSIZE 4096
char wbufspace[WRITEBUFSIZE+1];
buffer wbuf;

#define MAX_MAILBOXES_GUESSED 1000
#define MAX_MAILBOXES_SORTED 20
  char a_mailboxes[MAX_MAILBOXES_GUESSED+1][33];
  uint16_t a_mailboxcount[MAX_MAILBOXES_GUESSED+1];
  char *max_mailbox[MAX_MAILBOXES_SORTED];
  uint16_t max_count[MAX_MAILBOXES_SORTED];
int mode_top;

static const char *spooldir = "/var/spool/postfix/";
static const char *subdirs[]  =  {
  "./",
  "active/",
  "deferred/",
/*  "maildrop/", */
/*  "incoming/", */
  0
};
static const char *subsubdirs[]  =  {
  "0/", "1/", "2/", "3/", "4/", "5/", "6/", "7/",
  "8/", "9/", "A/", "B/", "C/", "D/", "E/", "F/",
  0
};

static const char usage[]="usage: postfix-saslusernames [-t] [postfix-spool-dir]\n";
static inline void printusage() { write1(usage, sizeof usage - 1); _exit(EX_USAGE); }

static void check_and_print(const char *dir, const char *filename) {
  register char *p;
  char *r;
  ssize_t bytes;
  int fd;
  char pathname[PATHNAMELEN+1];
  char buf[BUFSIZE+1];
  int c;

  p = stpcpy(pathname, dir);
  stpcpy(p, filename);

#ifdef __APPLE__
  fd = open(pathname, O_RDWR); /* force EISDIR on opening dir */
#else
  fd = open(pathname, O_RDWR | O_LARGEFILE); /* force EISDIR on opening dir */
#endif
  
  if (fd < 0) return;
  p = buf;
  bytes = read(fd, p, BUFSIZE);
  close(fd);
  if (bytes < 1) return;
  r = p + bytes;

  for (;;) {
    if (p == r) break; if (*p < ' ') *p = '\n'; ++p;
    if (p == r) break; if (*p < ' ') *p = '\n'; ++p;
    if (p == r) break; if (*p < ' ') *p = '\n'; ++p;
    if (p == r) break; if (*p < ' ') *p = '\n'; ++p;
  }
  *r = 0;

  p = strstr(buf, SASLUSERNAMESTRING);
  if (!p) goto tryauth;
  /* postfix uses Pascal-style strings with prefix length byte in spool format */
  --p;
  bytes = (ssize_t) *((unsigned char *) p);
  ++p;
  p[bytes] = 0;
  p += sizeof SASLUSERNAMESTRING - 1;
  goto printmailbox;

  tryauth:
  p = strstr(buf, AUTHENTICATEDSENDERSTRING);
  if (!p) return;
  /* header is in "normal" format */
  p += sizeof AUTHENTICATEDSENDERSTRING - 1;

  printmailbox:
  r = p;
  /* search end of mailbox name */
  for (;;) {
    if ((*p == ')') || (!*p) || (*p < ' ') || (*p == '\n')) break; ++p;
    if ((*p == ')') || (!*p) || (*p < ' ') || (*p == '\n')) break; ++p;
    if ((*p == ')') || (!*p) || (*p < ' ') || (*p == '\n')) break; ++p;
    if ((*p == ')') || (!*p) || (*p < ' ') || (*p == '\n')) break; ++p;
  }

  if (!mode_top) {
    *p = ' '; ++p;
    p = stpcpy(p, filename);
    *p = '\n'; ++p;
    buffer_put(&wbuf, r, p - r);
    return;
  }

  /* search for mailbox in our list and increment counter for this mailbox */
  *p = 0;
  c = 0;
  for(;;) {
    if (c >= MAX_MAILBOXES_GUESSED) break;
    if (a_mailboxcount[c] == 0) {
      strncpy(a_mailboxes[c], r, 33);
      a_mailboxcount[c] = 1;
      a_mailboxcount[++c] = 0; break;
    }
    if (!strncmp(a_mailboxes[c], r, 33)) {
      a_mailboxcount[c] += 1;
      break;
    }
    c++;
  }

  return;
}

int main(int argc,char *const argv[]) {
  register char *p;
  char *r;
  DIR *dptr;
  const char **ssd;
  const char **sd;
  struct dirent *de;
  int ret;
  char d[PATHNAMELEN+1];
  int c, m;

  p = wbufspace;
  wbuf.s = p;
  wbuf.c = p;
  wbuf.t = p + WRITEBUFSIZE;
  wbuf.fd = 1;

  mode_top = ret = 0;

  p = d;
  *p = 0;

  for (c = 1; argc > c; ++c) {
    if (*argv[c] == '-') {
      if (argv[c][2] != 0) printusage();
      switch (argv[c][1]) {
        case 't': mode_top = 1; break;
        default: printusage();
      }
    } else {
      if (strlen(argv[c]) < (PATHNAMELEN / 2)) {
        p = stpcpy(p, argv[1]);
        --p;
        if (*p != '/') { ++p; *p = '/'; }
        ++p;
      }
      ++c;
      if (argc > c) printusage();
      break;
    }
  }

  a_mailboxcount[0] = 0;

  if (*d == 0) p = stpcpy(d, spooldir);
  r = p;

  for (sd = subdirs; *sd; ++sd) {

    p = stpcpy(r, *sd);
    dptr = opendir(d);
    if (dptr) {
      for (; (de = readdir(dptr)) != 0;) check_and_print(d, de->d_name);
      closedir(dptr);
    } else {
      if (errno == EACCES) ret = 1;
    }

    for (ssd = subsubdirs; *ssd; ++ssd) {

      stpcpy(p, *ssd);
      dptr = opendir(d);
      if (dptr) {
        for (; (de = readdir(dptr)) != 0;) check_and_print(d, de->d_name);
        closedir(dptr);
      } else {
        if (errno == EACCES) ret = 1;
      }

    }

  }

  if (mode_top) {
    /* walk through our list and get the top mailboxes */
    for (c = 0; c < MAX_MAILBOXES_SORTED; ++c) max_count[c] = 0;
    c = 0;
    for(;;) {
      if (c >= MAX_MAILBOXES_GUESSED) break;
      if (a_mailboxes[c][0] == 0) { break; }
      for (m = 0; m < MAX_MAILBOXES_SORTED; ++m) {
        if (a_mailboxcount[c] > max_count[m]) {
          int n;
          n = MAX_MAILBOXES_SORTED - 1;
          for (;;) {
            max_count[n] = max_count[n-1];
            max_mailbox[n] = max_mailbox[n-1];
            --n; if (n <= m) break;
          }
          max_count[m] = a_mailboxcount[c];
          max_mailbox[m] = a_mailboxes[c];
          break;
        }
      }
      ++c;
    }
    m = MAX_MAILBOXES_SORTED - 1;
    for (;;) {
      if (max_count[m]) {
        buffer_putstr(&wbuf, max_mailbox[m]); buffer_putc(&wbuf, '=');
        buffer_putuint32(&wbuf, max_count[m]); buffer_putc(&wbuf, '\n');
      }
      if (!m) break;
      --m;
    }
  }

  buffer_flush(&wbuf);
  _exit(ret);
}
