#define _DARWIN_C_SOURCE
/*
 * procage.c
 * show age of a process (not running time)
 *
 * $Id: procage.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#include "../lib/libfwb.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <pwd.h>
#include <stdint.h>
#include <sysexits.h>

#define BUFSIZE 2048

static const char usage[]="usage: procage [-s] [-m] [-H] [-D] PID\n";

static inline void printusage() {
  write1(usage, sizeof(usage)-1);
  _exit(EX_USAGE);
}

static char *field(const char *s, uint16_t index) {
  register const char* t=s;

  if (!index) return (char *) s;
  for(;;) {
    if (!*t) return 0; if (*t == ' ') { --index; if (!index) return (char *) ++t; } ++t;
  }
}

int main(int argc,char *const argv[]) {
  register char *p;
  register char *s;
  uint64_t val64;
  ssize_t c;
  int rf;
  char buf[BUFSIZE+1];
  char name[BUFSIZE+1];
  uint64_t start_time;
  uint64_t process_age;
  int jiffies_per_second;
  int mode_time;
  const char *pid_string;

  enum { SECONDS, MINUTES, HOURS, DAYS };

  mode_time = SECONDS;
  pid_string = 0;
  for (c = 1; argc > c; ++c) {
    if (*argv[c] == '-') {
      if (argv[c][2] != '\0') printusage();
      switch (argv[c][1]) {
        case 's': mode_time = SECONDS; break;
        case 'm': mode_time = MINUTES; break;
        case 'H': mode_time = HOURS; break;
        case 'D': mode_time = DAYS; break;
        default: printusage();
      }
    } else {
      pid_string = argv[c];
      break;
    }
  }
  if (!pid_string) printusage();

  /* calculate HZ value */
  p = buf;
  rf = open("/proc/self/stat", O_RDONLY);
  if (rf == -1) _exit(EX_OSFILE);
  c = read(rf, p, BUFSIZE);
  close(rf);
  if (c == -1) _exit(EX_OSFILE);
  p[c] = '\0';
  /* starttime */
  str2uint64(field(p, 21), &val64); /* no error checks */
  start_time = val64;
  process_age = start_time;
  rf = open("/proc/uptime", O_RDONLY);
  if (rf == -1) _exit(EX_OSFILE);
  c = read(rf, p, BUFSIZE);
  close(rf);
  if (c == -1) _exit(EX_OSFILE);
  p[c] = '\0';
  /* uptime */
  str2uint64(p, &val64); /* no error checks */
  jiffies_per_second = ((int)(start_time / val64)) + 1;
  /**
   * 10 S/390 (sometimes)
   * 20 User-mode Linux
   * 32 ia64 emulator
   * 64 StrongARM /Shark
   * 100 normal Linux
   * 128 MIPS, ARM
   * 1000 ARM
   * 1024 Alpha, ia64
   * 1200 Alpha
   */
  jiffies_per_second = ((int)(jiffies_per_second / 2)) * 2; /* HZ */

  p = stpcpy(name, "/proc/");
  p = stpcpy(p, pid_string);
  stpcpy(p, "/stat");

  rf = open(name, O_RDONLY);
  if (rf >= 0) {
    p = buf;
    c = read(rf, p, BUFSIZE);
    close(rf);
    if (c == -1) goto err;
    p[c] = '\0';
    s = field(p, 21);
    if (s) {
      if(!str2uint64(s, &val64)) val64 = start_time;
      s = p;
    } else {
      val64 = start_time;
    }
    if (val64 > start_time) val64 = start_time;
    if (mode_time == SECONDS) {
      process_age = (start_time - val64) / jiffies_per_second;
    } else {
      if (mode_time == MINUTES) {
        process_age = (start_time - val64) / jiffies_per_second / 60;
      } else {
        if (mode_time == HOURS) {
          process_age = (start_time - val64) / jiffies_per_second / 3600;
        } else {
          process_age = (start_time - val64) / jiffies_per_second / 3600 / 24;
        }
      }
    }
  }

  p = sputuint64(buf, process_age);
  *p = '\n'; ++p;
  write1(buf, p-buf);
  _exit(0);

  err:
  _exit(1);

}

