/**
 * process_perfdata.c
 * $Id: process_perfdata.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#ifndef _FILE_OFFSET_BITS
#define _FILE_OFFSET_BITS 64
#endif
#include <features.h>

#include <stdint.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <errno.h>
#include <limits.h>
#include <sys/wait.h>

#include <string.h>

#include "../lib/libfwb.h"

/*
#include <stdio.h>
#define PERFDATA_SPOOL_DIR "/var/spool/nagios3/perfdata"
*/

#define PERFDATA_RRD_DIR "/usr/share/nagios/perfdata"

#define PIPEBUF_SIZE 4096

typedef struct struct_perfdataentry {
  int type;
  time_t ts;
  char *hostname;
  char *service;
  char *perfdata;
  uint32_t value;
} PDE;

static const char usage[] =
  "usage: process_perfdata [-D perfdir] -b perfdata_file\n"
  "example: process_perfdata -D /usr/share/nagios/perfdata -b /var/spool/nagios3/perfdata/service-perfdata.1282466267\n"
  "example: process_perfdata -D /usr/share/nagios/perfdata -b /var/spool/nagios3/perfdata/service-perfdata.1282466267\n"
  "example: dirscan -m -d /var/spool/nagios3/perfdata/ -I perfdata process_perfdata -D /usr/share/nagios/perfdata -b\n"
  "example: dirscan -m -d /var/spool/nagios3/perfdata/ -D /usr/share/nagios/perfdata -I perfdata process_perfdata -b\n";

enum { NULLDATA, HOSTPERFDATA, SERVICEPERFDATA };

char rrdtool_path[PATH_MAX+1];
char rrdtool_string[PATH_MAX+1];
static char * const rrd_command[] = {
  "/usr/local/bin/rrdupdate",
  rrdtool_path,
  rrdtool_string,
  0
};

static pid_t child;

static void printusage() { write1(usage, sizeof usage - 1); exit(1); }

const char *map_spool_file(const char *spoolfile, const char **spoolbuf, size_t *spoollen) {
  int fd;
  int sv_errno;
  struct stat stbuf;

  fd = open(spoolfile, O_RDONLY);
  if (fd < 0) goto error2;
  if (fstat(fd, &stbuf) == -1) goto error1;
  *spoollen = stbuf.st_size;

  *spoolbuf = mmap((void *) 0, *spoollen, PROT_READ, MAP_PRIVATE, fd, 0);
  if (*spoolbuf == (char *) -1) goto error1;

  return *spoolbuf;

  error1:
  sv_errno = errno;
  close(fd);
  errno = sv_errno;

  error2:
  return (char *) -1;
}


int main(int argc,char *const argv[]) {
  register char *p;
  register char *r;
  const char *spoolbuf;
  size_t spoollen;
  const char *spoolfile;
  int32_t tmpint;
  int retval = 0;
  int datatype;
  time_t timet;
  char hostname[PATH_MAX+1];
  char servicedesc[PATH_MAX+1];
  uint64_t serviceperfdata_value;
  uint64_t val;
  struct stat statbuf;
  char *changedir;
/*
  int fd[2];
  int status_child;
  char pipebuf[2*PIPEBUF_SIZE+1];
  ssize_t i;
*/
/*
  char serviceperfdata[PATH_MAX+1];
  char *servicecheckcommand;
  char *hoststate;
  char *hoststatetype;
  char *servicestate;
  char *servicestatetype;
*/

  changedir = 0;
  spoolfile = "";
  if (argc < 2) printusage();
  tmpint = 0;
  for(tmpint=1; tmpint<argc; ++tmpint) {
    if (*argv[tmpint] == '-') {
      if (argv[tmpint][2] != '\0') printusage();
      switch (argv[tmpint][1]) {
        case 'h': printusage(); break;
        case 'b': ++tmpint; spoolfile=argv[tmpint]; break;
        case 'D': changedir = argv[++tmpint]; break;
        default: break;
      }
    } else {
      printusage();
    }
  }

/*
#include <stdio.h>
printf("spoolfile: %s\n", spoolfile);
printf("(size_t) -1: %u\n", (size_t) -1);
printf("(size_t) -1 mod 4096: %u\n", ((size_t) -1) % 4096);
printf("(size_t) -1 & 0x7FF: %u\n", ((size_t) -1) & 0x7FF);
*/

  p = (char *) map_spool_file(spoolfile, &spoolbuf, &spoollen);
  if (p == (char *) -1) {
    if (lstat(spoolfile, &statbuf) != 0) {
      if (statbuf.st_size == 0) goto error1;
    }
    goto error3;
  }
/*
  if (spoollen == 0) goto unmap_and_exit;
*/

  if (changedir && (chdir(changedir) != 0)) goto error2;

/*
printf("spoollen: %u\n", spoollen);
  alloclen = (size_t) ((uint32_t) ((spoollen / 4096) + 1) * 4096);
  if (alloclen < spoollen) goto error1;
printf("alloclen: %u\n", alloclen);
*/

/*
  allocbuf = malloc(alloclen);
*/

  /* step through all lines */
  for (;;) {


    while (*p == '\n') ++p;
    if (!*p) break;

    *hostname = 0;
    *servicedesc = 0;
    timet = 0;
    serviceperfdata_value = (uint64_t) -1;
    datatype = NULLDATA;

    /* step through one line */
    for (;;) {

      while (*p == '\t') ++p;

      tmpint = strncmp(p, "DATATYPE::", 10);
      if (tmpint) goto ck_hostname;
      p += 10;
      tmpint = strncmp(p, "SERVICEPERFDATA\t", 16);
      if (!tmpint) datatype = SERVICEPERFDATA;
      else datatype = HOSTPERFDATA;
      goto read_til_endofkey;

      ck_hostname:
      tmpint = strncmp(p, "HOSTNAME::", 10);
      if (tmpint) goto ck_servicedesc;
      p += 10;
      tmpint = 0;
      for (;(*p != '\t') && (*p != '\n');) { hostname[tmpint] = *p; ++tmpint; ++p; }
      hostname[tmpint] = 0;
      goto read_til_endofkey;

      ck_servicedesc:
      tmpint = strncmp(p, "SERVICEDESC::", 13);
      if (tmpint) goto ck_perfdata;
      p += 13;
      tmpint = 0;
      for (;(*p != '\t') && (*p != '\n');) { servicedesc[tmpint] = *p; ++tmpint; ++p; }
      servicedesc[tmpint] = 0;
      goto read_til_endofkey;

      ck_perfdata:
      tmpint = strncmp(p, "SERVICEPERFDATA::", 17);
      if (tmpint) goto ck_timet;
      p += 17;
      for (;;) {
        if (*p == '=') break;
        if ((*p == '\t') || (*p == '\n')) goto read_til_endofkey;
        ++p;
      }
      ++p;
      str2uint64(p, &serviceperfdata_value);
      goto read_til_endofkey;

      ck_timet:
      tmpint = strncmp(p, "TIMET::", 7);
      if (tmpint) goto read_til_endofkey;
      p += 7;
      if (!str2uint64(p, &val)) goto read_til_endofkey;
      timet = (time_t) val;
  
      read_til_endofkey:
      if (*p == '\t') continue;
      while (*p && (*p != '\t') && (*p != '\n')) ++p;
      
      if (!*p || (*p == '\n')) break;

    }

/*
if (*hostname) printf("hostname: %s ", hostname);
if (*servicedesc) printf("servicedesc: %s ", servicedesc);
if (datatype != NULLDATA) printf("datatype=%d ", datatype);
if (timet) printf("timet=%d ", timet);
if (serviceperfdata_value != (uint64_t) -1) printf("serviceperfdata_value=%lu ", serviceperfdata_value);
write1("\n",1);
    if (*hostname && *servicedesc && timet && (serviceperfdata_value != (uint64_t) -1)) {
printf("hostname: %s, servicedesc: %s, timet=%u, serviceperfdata_value=%u\n", hostname, servicedesc, timet, serviceperfdata_value);
    }
*/

    if (!*hostname) goto next_line;
    if (!timet) goto next_line;
    if (serviceperfdata_value == (uint64_t) -1) goto next_line;
    if (datatype == NULLDATA) goto next_line;
    if (datatype == HOSTPERFDATA) stpcpy(servicedesc, "_HOST_");
    if (!*servicedesc) goto next_line;

    r = stpcpy(rrdtool_path, hostname);
    *r = '/'; ++r;
    r = stpcpy(r, servicedesc);
    stpcpy(r, ".rrd");

    r = sputuint64(rrdtool_string, timet);
    *r = ':'; ++r;
    sputuint64(r, serviceperfdata_value);

    /* before doing expensive fork() check for existing file */
    if (stat(rrdtool_path, &statbuf)) goto next_line;

    /* last mod have to be made > 4 min before */
    if ((timet - statbuf.st_mtime) < 240) goto next_line;

    /* simple popen */
/*
printf("%s %s %s %s\n", rrd_command[0], rrd_command[1], rrd_command[2], rrd_command[3]);
    if (pipe(fd) < 0) goto next_line;
    if ((child = fork()) < 0) {
      close(fd[0]); close(fd[1]);
      goto next_line;
    }
    if (!child) {
      close(fd[0]); close(0);
      dup2(fd[1],1); dup2(fd[1],2); close(fd[1]); 
      execve(rrd_command[0], rrd_command, environ);
      write1("exec err\n",9);
      close(1);
      _exit(111);
    }
    close(fd[1]);
    r = pipebuf;
    i = read(fd[0], r, PIPEBUF_SIZE);
    if (i > 0) { 
      r[i] = 0;
      r += PIPEBUF_SIZE;
      while (i > 0) {
        i = read(fd[0], r, PIPEBUF_SIZE);
      }
    } else {
      stpcpy(r, "no data from extcmd\n");
    }
    close(fd[0]);
    waitpid(child, &status_child, 0);
    if (!WIFEXITED(status_child)) {
      kill(child, SIGKILL);
    }

*/

    child = vfork();
    if (child < 0) goto next_line;
    if (!child) {
      execve(rrd_command[0], rrd_command, environ);
      write1("exec err\n",9);
      _exit(111);
    }

    next_line:
    while (*p && (*p != '\n')) ++p;

  }

  munmap((void *)spoolbuf, spoollen);
  unlink(spoolfile);
  return 0;

  error3:
  if (!retval) retval = 3;
  unlink(spoolfile);
  goto error1;

  error2:
  if (!retval) retval = 2;
  munmap((void *)spoolbuf, spoollen);

  error1:
  if (!retval) retval = 1;

  return retval;
}

/*

write(4, "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n<NAGIOS>\n  <DATASOURCE>\n    <TEMPLATE>check_remote_generic_load15</TEMPLATE>\n    <IS_MULTI>0</IS_MULTI>\n    <DS>1</DS>\n    <NAME>load15</NAME>\n    <UNIT></UNIT>\n    <ACT>0.00</ACT>\n    <WARN>15</WARN>\n    <WARN_MIN></WARN_MIN>\n    <WARN_MAX></WARN_MAX>\n    <WARN_RANGE_TYPE></WARN_RANGE_TYPE>\n    <CRIT>45</CRIT>\n    <CRIT_MIN></CRIT_MIN>\n    <CRIT_MAX></CRIT_MAX>\n    <CRIT_RANGE_TYPE></CRIT_RANGE_TYPE>\n    <MIN></MIN>\n    <MAX></MAX>\n  </DATASOURCE>\n  <RRD>\n    <RC>1</RC>\n    <TXT>/usr/share/nagios/perfdata/db3089/load15.rrd: illegal attempt to update using time 1282646285 when last update time is 1282646285 (minimum one second step)</TXT>\n  </RRD>\n  <NAGIOS_CHECK_COMMAND>check_remote_generic!15!45!load15</NAGIOS_CHECK_COMMAND>\n  <NAGIOS_DATATYPE>SERVICEPERFDATA</NAGIOS_DATATYPE>\n  <NAGIOS_HOSTNAME>db3089</NAGIOS_HOSTNAME>\n  <NAGIOS_HOSTSTATE>UP</NAGIOS_HOSTSTATE>\n  <NAGIOS_HOSTSTATETYPE>SOFT</NAGIOS_HOSTSTATETYPE>\n  <NAGIOS_MULTI_PARENT></NAGIOS_MULTI_PA"..., 1616) = 1616


DATATYPE::SERVICEPERFDATA	TIMET::1282466266	HOSTNAME::db1007	SERVICEDESC::mysql	SERVICEPERFDATA::	SERVICECHECKCOMMAND::check_mysql_database!nagios!!nagios_db	HOSTSTATE::UP	HOSTSTATETYPE::HARD	SERVICESTATE::OK	SERVICESTATETYPE::HARD
DATATYPE::SERVICEPERFDATA	TIMET::1282466266	HOSTNAME::s293	SERVICEDESC::load15	SERVICEPERFDATA::load15=0.10;15;45	SERVICECHECKCOMMAND::check_remote_generic!15!45!load15	HOSTSTATE::UP	HOSTSTATETYPE::HARD	SERVICESTATE::OK	SERVICESTATETYPE::HARD
DATATYPE::SERVICEPERFDATA	TIMET::1282466266	HOSTNAME::s1032	SERVICEDESC::cpusystem	SERVICEPERFDATA::cat/nagios-cache/cpusc/55=1%;99;101	SERVICECHECKCOMMAND::check_remote_generic_w_unit!99!101!cat/nagios-cache/cpusc/55!%	HOSTSTATE::UP	HOSTSTATETYPE::HARD	SERVICESTATE::OK	SERVICESTATETYPE::HARD
DATATYPE::SERVICEPERFDATA	TIMET::1282466266	HOSTNAME::s296	SERVICEDESC::load15	SERVICEPERFDATA::load15=0.09;15;45	SERVICECHECKCOMMAND::check_remote_generic!15!45!load15	HOSTSTATE::UP	HOSTSTATETYPE::HARD	SERVICESTATE::OK	SERVICESTATETYPE::HARD
DATATYPE::SERVICEPERFDATA	TIMET::1282466266	HOSTNAME::s1027	SERVICEDESC::load15	SERVICEPERFDATA::load15=0.16;15;45	SERVICECHECKCOMMAND::check_remote_generic!15!45!load15	HOSTSTATE::UP	HOSTSTATETYPE::HARD	SERVICESTATE::OK	SERVICESTATETYPE::HARD
DATATYPE::SERVICEPERFDATA	TIMET::1282466266	HOSTNAME::db1001	SERVICEDESC::time	SERVICEPERFDATA::timediff=0	SERVICECHECKCOMMAND::check_remote_time!20!300	HOSTSTATE::UP	HOSTSTATETYPE::HARD	SERVICESTATE::OK	SERVICESTATETYPE::HARD
DATATYPE::SERVICEPERFDATA	TIMET::1282466266	HOSTNAME::s285	SERVICEDESC::cpusystem	SERVICEPERFDATA::cat/nagios-cache/cpusc/55=4%;99;101	SERVICECHECKCOMMAND::check_remote_generic_w_unit!99!101!cat/nagios-cache/cpusc/55!%	HOSTSTATE::UP	HOSTSTATETYPE::HARD	SERVICESTATE::OK	SERVICESTATETYPE::HARD
DATATYPE::SERVICEPERFDATA	TIMET::1282466266	HOSTNAME::s1019	SERVICEDESC::load15	SERVICEPERFDATA::load15=0.20;15;45	SERVICECHECKCOMMAND::check_remote_generic!15!45!load15	HOSTSTATE::UP	HOSTSTATETYPE::HARD	SERVICESTATE::OK	SERVICESTATETYPE::HARD
DATATYPE::SERVICEPERFDATA	TIMET::1282466266	HOSTNAME::invoices1	SERVICEDESC::cpuwait	SERVICEPERFDATA::cat/nagios-cache/cpuwc/55=0%;97;100	SERVICECHECKCOMMAND::check_remote_generic_w_unit!97!100!cat/nagios-cache/cpuwc/55!%	HOSTSTATE::UP	HOSTSTATETYPE::HARD	SERVICESTATE::OK	SERVICESTATETYPE::HARD
DATATYPE::SERVICEPERFDATA	TIMET::1282466266	HOSTNAME::s282	SERVICEDESC::time	SERVICEPERFDATA::timediff=0	SERVICECHECKCOMMAND::check_remote_time!20!300	HOSTSTATE::UP	HOSTSTATETYPE::HARD	SERVICESTATE::OK	SERVICESTATETYPE::HARD

DATATYPE::SERVICEPERFDATA       TIMET::1282489656       HOSTNAME::db3026        SERVICEDESC::dfroot     SERVICEPERFDATA::diskusage/=41%;90;97 SERVICECHECKCOMMAND::check_df!90!97!/   HOSTSTATE::UP   HOSTSTATETYPE::HARD     SERVICESTATE::OK      SERVICESTATETYPE::HARD

/usr/share/nagios/perfdata/db3084/dfroot.rrd updated, /usr/share/nagios/perfdata/db3084/dfroot.rrd 1282489256:36

<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<NAGIOS>
  <DATASOURCE>
    <TEMPLATE>check_df_/</TEMPLATE>
    <IS_MULTI>0</IS_MULTI>
    <DS>1</DS>
    <NAME>diskusage_</NAME>
    <UNIT>%%</UNIT>
    <ACT>36</ACT>
    <WARN>90</WARN>
    <WARN_MIN></WARN_MIN>
    <WARN_MAX></WARN_MAX>
    <WARN_RANGE_TYPE></WARN_RANGE_TYPE>
    <CRIT>97</CRIT>
    <CRIT_MIN></CRIT_MIN>
    <CRIT_MAX></CRIT_MAX>
    <CRIT_RANGE_TYPE></CRIT_RANGE_TYPE>
    <MIN></MIN>
    <MAX></MAX>
  </DATASOURCE>
  <RRD>
    <RC>0</RC>
    <TXT>successful updated</TXT>
  </RRD>
  <NAGIOS_CHECK_COMMAND>check_df!90!97!/</NAGIOS_CHECK_COMMAND>
  <NAGIOS_DATATYPE>SERVICEPERFDATA</NAGIOS_DATATYPE>
  <NAGIOS_HOSTNAME>db3084</NAGIOS_HOSTNAME>
  <NAGIOS_HOSTSTATE>UP</NAGIOS_HOSTSTATE>
  <NAGIOS_HOSTSTATETYPE>HARD</NAGIOS_HOSTSTATETYPE>
  <NAGIOS_MULTI_PARENT></NAGIOS_MULTI_PARENT>
  <NAGIOS_PERFDATA>diskusage/=36%;90;97 </NAGIOS_PERFDATA>
  <NAGIOS_RRDFILE>/usr/share/nagios/perfdata/db3084/dfroot.rrd</NAGIOS_RRDFILE>
  <NAGIOS_SERVICECHECKCOMMAND>check_df!90!97!/</NAGIOS_SERVICECHECKCOMMAND>
  <NAGIOS_SERVICEDESC>dfroot</NAGIOS_SERVICEDESC>
  <NAGIOS_SERVICEPERFDATA>diskusage/=36%;90;97</NAGIOS_SERVICEPERFDATA>
  <NAGIOS_SERVICESTATE>OK</NAGIOS_SERVICESTATE>
  <NAGIOS_SERVICESTATETYPE>HARD</NAGIOS_SERVICESTATETYPE>
  <NAGIOS_TIMET>1282489256</NAGIOS_TIMET>
  <NAGIOS_XMLFILE>/usr/share/nagios/perfdata/db3084/dfroot.xml</NAGIOS_XMLFILE>
</NAGIOS>

*/
