/*
 * scanjavascriptevals.c
 * $Id: scanjavascriptevals.c 691 2017-06-04 23:12:04Z fwbergm $
 *
 * 1)
 * Scan all files <2MB ending in .html, .htm, .php and .js for strings
 *    '<script'.*'eval'
 *
 * Frank W. Bergmann 08/2010
 *
 */

#define _FILE_OFFSET_BITS 64

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <ftw.h>

#include "lib/buffer.h"
#include "lib/libfwb.h"

/*
#define LARGEBUF_SIZE 4096
*/
#define WRITEBUFSIZE 2048
char wbufspace[WRITEBUFSIZE+1];
buffer wbuf;

static const char *fnamesuffixes[] = {
  ".php",
  ".js",
  ".html",
  ".htm",
  0
};

/* static const char searchstring1[] = "<script"; */
/* static const char searchstring2[] = "gzinflate(base64_decode"; */
/* static const char searchstring2[] = "http://";*/

/**
 *
 */
int scan_file(const char *fpath, const struct stat* sb, int typeflag) {
  register const char *p;
  const char *linestart;
  const char *lineend;
  const char *t;
  int f;
  /* char rbuf[LARGEBUF_SIZE+1]; */
  const char *rbuf;
  off_t rlen;


  /* scan only regular files */
  if (typeflag != FTW_F) return 0;


  /* don't scan too small or too big files */
  if ((sb->st_size < 16) ||
      (sb->st_size > 2097152)) return 0;


  /* is filename suspicious? */
  f = 0;
  /* p = (char *)strrchr(fpath, '/'); */
  p = (char *)strrchr(fpath, '.');
  if (!p) return 0;
  for(;;) {
    if(!fnamesuffixes[f]) break;
    if(!strcmp(p, fnamesuffixes[f])) goto suffixmatched;
    ++f;
  }
  return 0;

  suffixmatched:

  /* scan content of file */
  p = (char *) map_file(fpath, &rbuf, &rlen);
  if (p == (char *) -1) { write2(fpath, strlen(fpath)); write2(" err\n", 5); return 0; }

  p = rbuf;

  next_lighter_char:

  if (!*p) goto finished_scan;
  p = strchr(p, '<');
  if (!p) goto finished_scan;
  linestart = p;
  ++p;
  if ((*p != 's') && (*p != 'S')) goto next_lighter_char;
  ++p;
  if ((*p != 'c') && (*p != 'C')) goto next_lighter_char;
  ++p;
  if ((*p != 'r') && (*p != 'R')) goto next_lighter_char;
  ++p;
  if ((*p != 'i') && (*p != 'I')) goto next_lighter_char;
  ++p;
  if ((*p != 'p') && (*p != 'P')) goto next_lighter_char;
  ++p;
  if ((*p != 't') && (*p != 't')) goto next_lighter_char;
  ++p;
  if ((*p != ' ') && (*p != '>') && (*p != '\t')) goto next_lighter_char;

  t = p; /* search for eol */
  for (;;) {
    ++p; if (!*p) break; if (*p == '\n') break;
    ++p; if (!*p) break; if (*p == '\n') break;
    ++p; if (!*p) break; if (*p == '\n') break;
    ++p; if (!*p) break; if (*p == '\n') break;
  }
  lineend = p;
  p = t;

  next_e_char:

  if (!*p) goto finished_scan;
  if (p == lineend) goto finished_scan;
  p = memchr(p, 'e', lineend - p);
  if (!p) { p = t; goto next_lighter_char; }
  ++p;
  if (*p != 'v') goto next_e_char;
  ++p;
  if (*p != 'a') goto next_e_char;
  ++p;
  if (*p != 'l') goto next_e_char;

  buffer_put(&wbuf, fpath, strlen(fpath)); buffer_putc(&wbuf, ' ');
  buffer_put(&wbuf, linestart, lineend - linestart); buffer_putc(&wbuf, '\n');
  p = lineend;
  goto next_lighter_char;

  finished_scan:
  munmap((void *) rbuf, rlen);

  return 0;

}


int main() {
  register char *p;
  int ret;

  p = wbufspace;
  wbuf.s = p;
  wbuf.c = p;
  wbuf.t = p + WRITEBUFSIZE;
  wbuf.fd = 1;
  ret = ftw(".", &scan_file, 64);
  buffer_flush(&wbuf);
  _exit(ret);
}

