/*
 * showip.c
 * log dialin hosts IP# by calling an
 * internet connected HTTP-server cgi-script
 * and print the returned IP#
 * (used as dyndns service on tuxad.net)
 *
 * Used for:
 * - getting my own dynamic or NAT IP address with a simple
 *   http-get, most compatible with firewalls
 * (- log it (IP# & name) for remote access or for statistics)
 * (- register my name at tuxad.net dyndns service)
 *
 * compile with i.e.
 *   diet -Os gcc -Os -Wall -o showip showip.c;strip showip
 *
 * Frank Bergmann, 1/2006
 *
 * $Id: showip.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#ifndef write1
#include <sys/syscall.h>
#define write1(b,c) (syscall(__NR_write, 1, b, c))
#endif

/*
 * we call the server on IP# STR_SERVER_IP on tcp port 80
 * and send the following HTTP-command:
 *    GET /showip HTTP/1.1
 *    Host: dyndns.tuxad.net
 * The IP# of our client my-private-webserver-at-home will be
 * created on the server by the following CGI-script logip:
 *    #!/bin/sh
 *    echo "Content-Type: text/plain
 *    "
 *    echo $REMOTE_ADDR
 * After sending the HTTP command this program will read the
 * output from the HTTP-server (output of the script logip), strip
 * HTTP-header and bogus stuff and print the IP# of the client
 * it runs on to stdout.
 */

#define READBYTES 128

#define STR_SERVER_IP "81.89.239.232"

static const char get_string[] =
  "GET /showip HTTP/1.1\r\n"
  "User-Agent: showip/1.0 (http://www.tuxad.com)\r\n"
  "Host: dns.tdyn.de\r\n\r\n";

int main() {
  register char *p;
  ssize_t buflen;
  int sockfd;
  char http_buf[2*READBYTES];
  char *read_end;
  struct sockaddr_in server_sockaddr;

  if (inet_aton(STR_SERVER_IP, &server_sockaddr.sin_addr) == 0) goto err_exit1;
  server_sockaddr.sin_family = AF_INET;
  server_sockaddr.sin_port = htons(80);
  if ((sockfd = socket(PF_INET, SOCK_STREAM, 0)) == -1) goto err_exit1;
  if (connect(sockfd, (struct sockaddr *) &server_sockaddr, sizeof server_sockaddr) == -1) goto err_exit2;
  if (write(sockfd, get_string, sizeof get_string - 1) == -1) goto err_exit2;
  p = http_buf;
  if ((buflen = read(sockfd, p, READBYTES)) == -1) goto err_exit2;
  p += buflen;
  if (buflen < 70) {
    /* maybe we got only the header, try another read */
    buflen = read(sockfd, p, READBYTES);
    if (buflen > 0) p += buflen;
  }
  close(sockfd);
  *p = 0;
  for (;;--p) if ((*p < 32) && (*(p-1) > 32)) break;
  *p = '\n'; read_end = p + 1;
  for(;;) { --p; if (*p < 32) break; }
  ++p;
  write1(p, read_end - p);
  _exit(0);

  err_exit2:
  close(sockfd);

  err_exit1:
  _exit(1);
}
