/**
 * svscanboot3.c
 * like DJBs svscanboot.sh but as binary executable and without readproctitle pipe
 * $Id: svscanboot3.c 691 2017-06-04 23:12:04Z fwbergm $
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <errno.h>
#include <dirent.h>

#define SERVICES 1000
#define SERVICEDIR "/service/"
#define LOGDIR "/log"


/* simple system() replacement with no signal handling */
static int my_system(char **call) {
  pid_t callpid;
  int childstatus;
  
  if (!call) return(1);
  callpid = fork();
  if (callpid == -1) return(-1);
  if (callpid == 0) {
    /* child */
    execve(*call, call, 0);
    _exit(127);
  } else {
    /* parent */
    while (waitpid(callpid, &childstatus, 0) < 0) {
      if (errno != EINTR) return(-1);
    }
  }
  return(childstatus);
}


int main() {
  register char *p;
  int i;
  DIR *dptr;
  struct dirent *eptr;
  struct stat logstat;
  static const char newpath[] = "/command:/usr/local/bin:/usr/local/sbin:/bin:/sbin:/usr/bin:/usr/sbin:/usr/X11R6/bin";
  static char *const svscan[] = { "/command/svscan", SERVICEDIR, 0 };
  char envpath[sizeof(newpath)+6];
  char *newenv[2];
  char *svc_call[(2*SERVICES)+3];

  i = open("/dev/null", O_RDWR);
  if(i < 0) goto error_out;
  if(dup2(i,0) < 0) goto error_out;
  if(dup2(i,1) < 0) goto error_out;
  if(dup2(i,2) < 0) goto error_out;
  close(i);
  
  /*  /command/svc -dx /service/?* /service/?*?/log  */
  dptr = opendir("/service/");
  if(!dptr) _exit(1);

  for (i=2; (eptr=readdir(dptr));) {
    if (*(eptr->d_name) == '.') continue;
    p = alloca(sizeof(eptr->d_name)+sizeof(SERVICEDIR)+sizeof(LOGDIR));
    if (!p) { closedir(dptr); _exit(2); }
    strcpy(p, SERVICEDIR);
    strcat(p, eptr->d_name);
    strcat(p, "/log");
    if (!stat(p, &logstat)) {
      svc_call[++i] = p;
      if (i >= (2*SERVICES)) { closedir(dptr); _exit(3); }
      p = alloca(sizeof(eptr->d_name)+sizeof(SERVICEDIR));
      if (!p) { closedir(dptr); _exit(2); }
      strcpy(p, SERVICEDIR);
      strcat(p, eptr->d_name);
      svc_call[--i] = p;
      i += 2;
      if (i >= (2*SERVICES)) { closedir(dptr); _exit(3); }
    } else {
      strcpy(p, SERVICEDIR);
      strcat(p, eptr->d_name);
      svc_call[i++] = p;
      if (i >= (2*SERVICES)) { closedir(dptr); _exit(3); }
    }
  }
  closedir(dptr);
  svc_call[i] = 0;

  strcpy(envpath ,"PATH=");
  strcat(envpath, newpath);
  newenv[0] = envpath;
  newenv[1] = 0;

  svc_call[0] = "/command/svc";
  svc_call[1] = "-dx";

  my_system(svc_call);

  /*
env - PATH=$PATH svscan /service >/dev/null 2>&1
  */
  execve(*svscan, svscan, newenv);

error_out:
  sleep(5); /* don't allow respawning too fast */
  _exit(111);
}

